/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.applications;

import examples.invoice.persistclass.Address;
import examples.invoice.persistclass.AddressHelper;
import examples.invoice.persistclass.MapHelper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PName;

import java.util.Iterator;
import java.io.InputStream;
import javax.resource.ResourceException;

/**
 * @author	P. Dechamboux
 */
public class AddressAppli implements ShellExtent {
	private static AddressAppli singleton = null;
	private AddressHelper mapHelper;
	private BasicShell basicShell;

	public static AddressAppli getInstance() {
		if (singleton == null) {
			singleton = new AddressAppli();
		}
		return singleton;
	}

	private AddressAppli() {
		mapHelper = new AddressHelper();
		basicShell = BasicShell.getInstance();
	}

	public boolean interpCmd(String cmd) {
		if (cmd.equals("ca") || cmd.equals("createaddress"))
			createAddress();
		else if (cmd.equals("la") || cmd.equals("listaddress"))
			listAddress();
		else if (cmd.equals("da") || cmd.equals("deleteaddress"))
			deleteAddress();
		else
			return false;
		return true;
	}

	public void help() {
		System.out.println("\t\t- createaddress | ca : creates a new Address using the active mapper.");
		System.out.println("\t\t- listaddress | la : lists existing Addresses using the active mapper.");
		System.out.println("\t\t- deleteaddress | da : deletes an existing Address using the active mapper.");
	}

	public MapHelper getHelper() {
		return mapHelper;
	}

	public static void main(String[] args) {
		InputStream in = System.in;
		try {
			switch (args.length) {
			case 2:
				in = ObsInputStream.observes(args[1]);
			case 1:
				BasicShell.getInstance().shellLoop(args[0], getInstance(), in);
				break;
			default:
				System.err.println("Usage: AddressAppli prop_file_name [jormexsh_cmdfile]");
				System.exit(-1);
			}
		} catch (Exception e) {
			System.err.println("Exit with exception: " + e.getMessage());
			do {
				e.printStackTrace();
				if (e instanceof PException)
					e = ((PException) e).getNestedException();
				if (e instanceof ResourceException)
					e = ((ResourceException) e).getLinkedException();
				else
					e = null;
			} while (e != null);
		}
	}

	private void createAddress() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		try {
			String n, s, z, t, c;
			n = basicShell.promptString("\tName: ", 50);
			s = basicShell.promptString("\tStreet: ", 50);
			z = basicShell.promptString("\tZip code: ", 50);
			t = basicShell.promptString("\tTown: ", 50);
			c = basicShell.promptString("\tCountry: ", 50);
			if ((n == null) || (s == null) || (n.length() == 0) || (s.length() == 0)) {
				basicShell.errorMsg("wrong name or street (mandatory)");
				return;
			}
			if (z.length() == 0)
				z = null;
			if (t.length() == 0)
				t = null;
			if (c.length() == 0)
				c = null;
			Address ad = mapHelper.createAddress(basicShell.activeMapper,
												 n, s, z, t, c);
			System.out.println("\tCreated Address:\n\t\t" + ad.toPrintable());
		} catch (Exception e) {
            System.err.println("EXCEPTION: " + e.getMessage());
            do {
                e.printStackTrace();
                if (e instanceof PException)
                    e = ((PException) e).getNestedException();
                if (e instanceof ResourceException)
                    e = ((ResourceException) e).getLinkedException();
                else
                    e = null;
            } while (e != null);
			System.out.println();
		}
	}

	private void listAddress() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		try {
			Iterator adit = mapHelper.listAddress(basicShell.activeMapper);
			System.out.println("\tExisting Addresses:");
			while (adit.hasNext()) {
				PName pn = (PName) adit.next();
				Address ad = mapHelper.getAddress(basicShell.activeMapper, pn);
				System.out.println("\t\t" + ad.toPrintable());
			}
		} catch (Exception e) {
            System.err.println("EXCEPTION: " + e.getMessage());
            do {
                e.printStackTrace();
                if (e instanceof PException)
                    e = ((PException) e).getNestedException();
                if (e instanceof ResourceException)
                    e = ((ResourceException) e).getLinkedException();
                else
                    e = null;
            } while (e != null);
			System.out.println();
		}
	}

	private void deleteAddress() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		String sid = basicShell.promptString("\tID: ", 256);
		try {
			if (mapHelper.deleteAddress(basicShell.activeMapper, sid))
				System.out.println("Address deleted.");
			else
				basicShell.errorMsg("cannot delete Address");
		} catch (Exception e) {
            System.err.println("EXCEPTION: " + e.getMessage());
            do {
                e.printStackTrace();
                if (e instanceof PException)
                    e = ((PException) e).getNestedException();
                if (e instanceof ResourceException)
                    e = ((ResourceException) e).getLinkedException();
                else
                    e = null;
            } while (e != null);
			System.out.println();
		}
	}
}
