/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.applications;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * @author	P. Dechamboux
 */
public final class BasicShell {
	private final static String PROMPT = "jormexsh";
	private final static int MAXCMDSIZE = 20;
	private static BasicShell singleton = new BasicShell();
	private ShellExtent shellExtent;
	protected InputStream in;
	PMapper activeMapper = null;

	// BasicShell SINGLETON INITIALIZATION

	static BasicShell getInstance() {
		return singleton;
	}

	private BasicShell() {
	}

	// PUBLIC METHODS FROM BasicShell

	public String promptString(String prompt, int maxsize) {
		byte[] cmd = new byte[maxsize];
		int nextbyte = 0, inbyte;
		try {
			System.out.print(prompt);
			System.out.flush();
			do {
				inbyte = in.read();
				if (nextbyte < maxsize)
					cmd[nextbyte] = (byte) inbyte;
				nextbyte++;
			} while ((inbyte != '\r') && (inbyte != '\n'));
			if (inbyte == '\r') {
			in.read();
			}
			if (nextbyte <= maxsize) {
				if (nextbyte != 1)
					return new String(cmd, 0, nextbyte - 1);
				else
					return "";
			} else
				return null;
		} catch (Exception e) {
			return null;
		}
	}

	public void shellLoop(String propfilename, ShellExtent se, InputStream in) throws Exception {
		Properties prop = new Properties();
		prop.load(new FileInputStream(propfilename));
		JormInit.getInstance().initLogSystem(prop);
		JormInit.getInstance().initResourceAdapter(prop);
		prop = null;
		this.in = in;
		shellExtent = se;
		String cmd;
		do {
			cmd = readCmd();
			System.out.println();
			if (cmd.equals("q") || cmd.equals("quit"))
				return;
			if (!interpCmd(cmd))
				errorMsg("unknown");
			System.out.println();
		} while (true);
	}

	public void errorMsg(String msg) {
		System.out.println("\tCommand failed: " + msg + ".\n");
	}

	public void printStringBlankApp(String str, int size) {
		System.out.print(str);
		if (str.length() >= size)
			return;
		int bl = size - str.length();
		while (bl > 0) {
			System.out.print(" ");
			bl--;
		}
	}

	// PRIVATE METHODS FROM BasicShell

	private String readCmd() {
		String cmd;
		do {
			cmd = promptString(
				PROMPT + ((activeMapper == null)
						  ? ""
						  : ":mapper=" + activeMapper.getMapperName()) + "> ",
				MAXCMDSIZE);
			if (cmd == null)
				errorMsg("wrong format");
			else if (cmd.length() > 0)
				return cmd;
		} while (true);
	}

	private boolean interpCmd(String cmd) throws Exception {
		if (shellExtent.interpCmd(cmd))
			return true;
		if (cmd.equals("lm") || cmd.equals("listmapper"))
			listMapper();
		else if (cmd.equals("am") || cmd.equals("activatemapper"))
			activateMapper();
		else if (cmd.equals("map"))
			mapPClass();
		else if (cmd.equals("?") || cmd.equals("h") || cmd.equals("help"))
			help();
		else
			return false;
		return true;
	}

	private void help() {
		System.out.println("\tAvailable commands:");
		System.out.println("\t\t- help | h | ? : this command. It provides help information about available commands of this application.");
		System.out.println("\t\t- listmapper | lm : lists all available mappers.");
		System.out.println("\t\t- activatemapper | am : activates a mapper as the default one used by commands that require a mapper. It does it for the active mapper if any, or requests this mapper.");
		System.out.println("\t\t- map : maps all the persistent classes involved in this application.");
		shellExtent.help();
	}

	/**
	 * Lists the mappers that have been created and registered with JormInit.
	 */
	private void listMapper() throws Exception {
		Iterator it = JormInit.getInstance().registeredMapper();
		PMapper mapper;
		if (!it.hasNext()) {
			System.out.println("\n\tNo mapper registered yet.\n");
			return;
		}
		System.out.println("\n\tMAPPER NAME      | URL");
		System.out.println("\t--------------------------------------------------");
		while (it.hasNext()) {
			mapper = (PMapper) it.next();
			System.out.print("\t");
			printStringBlankApp(mapper.getMapperName(), 16);
			System.out.print(" | ");
			printStringBlankApp(JormInit.getInstance().getURL(mapper), 10);
			System.out.println();
		}
	}

	private void mapPClass() throws PException {
		String name;
		PMapper m;
		if (activeMapper != null) {
			m = activeMapper;
		} else {
			name = promptString("\tMapper name: ", 16);
			if (name == null) {
				errorMsg("wrong input name for mapper");
				return;
			}
			m = JormInit.getInstance().getMapper(name);
			if (m == null) {
				errorMsg("no associated mapper");
				return;
			}
		}
		String rem = promptString("\tInit OP (REMS, REMD, CREATE, NONE): ", 6);
		byte cleanup_action;
		if ((rem == null) || (rem.length() == 0))
			cleanup_action = PClassMapping.CREATE_STRUCTURE_IF_NEEDED;			
		else if (rem.equals("REMS"))
			cleanup_action = PClassMapping.CLEANUP_REMOVEALL;
		else if (rem.equals("REMD"))
			cleanup_action = PClassMapping.CLEANUP_REMOVEDATA;
		else if (rem.equals("NONE"))
			cleanup_action = PClassMapping.CLEANUP_DONOTHING;
		else
			cleanup_action = PClassMapping.CREATE_STRUCTURE_IF_NEEDED;
		try {
			shellExtent.getHelper().map(m,
										JormInit.getInstance().getLoggerFactory(),
										cleanup_action);
			System.out.println("Persistent classes mapped.");
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}

	private void activateMapper() throws PException {
		String name = promptString("\tMapper name: ", 16);
		if (name == null) {
			errorMsg("wrong input names for mapper");
			return;
		}
		activeMapper = JormInit.getInstance().getMapper(name);
		if (activeMapper == null) {
			errorMsg("no associated mapper");
			return;
		}
		System.out.println("Mapper <" + name + "> activated.");
	}
}
