/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.applications;

import examples.invoice.persistclass.AddressHelper;
import examples.invoice.persistclass.Invoice;
import examples.invoice.persistclass.InvoiceHelper;
import examples.invoice.persistclass.MapHelper;
import examples.invoice.persistclass.ProductHelper;
import examples.invoice.persistclass.ProdUnitsHelper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PName;

import java.util.Iterator;
import java.io.InputStream;
import javax.resource.ResourceException;

/**
 * @author	P. Dechamboux
 */
public class InvoiceAppli implements ShellExtent {
	private static InvoiceAppli singleton =  null;
	private static AddressAppli addressAppli;
	private static ProductAppli productAppli;
	private static ProdUnitsAppli prodUnitsAppli;
	private AddressHelper addressHelper;
	private ProductHelper productHelper;
	private ProdUnitsHelper prodUnitsHelper;
	private InvoiceHelper mapHelper;
	private BasicShell basicShell;

	public static InvoiceAppli getInstance() {
		if (singleton == null) {
			addressAppli = AddressAppli.getInstance();
			productAppli = ProductAppli.getInstance();
			prodUnitsAppli = ProdUnitsAppli.getInstance();
			singleton = new InvoiceAppli();
		}
		return singleton;
	}

	private InvoiceAppli() {
		addressHelper = (AddressHelper) addressAppli.getHelper();
		productHelper = (ProductHelper) productAppli.getHelper();
		prodUnitsHelper = (ProdUnitsHelper) prodUnitsAppli.getHelper();
		mapHelper = new InvoiceHelper(addressHelper,
									  productHelper,
									  prodUnitsHelper);
		basicShell = BasicShell.getInstance();
	}

	public boolean interpCmd(String cmd) {
		if (cmd.equals("ci") || cmd.equals("createinvoice"))
			createInvoice();
		else if (cmd.equals("li") || cmd.equals("listinvoice"))
			listInvoice();
		else if (cmd.equals("di") || cmd.equals("deleteinvoice"))
			deleteInvoice();
		else if (addressAppli.interpCmd(cmd))
			return true;
		else if (productAppli.interpCmd(cmd))
			return true;
		else if (prodUnitsAppli.interpCmd(cmd))
			return true;
		else
			return false;
		return true;
	}

	public void help() {
		addressAppli.help();
		productAppli.help();
		prodUnitsAppli.help();
		System.out.println("\t\t- createinvoice | ci : creates a new Invoice using the active mapper.");
		System.out.println("\t\t- listinvoice | li : lists existing Invoices using the active mapper.");
		System.out.println("\t\t- deleteinvoice | di : deletes an existing Invoice using the active mapper.");
	}

	public MapHelper getHelper() {
		return mapHelper;
	}

	public static void main(String[] args) {
		InputStream in = System.in;
		try {
			switch (args.length) {
			case 2:
				in = ObsInputStream.observes(args[1]);
			case 1:
				BasicShell.getInstance().shellLoop(args[0], getInstance(), in);
				break;
			default:
				System.err.println("Usage: AddressAppli prop_file_name [jormexsh_cmdfile]");
				System.exit(-1);
			}
		} catch (Exception e) {
			System.err.println("Exit with exception: " + e.getMessage());
			do {
				e.printStackTrace();
				if (e instanceof PException)
					e = ((PException) e).getNestedException();
				if (e instanceof ResourceException)
					e = ((ResourceException) e).getLinkedException();
				else
					e = null;
			} while (e != null);
		}
	}

	private void createInvoice() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		try {
			String n, a, np;
			n = basicShell.promptString("\tNumber: ", 20);
			a = basicShell.promptString("\tAddress ID: ", 200);
			np = basicShell.promptString("\tNumber of ProdUnits: ", 20);
			if ((n == null) || (a == null) ||
				(n.length() == 0) || (a.length() == 0)) {
				basicShell.errorMsg("wrong number or address ID (mandatory)");
				return;
			}
			String[] pus;
			if ((np == null) || (np.length() == 0)) {
				pus = new String[0];
			} else {
				int inp = Integer.parseInt(np);
				pus = new String[inp];
				for (int i = 0; i < inp; i++) {
					pus[i] = basicShell.promptString("\tProdUnits ID-" + i + ": ", 20);
				}
			}
			Invoice pr = mapHelper.createInvoice(basicShell.activeMapper,
												 Integer.parseInt(n), a, pus);
			System.out.println("\tCreated Invoice:\n\t\t" + pr.toPrintable());
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}

	private void listInvoice() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		try {
			Iterator pit = mapHelper.listInvoices(basicShell.activeMapper);
			System.out.println("\tExisting Invoices:");
			while (pit.hasNext()) {
				PName pn = (PName) pit.next();
				Invoice p = mapHelper.getInvoice(basicShell.activeMapper, pn);
				System.out.println("\t\t" + p.toPrintable());
			}
		} catch (Exception e) {
			if (e instanceof PException)
				((PException) e).getNestedException().printStackTrace();
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}

	private void deleteInvoice() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		String sid;
		sid = basicShell.promptString("\tID: ", 200);
		try {
			if (mapHelper.deleteInvoice(basicShell.activeMapper, sid))
				System.out.println("Invoice deleted.");
			else
				basicShell.errorMsg("cannot delete Invoice");
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}
}
