/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package examples.invoice.applications;

import examples.invoice.persistclass.MapHelper;
import examples.invoice.persistclass.Product;
import examples.invoice.persistclass.ProductHelper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PName;

import java.util.Iterator;
import java.io.InputStream;
import javax.resource.ResourceException;

/**
 * @author	P. Dechamboux
 */
public class ProductAppli implements ShellExtent {
	private static ProductAppli singleton = new ProductAppli();
	private ProductHelper mapHelper;
	private BasicShell basicShell;

	public static ProductAppli getInstance() {
		if (singleton == null) {
			singleton = new ProductAppli();
		}
		return singleton;
	}

	private ProductAppli() {
		mapHelper = new ProductHelper();
		basicShell = BasicShell.getInstance();
	}

	public boolean interpCmd(String cmd) {
		if (cmd.equals("cp") || cmd.equals("createproduct"))
			createProduct();
		else if (cmd.equals("lp") || cmd.equals("listproduct"))
			listProduct();
		else if (cmd.equals("dp") || cmd.equals("deleteproduct"))
			deleteProduct();
		else
			return false;
		return true;
	}

	public void help() {
		System.out.println("\t\t- createproduct | cp : creates a new Product using the active mapper.");
		System.out.println("\t\t- listproduct | lp : lists existing Products using the active mapper.");
		System.out.println("\t\t- deleteproduct | dp : deletes an existing Product using the active mapper.");
	}

	public MapHelper getHelper() {
		return mapHelper;
	}

	public static void main(String[] args) {
		InputStream in = System.in;
		try {
			switch (args.length) {
			case 2:
				in = ObsInputStream.observes(args[1]);
			case 1:
				BasicShell.getInstance().shellLoop(args[0], getInstance(), in);
				break;
			default:
				System.err.println("Usage: AddressAppli prop_file_name [jormexsh_cmdfile]");
				System.exit(-1);
			}
		} catch (Exception e) {
			System.err.println("Exit with exception: " + e.getMessage());
			do {
				e.printStackTrace();
				if (e instanceof PException)
					e = ((PException) e).getNestedException();
				if (e instanceof ResourceException)
					e = ((ResourceException) e).getLinkedException();
				else
					e = null;
			} while (e != null);
		}
	}

	private void createProduct() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		try {
			String n, s, c, w, p;
			n = basicShell.promptString("\tName: ", 50);
			s = basicShell.promptString("\tSize: ", 4);
			c = basicShell.promptString("\tColor: ", 20);
			w = basicShell.promptString("\tWeight: ", 10);
			p = basicShell.promptString("\tPrice: ", 10);
			if ((n == null) || (s == null) || (c == null) || (w == null) || (p == null) ||
				(n.length() == 0) || (s.length() == 0) || (c.length() == 0) || (w.length() == 0) || (p.length() == 0)) {
				basicShell.errorMsg("wrong name, size, color, weight or price (mandatory)");
				return;
			}
			float wf, pf;
			wf = Float.parseFloat(w);
			pf = Float.parseFloat(p);
			Product pr = mapHelper.createProduct(basicShell.activeMapper,
												 n, s.charAt(0), c, wf, pf);
			System.out.println("\tCreated Product:\n\t\t" + pr.toPrintable());
		} catch (PException e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			if (e.getNestedException() != null)
				e.getNestedException().printStackTrace();
			System.out.println();
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}

	private void listProduct() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		try {
			Iterator pit = mapHelper.listProducts(basicShell.activeMapper);
			System.out.println("\tExisting Products:");
			while (pit.hasNext()) {
				PName pn = (PName) pit.next();
				Product p = mapHelper.getProduct(basicShell.activeMapper, pn);
				System.out.println("\t\t" + p.toPrintable());
			}
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}

	private void deleteProduct() {
		if (basicShell.activeMapper == null) {
			basicShell.errorMsg("requires an active mapper (run <activatemapper> command before)");
			return;
		}
		String sid;
		sid = basicShell.promptString("\tID: ", 200);
		try {
			if (mapHelper.deleteProduct(basicShell.activeMapper, sid))
				System.out.println("Product deleted.");
			else
				basicShell.errorMsg("cannot delete Product");
		} catch (Exception e) {
			System.out.println("\n\tEXCEPTION");
			e.printStackTrace();
			System.out.println();
		}
	}
}
