/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MappingFactory;
import org.objectweb.jorm.xml2mi.api.MappingParser;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.jorm.mi2xml.api.MappingDomtreeBuilder;
import org.objectweb.jorm.mi2xml.api.Writer;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.Iterator;
import java.util.Properties;
import java.util.Collection;

/**
 * Used to hold the configuration information related to the JORM runtime.
 * @author	P. Dechamboux
 */
public interface JormConfigurator {
	public static final String LOGGER_NAME =
			"org.objectweb.jorm.config";
	public static final String DEFAULT_JORM_CONFIGURATION_FILE =
			"jorm.properties";
	public static final String USE_CONTEXT_CLASSLOADER =
			"use.context.classloader";

	/**
	 * Configures the JORM compiler by loading information related to the
	 * different mappers supported by this instance of JORM using the given
	 * property file.
	 * @throws PException
	 */
	public void configure(String propertyfile) throws PException;

	/**
	 * Configures the JORM compiler by loading information related to the
	 * different mappers supported by this instance of JORM using the default
	 * property file.
	 * @throws PException
	 */
	public void configure() throws PException;

	/**
	 * Configures the JORM compiler by loading information related to the
	 * different mappers supported by this instance of JORM using the given
	 * properties.
	 * @param p is the properties containing the jorm configuration
	 */
	public void configure(Properties p);

	/**
	 * Configures the logger factory object with a properties object.
	 * @param propertyfile    a properties object
	 */
	public void configureLog(String propertyfile) throws PException;

	/**
	 * Lists the names of supported mappers.
	 * @return	This list Iterator.
	 */
	public Iterator knownMappers() throws PException;

	/**
	 * Gets the MI manager to be used by JORM.
	 * @return	That MI manager.
	 */
	public Manager getMIManager() throws PException;

	/**
	 * Gets the parser to be used by JORM.
	 * @return	That parser.
	 */
	public Parser getParser() throws PException;

	/**
	 * Gets the MI factory associated with the given mapper.
	 * @param mappername	The name of the concerned mapper.
	 * @return The MappingFactory object.
	 * @throws org.objectweb.jorm.api.PException
	 */
	public MappingFactory getMIFactory(String mappername) throws PException;

	/**
	 * Gets the schema manager associated with the given mapper.
	 * @param mappername	The name of the concerned mapper.
	 * @return The PMappingStructuresManager object.
	 * @throws org.objectweb.jorm.api.PException
	 */
	public PMappingStructuresManager getSchMgr(String mappername) throws PException;

	/**
	 * Gets the class name of the GenClassMapping associated with the given mapper.
	 * @param mappername	The name of the concerned mapper.
	 * @return The string giving the class name.
	 * @throws org.objectweb.jorm.api.PException
	 */
	public Class getGcmClass(String mappername) throws PException;

	/**
	 * Gets the parser associated with the given mapper for mapping infos.
	 * @param mappername	The name of the concerned mapper.
	 * @return The MappingParser object.
	 * @throws org.objectweb.jorm.api.PException
	 */
	public MappingParser getMappingParser(String mappername) throws PException;

	/**
	 * Gets the BasicDomWriter object.
	 * @return The BasicDomWriter object.
	 * @throws org.objectweb.jorm.api.PException
	 */
	public Writer getWriter() throws PException;

	/**
	 * Gets the MappingDomTreeBuilder object associated with the given mapper.
	 * @param mappername	The name of the concerned mapper.
	 * @return The MappingDomTreeBuilder object.
	 * @throws org.objectweb.jorm.api.PException
	 */
	public MappingDomtreeBuilder getMappingDomtreeBuilder(String mappername)
			throws PException;

	/**
	 * Gets the submappers associated with a given mapper.
	 * @param mappername    The concerned mapper.
	 * @return  The collection of the names of submappers.
	 * @throws org.objectweb.jorm.api.PException
	 */
	public Collection getSubMappers(String mappername) throws PException;

	/**
	 * Removes this mapper from the list of the ones used for generation.
	 * @param mn    The name of the concerned mapper.
	 */
	public void removeMapper(String mn);

	/**
	 * Removes all mappers from the list of the ones used for generation.
	 */
	public void removeAllMappers();

	/**
	 * Removes this submapper from the list of the ones used for generation.
	 * @param mn    The concerned mapper.
	 * @param smn   The submapper to be removed.
	 */
	public void removeSubMapper(String mn, String smn);

	/**
	 * Adds this submapper to the list of the ones used for generation.
	 * @param mn    The concerned mapper.
	 * @param smn   The submapper to be added.
	 */
	public void addSubMapper(String mn, String smn);

	/**
	 * It assignes the logger factory.
	 * @param lf
	 */
	public void setLoggerFactory(LoggerFactory lf);

	/**
	 * Gets the logger factory associated with this instance of JORM.
	 * @return  The logger factory.
	 */
	public LoggerFactory getLoggerFactory() throws PException;
}
