/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.tuple.api.TupleCollection;

import java.util.HashMap;
import java.util.Iterator;

/**
 * The <b>PClassMapping</b> interface defines the behaviour of objects that
 * contain all information required to map a JORM class. It is a PBinder but
 * mainly delegates this behaviour to a nested PBinder that it must reference.
 * It allows the creation or deletion of the persistent structures required to
 * map this class within a particular hosting data store.
 * It also holds references to naming contexts associated to reference fields,
 * as well as to PClassMapping associated to generic class instances of
 * relevant fields.
 * @author	P. Dechamboux
 */
public interface PClassMapping {
	/**
	 * It tells that a cleanup is required for a JORM class with
	 * the removal of existing data as well as hosting structures.
	 */
	final static byte CLEANUP_REMOVEALL = 1;
	/**
	 * It tells that a cleanup is required for a JORM class with
	 * the removal of only existing data.
	 */
	final static byte CLEANUP_REMOVEDATA = 2;
	/**
	 * It permits to specify to create the if it does not already exist. If the
	 * table exist then nothing is done.
	 */
	final static byte CREATE_STRUCTURE_IF_NEEDED = 3;
	/**
	 * It tells that a cleanup is required for a JORM class with
	 * nothing to do on existing data or hosting structures.
	 */
	final static byte CLEANUP_DONOTHING = 4;

	/**
	 * It creates a new PBinding.
	 * @return	The new PBinding with lifecycle state set to
	 * 			LIFECYCLE_NOTBOUND.
	 */
	PBinding createPBinding() throws PException;

	/**
	 * It intialises the class mapping. It especially assigns a mapper as well
	 * as a meta-object describing either a class or a generic class. It also
	 * defines all type information related to this class mapping.
	 * @param mapper	The mapper to be assigned to this class mapping.
	 * @param metaclass	The meta-object describing the associated class. It may
	 * 					be null.
	 */
	void init(PMappingCallback mapper, MetaObject metaclass)
			throws PException;

	/**
	 * Register the mapping structures and the dependencies.
	 * @param pmsm The PMappingStructuresManager in which mapping structures
	 * 				are registered.
	 * @throws PException
	 */
	void init(PMappingStructuresManager pmsm)
		throws PException;
	
	/**
	 * Register the class in the cluster.
	 * @param pmsm
	 * @throws PException
	 */
	void classDefined(PMappingStructuresManager pmsm) throws PException;
	
	/**
	 * It retrieves the name of the class referenced by this PClassMapping.
	 * should be the same as the one get when calling
	 * "getPBinder().getPType().getJormName()" if the PType exists, that is a
	 * fully qualified name.
	 * @return	The name of this class.
	 */
	String getClassName();

	/**
	 * It retrieves the project name.
	 */
	String getProjectName();

	/**
	 * It gives access to the PClassMapping associated to the element of a
	 * generic class whose type is a generic class.
	 * @return	The PClassMapping of the generic class instance.
	 * @exception UnsupportedOperationException	Raised when the method is not
	 * 											supported by the PClassMapping.
	 */
	PClassMapping getGenClassMapping() throws UnsupportedOperationException;

	/**
	 * It gives access to the PClassMapping associated to a field whose type is
	 * a generic class.
	 * @param fn	The name of the generic class field.
	 * @return	The PClassMapping of the generic class instance.
	 * @exception UnsupportedOperationException	Raised when the method is not
	 * 											supported by the PClassMapping.
	 */
	PClassMapping getGenClassMapping(String fn)
			throws UnsupportedOperationException;

	/**
	 * It returns the Class/GenClass object representing the meta-information
	 * associated to the JORM class/generic class whose mapping is defined by
	 * this PClassMapping. It may be null if not requested at mapping time.
	 * @return	The Class/GenClass meta-object or null.
	 * @see	org.objectweb.jorm.api.PMapper#map
	 */
	MetaObject getMetaInfo();

	/**
	 * It yields the PBinder associated to this PClassMapping. As a PBinder
	 * itself, this PClassMapping delegates this behaviour to this PBinder.
	 * @return	The PBinder associated to this PClassMapping.
	 */
	PBinder getPBinder();

	/**
	 * It gives access to the mapper that has mapped the associated JORM class
	 * as defined by this PClassMapping.
	 * @return	The associated PMapper.
	 */
	PMapper getPMapper();

	/**
	 * It returns the naming context that manages the references contained by
	 * the generic class instance whose mapping is defined by this
	 * PClassMapping.
	 * @return	The naming context that manages references contained by a
	 * 			generic class instance.
	 * @exception	UnsupportedOperationException	It is raised when this
	 * 												PClassMapping does not map
	 * 												a generic class instance, or
	 * 												when the elements of this
	 * 												generic class instance are
	 * 												not references.
	 */
	PNameCoder getPNameCoder()
			throws UnsupportedOperationException;

	/**
	 * It returns the naming context that manages the reference associated to
	 * a field whose type is a JORM class.
	 * @return	The naming context that manages the reference defined by a
	 * 			field.
	 * @exception	UnsupportedOperationException	It is raised when no naming
	 * 												context is associated to
	 * 												the corresponding field.
	 */
	PNameCoder getPNameCoder(String fn)
			throws UnsupportedOperationException;

	/**
	 * It returns the PNameCoder to use for the class. This PNamingContext
	 * manages also the sub classes.
	 */
	PNameCoder getClassPNameCoder();

	/**
	 * It returns an iterator over all pname of persitent objects availlable in
	 * this class. When you use the returned iterator you must leave the
	 * connection opened.
	 * @param conn is the connection which permits to access to the support
	 * @param withSubType indicates if the subclass instances must be included
	 * into the iterator
	 * @param prefetching indicates if prefething must be activated during the
	 * query on the persistent support.
	 * @param txctx The transaction context that can be used in case of
	 *              prefetching.
	 * @return the iterator over PName objects
	 * @exception PExceptionProtocol	A problem occured during the request
	 * 									that is not related to I/O.
	 * @exception PExceptionIO	A problem occured during the request.
	 */
	PNameIterator getPNameIterator(Object conn,
								   boolean withSubType,
								   boolean prefetching,
								   Object txctx) throws PException;

	/**
	 * @deprecated use  getPNameIterator(Object, boolean, boolean)
	 */
	Iterator getPNameIterator(Object conn) throws PException;


	/**
	 * Retrieves the PType of this mapped JORM class.
	 * @return The PTYpe.
	 */
	PType getPType();

	/**
	 * Gets the list of the sub PClassMappings.
	 * @return the list of sub pcms or null if the class has no child
	 * @exception PException if the PType is not defined for the PClassMapping
	 */
	PClassMapping[] getSubPCMs() throws PException;
	
	/**
	 * It specifies if the given kind of PMapper is supported by this
	 * PClassMapping.
	 * @param mappername	The name defining the PMapper kind.
	 * @return	true if this kind of mapper is supported.
	 */
	boolean isConform(String mappername);

	/**
	 * It assigns the PBinder associated to this PClassMapping. As a PBinder
	 * itself, this PClassMapping delegates this behaviour to this PBinder.
	 * @param pb	The PBinder associated to this PClassMapping.
	 * @exception	PExceptionTyping	It is raised when the structure of the
	 * 									the name that identifies the DSI for
	 * 									the PClassMapping is not supported by
	 * 									this PBinder that should be connected
	 * 									to this PClassMapping.
	 */
	void setPBinder(PBinder pb) throws PException;

	/**
	 * It permits to assign the PNameCoder of all the reference fields
	 * of the persitent class. The implementation of this method must call both
	 * method 'getPNameCoder' and 'isCodingOnly' of the specified PNCGetter.
	 * for each field which is a reference (to a class or a GenClass).
	 * @param rc is the callback object which knows the PNameCoder to use.
	 *
	 * @exception	PExceptionTyping	It is raised when the structure of the
	 * 									the names that reference the DSI stored
	 * 									within the generic class instance is not
	 * 									supported by the naming context returned
	 *                                  by the callBack method.
	 *
	 */
	void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException;

	/**
	 *
	 */
	public interface ReferenceConfigurator {
		/**
		 * It retrieves the PNameCoder which must be associated to the field
		 * specified in parameter.
		 * @param sourceclassName is the class name which reference the object.
		 * @param refFieldName is the reference field.
		 * @param destclassName is the class name of the referenced Object.
		 * @return the PNameCoder to use for the specified field
		 */
		PNameCoder getPNameCoder(String sourceclassName,
								 String refFieldName,
								 String destclassName);

		/**
		 * It retrieves the PNameCoder which should manage the reference.
		 * @param refFieldName is the reference field.
		 * @param sourceclassName is the class name of the non generic class name
		 * which references a begin of the chain of generic Object.
		 * @param genClassNames is the list of the generic class name.
		 * @return the PNameCoder to use for the specified field
		 */
		PNameCoder getPNameCoder(String sourceclassName,
								 String refFieldName,
								 String[] genClassNames);

		/**
		 * It retrieves the PClassMapping which must be associated to the field
		 * specified in parameter.
		 * @param refFieldName is the reference field.
		 * @param sourceclassName is the class name of the non generic class name
		 * which references a begin of the chain of generic Object.
		 * @param genClassNames is the list of the generic class name.
		 * @return the PClassMapping to use for the specified field
		 */
		PClassMapping getGenClassMapping(String sourceclassName,
										 String refFieldName,
										 String[] genClassNames);

		/**
		 * It retrieves the PClassMapping which must be associated to the field
		 * specified in parameter.
		 * @param refFieldName is the reference field.
		 * @param sourceclassName is the class name of the non generic class name
		 * which references a begin of the chain of generic Object.
		 * @param genClassNames is the list of the generic class name.
		 * @param destclassName is the class name of the referenced Object.
		 * @return the PClassMapping to use for the specified field
		 */
		PClassMapping getGenClassMapping(String sourceclassName,
										 String refFieldName,
										 String[] genClassNames,
										 String destclassName);
	}

	/**
	 * It verifies if the DSI attached to a binding exists within the DS.
	 * @param pb	is the PBinding instance
	 * @param conn	The connection that can be used to refer to the DS.
	 * @return	It returns true if the DSI exists.
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 */
	boolean exist(PBinding pb, Object conn) throws PException;

	/**
	 * It reads the data store instance designated by the PName of a
	 * binding and transfers each attribute value to the accessor object.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @param pa	The PAccessor used to access memory variables.
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 */
	void read(PBinding pb, Object conn, PAccessor pa) throws PException;

	/**
	 * It reads the data store instance designated by the PName of a
	 * binding and transfers each attribute value to the accessor object.
	 * This read method should be used when prefetching is requested.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @param pa	The PAccessor used to access memory variables.
	 * @param txctx The transaction context that can be used in case of
	 *              prefetching.
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 */
	void read(PBinding pb, Object conn, PAccessor pa, Object txctx) throws PException;

	/**
	 * It transfers each attribute value taken from the PAccessor
	 * object to the data store instance designated by the PName
	 * associated with a binding.
	 * @param conn	The connection that can be used to refer to the DS.
	 * @param pa	The PAccessor used to access memory variables.
	 * @exception PExceptionIO	It is raised when a problem occured while
	 * 							accessing to the DS.
	 * @exception PExceptionNaming	It is raised when there is a problem for the
	 * 								associated binder to manage this PName.
	 * @exception PExceptionProtocol	It is raised when it is called with a
	 * 									state different from the following ones:
	 * 									LIFECYCLE_ACTIVEFORIO
	 * 									LIFECYCLE_NEWTOWRITE
	 * 									LIFECYCLE_DELTOWRITE
	 * @exception PExceptionTyping	It is raised when a reference to be stored
	 * 								has a type incompatible with the respective
	 * 								field definition.
	 */
	void write(PBinding pb, Object conn, PAccessor pa) throws PException;

	/**
	 * Returns the map storing association between a pclassmapping on which is done a query 
	 * and a table of int representing the fields numbers to select in the tuple to fill in
	 * the field of an instance of the class.
	 * Example:
	 * Let's say we are instanciating D objects
	 * A(pk1, pk2, f1), B(f2), C(f3, f4), D(f5)
	 * 		_________A_______
	 * 		B				C____
	 * 							D
	 * 
	 * The prefetch query on A will be:
	 * 	Select pk1, pk2, f1, f2, f3, f4, f5
	 * For D, the index table will be [1,2,3,5,6,7] because we need only pk1, pk2, f1, f3, f4, f5
	 * to instanciate D objects
	 * The prefetch query on C will be:
	 *  Select pk1, pk2, f1, f3, f4, f5
	 * For D, the index table will be [1,2,3,4,5,6] because we need all the fields
	 * to instanciate D objects
	 * 
	 * Then, the map returned for D will be:
	 * [ pclassmapping of A, [1,2,3,5,6,7]]
	 * [ pclassmapping of C, [1,2,3,4,5,6]]
	 */
	HashMap getAssociationTable();
	
	/**
	 * Adds an association [pclassmapping-> table of indexes] to the association table.
	 * If the map already has an entry corresponding to the pclassmapping, 
	 * this entry is replaced by the new one. 
	 * @param targetClass the PClassMapping on which is done the query
	 * @param indexes a table of int composed of the indexes of fields required
	 */
	void addAssociation(PClassMapping targetClass, int[] indexes);
	
	/**
	 * Returns the indexes table associated to the pclassmapping targetClass, 
	 * null if not contained in the map.
	 */
	int[] getIndexesTable(PClassMapping targetClass);
	
	/**
	 * It returns the PName valid in the "good" binder.
	 * @param conn	The connection that can be used to refer to the DS.
	 */
	PName resolve(Object conn, PName pname)	throws PException;
	
	/**
	 * Returns true if the object obj matches (exactly) this class,
	 * else it returns false.
	 */
	boolean match(Object obj, boolean intermediaryTuple) throws PException;
	
	/**
	 * It returns the PName valid in the "good" binder according to the tuple collection.
	 * @throws PException
	 */
	PName getDecodedPName(TupleCollection tc, PName pname, boolean intermediaryTuple) throws PException;
}
