/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.type.api.PExceptionTyping;

import java.io.Serializable;
import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * The <b>PIndexedElem</b> interface defines the representation of an
 * element of a generic class. It provides a way to access the element value,
 * as well as its index. It also allows the association of a status for such
 * elements when the generic class accessor implemented by the MI supports
 * delta modifications (@see PGenClassAccessor#deltaSupported).
 * @author	R. Basset, P. Dechamboux
 */
public interface PIndexedElem {
	/**
	 * It specifies that this element is a new one within this generic class
	 * object. This means that is new index has been created.
	 */
	final static byte ELEM_CREATED = 1;

	/**
	 * It specifies that this element has been deleted from this generic class
	 * object. This means that no more element with this index exists anymore
	 * wiothin this generic class object.
	 */
	final static byte ELEM_DELETED = 2;

	/**
	 * It specifies that this element as been modified within this generic class
	 * object. This means that only the value of the indexed element has been
	 * modified, and especially not the index.
	 */
	final static byte ELEM_MODIFIED = 3;

	/**
	 * It specifies that this element existed at load time and that it has not
	 * been modified until then. This means that neither the value of the
	 * indexed element nor its index has been modified.
	 */
	final static byte ELEM_UNMODIFIED = 4;

	/**
	 * It yields the status of this indexed element. The status value belongs to
	 * the following ones:
	 * <ul>
	 * <li>ELEM_CREATED</li>
	 * <li>ELEM_DELETED</li>
	 * <li>ELEM_MODIFIED</li>
	 * <li>ELEM_UNMODIFIED</li>
	 * </ul>
	 */
	byte getElemStatus();

	/**
	 * It retrieves the value of a boolean element.
	 * @return	The boolean value of the field as stored within the memory
	 * instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	boolean pieGetBooleanElem() throws PException;

	/**
	 * It retrieves the value of a boolean element.
	 * @return	The Boolean value of the field as stored within the memory
	 * instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Boolean pieGetObooleanElem() throws PException;

	/**
	 * It retrieves the value of a byte element.
	 * @return	The byte value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	byte pieGetByteElem() throws PException;

	/**
	 * It retrieves the value of a byte element.
	 * @return	The Byte value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Byte pieGetObyteElem() throws PException;

	/**
	 * It retrieves the value of a byte index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The byte value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	byte pieGetByteIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a byte index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The Byte value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	Byte pieGetObyteIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a char element.
	 * @return	The char value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	char pieGetCharElem() throws PException;

	/**
	 * It retrieves the value of a char element.
	 * @return	The Character value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Character pieGetOcharElem() throws PException;

	/**
	 * It retrieves the value of a char index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The char value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	char pieGetCharIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a char index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The Character value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	Character pieGetOcharIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a short element.
	 * @return	The short value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	short pieGetShortElem() throws PException;

	/**
	 * It retrieves the value of a short element.
	 * @return	The Short value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Short pieGetOshortElem() throws PException;

	/**
	 * It retrieves the value of a short index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The short value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	short pieGetShortIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a short index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The Short value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	Short pieGetOshortIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of an int element.
	 * @return	The int value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	int pieGetIntElem() throws PException;

	/**
	 * It retrieves the value of an int element.
	 * @return	The Integer value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Integer pieGetOintElem() throws PException;

	/**
	 * It retrieves the value of an int index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The int value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	int pieGetIntIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of an int index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The Integer value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	Integer pieGetOintIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a long element.
	 * @return	The long value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	long pieGetLongElem() throws PException;

	/**
	 * It retrieves the value of a long element.
	 * @return	The Long value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Long pieGetOlongElem() throws PException;

	/**
	 * It retrieves the value of a long index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The long value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	long pieGetLongIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a long index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The Long value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	Long pieGetOlongIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a float element.
	 * @return	The float value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	float pieGetFloatElem() throws PException;

	/**
	 * It retrieves the value of a float element.
	 * @return	The Float value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Float pieGetOfloatElem() throws PException;

	/**
	 * It retrieves the value of a double element.
	 * @return	The double value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	double pieGetDoubleElem() throws PException;

	/**
	 * It retrieves the value of a double element.
	 * @return	The Double value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Double pieGetOdoubleElem() throws PException;

	/**
	 * It retrieves the value of a string element.
	 * @return	The java.lang.String value of the field as stored within the
	 * 			memory instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	String pieGetStringElem() throws PException;

	/**
	 * It retrieves the value of a string index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The java.lang.String value of the field as stored within the
	 * 			memory instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	String pieGetStringIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a date element.
	 * @return	The java.sql.Date value of the field as stored within the
	 * 			memory instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Date pieGetDateElem() throws PException;

	/**
	 * It retrieves the value of a date index field.
	 * @param fn	A String containing the name of the index field.
	 * @return	The java.util.Date value of the field as stored within the
	 * 			memory instance.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	Date pieGetDateIndexField(String fn) throws PException;

	/**
	 * It retrieves the value of a chararray element.
	 * @return	The char[] value of the field as stored within the
	 * 			memory instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	char[] pieGetCharArrayElem() throws PException;

	/**
	 * It retrieves the value of a bytearray element.
	 * @return	The byte[] value of the field as stored within the
	 * 			memory instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	byte[] pieGetByteArrayElem() throws PException;

	/**
	 * It retrieves the value of a serialized element.
	 * @return	The java.io.Serializable value of the field as stored within
	 * 			the memory instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	Serializable pieGetSerializedElem() throws PException;

	/**
	 * It retrieves the value of a BigInteger element.
	 * @return	The java.math.BigInteger value of the field as stored within
	 * 			the memory instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	BigInteger pieGetBigIntegerElem() throws PException;

	/**
	 * It retrieves the value of a BigDecimal element.
	 * @return	The java.math.BigDecimal value of the field as stored within
	 * 			the memory instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	BigDecimal pieGetBigDecimalElem() throws PException;

	/**
	 * It retrieves the value of a reference element.
	 * @return	The reference value of the field as stored within the memory
	 * 			instance.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	PName pieGetRefElem() throws PException;

	/**
	 * It assigns the value to a boolean element.
	 * @param value	The boolean value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetBooleanElem(boolean value) throws PException;

	/**
	 * It assigns the value to a boolean element.
	 * @param value	The Boolean value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetObooleanElem(Boolean value) throws PException;

	/**
	 * It assigns the value to a byte element.
	 * @param value	The byte value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetByteElem(byte value) throws PException;

	/**
	 * It assigns the value to a Byte element.
	 * @param value	The Byte value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetObyteElem(Byte value) throws PException;

	/**
	 * It assigns the value to a byte index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The byte value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetByteIndexField(String fn, byte value) throws PException;

	/**
	 * It assigns the value to a Byte index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The Byte value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetObyteIndexField(String fn, Byte value) throws PException;

	/**
	 * It assigns the value to a char element.
	 * @param value	The char value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetCharElem(char value) throws PException;

	/**
	 * It assigns the value to a Character element.
	 * @param value	The Character value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetOcharElem(Character value) throws PException;

	/**
	 * It assigns the value to a char index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The char value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetCharIndexField(String fn, char value) throws PException;

	/**
	 * It assigns the value to a Character index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The Character value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetOcharIndexField(String fn, Character value) throws PException;

	/**
	 * It assigns the value to a short element.
	 * @param value	The short value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetShortElem(short value) throws PException;

	/**
	 * It assigns the value to a Short element.
	 * @param value	The Short value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetOshortElem(Short value) throws PException;

	/**
	 * It assigns the value to a short index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The short value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetShortIndexField(String fn, short value) throws PException;

	/**
	 * It assigns the value to a Short index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The Short value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetOshortIndexField(String fn, Short value) throws PException;

	/**
	 * It assigns the value to an int element.
	 * @param value	The int value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetIntElem(int value) throws PException;

	/**
	 * It assigns the value to an Integer element.
	 * @param value	The Integer value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetOintElem(Integer value) throws PException;

	/**
	 * It assigns the value to an int index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The int value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetIntIndexField(String fn, int value) throws PException;

	/**
	 * It assigns the value to an Integer index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The Integer value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetOintIndexField(String fn, Integer value) throws PException;

	/**
	 * It assigns the value to a long element.
	 * @param value	The long value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetLongElem(long value) throws PException;

	/**
	 * It assigns the value to a Long element.
	 * @param value	The Long value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetOlongElem(Long value) throws PException;

	/**
	 * It assigns the value to a long index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The long value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetLongIndexField(String fn, long value) throws PException;

	/**
	 * It assigns the value to a Long index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The Long value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetOlongIndexField(String fn, Long value) throws PException;

	/**
	 * It assigns the value to a float element.
	 * @param value	The float value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetFloatElem(float value) throws PException;

	/**
	 * It assigns the value to a Float element.
	 * @param value	The Float value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetOfloatElem(Float value) throws PException;

	/**
	 * It assigns the value to a double element.
	 * @param value	The double value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetDoubleElem(double value) throws PException;

	/**
	 * It assigns the value to a Double element.
	 * @param value	The Double value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetOdoubleElem(Double value) throws PException;

	/**
	 * It assigns the value to a string element.
	 * @param value	The java.lang.String value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetStringElem(String value) throws PException;

	/**
	 * It assigns the value to a string index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The java.lang.String value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetStringIndexField(String fn, String value) throws PException;

	/**
	 * It assigns the value to a date element.
	 * @param value	The java.sql.Date value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetDateElem(Date value) throws PException;

	/**
	 * It assigns the value to a date index field.
	 * @param fn	A String containing the name of the field to be set.
	 * @param value	The java.util.Date value to assign.
	 * @exception	PExceptionTyping	There is no field with this name and
	 * 									this type within this index.
	 */
	void pieSetDateIndexField(String fn, Date value) throws PException;

	/**
	 * It assigns the value to a chararray element.
	 * @param value	The char[] value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetCharArrayElem(char[] value) throws PException;

	/**
	 * It assigns the value to a bytearray element.
	 * @param value	The byte[] value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetByteArrayElem(byte[] value) throws PException;

	/**
	 * It assigns the value to a serialized element.
	 * @param value	The java.io.Serializable value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetSerializedElem(Serializable value) throws PException;

	/**
	 * It assigns the value to a BigInteger element.
	 * @param value	The java.math.BigInteger value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetBigIntegerElem(BigInteger value) throws PException;

	/**
	 * It assigns the value to a BigDecimal element.
	 * @param value	The java.math.BigDecimal value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetBigDecimalElem(BigDecimal value) throws PException;

	/**
	 * It assigns the value to a reference element.
	 * @param value	The reference value to assign.
	 * @exception	PExceptionTyping	Elements are not of this type.
	 */
	void pieSetRefElem(PName value) throws PException;
}
