/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.api;

import java.util.Collection;
import java.util.Set;

/**
 * The <b>PMapCluster</b> interface defines a the set of JORM classes which
 * mapping structures have to be managed altogether. It gives access to this
 * set of classes defined at mapping-time, and provides functions to control
 * the associated mapping structures.
 * @author	P. Dechamboux
 */
public interface PMapCluster {
	/**
	 * Retrieves the collection of all JORM classes whose
	 * mapping structures have to be controlled altogether.
	 * @return  The collection of jorm class name included in this PMapCluster
	 * (String)
	 */
	Collection getClusterClasses();

	/**
	 * Indicates if the mapping cluster is fully defined.
	 * @return the 'true' value if the mapping cluster is fully defined, false
	 * otherwise.
	 */
	boolean isDefined();

	/**
	 * @return the collection of unresolved dependencies. This is the set of
	 * jorm class names which must be mapped before starting the cluster.
	 */
	Set getUnResolvedDependencies();

	/**
	 * Creates the mapping structures defined by this map cluster.
	 * @param force	If true, the creation of an existing table leads to an exception.
	 * @throws PException   Thrown if it cannot be performed (especially
	 * if all or some of these mapping structures already exist.
	 */
	void createMappingStructures(boolean force) throws PException;

	/**
	 * Deletes the data that have been inserted into the mapping structures
	 * defined by this map cluster.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 */
	void deleteData() throws PException;

	/**
	 * Deletes all or part of the mapping structures defined by this map
	 * cluster when they exist. If they contain some data, they are also
	 * deleted.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 */
	void deleteMappingStructures() throws PException;

	/**
	 * Starts this map cluster. The following operation cannot be executed
	 * anymore: createMappingStructures, deleteMappingStructures, and
	 * updateMappingStructures.
	 * @throws PException
	 */
	void start() throws PException;

	/**
	 * Stops this map cluster.
	 * @throws PException
	 */
	void stop() throws PException;

	/**
	 * Aligns existing mapping structures to the ones define by this map
	 * cluster. This operation allows schema evolution on existing data store.
	 * It is thus usually difficult to support in the general case.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 * @throws UnsupportedOperationException    Thrown if this operation is not
	 * supported by the mapping.
	 */
	void updateMappingStructures()
			throws PException, UnsupportedOperationException;

	void classDefined(String jcname);

    /**
     * Add a dependency to the cluster
     * @param jcname the name of the jorm class
     */
	void addDependency(String jcname);


}
