/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.compiler.api;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.util.io.api.PathExplorer;
import org.objectweb.jorm.util.api.Loggable;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * A JormCompilerParameter object is an object which manages informations
 * read by the command line parser. These informations are needed to
 * the compiler system to parse, generate, and compile.
 * @author	X. Spengler
 */
public interface JormCompilerParameter extends Cloneable, Loggable {
	/**
	 * The pattern that represents the JORM class name to be replaced into
	 * the string defining the inherited class of a either the PBinding or the
	 * PClassMapping that is generated.
	 */
	String INHERITANCE_CLASSNAME_PATTERN = "%c";
	String INHERITANCE_PACKAGE_PATTERN = "%p";

	/**
	 * Prints informations managed by the current object.
	 */
	void print();

	/**
	 * Computes the actual class name which the generated PClassMapping class
	 * must extend.
	 * @param cn	The class name of the persistent class for which the
	 * 				code is generated.
	 * @return	The computed class name.
	 */
	String computePClassMappingInheritance(String cn);

	/**
	 * Computes the actual class name which the generated PBinding class
	 * must extend.
	 * @param cn	The class name of the persistent class for which the
	 * 				code is generated.
	 * @return	The computed class name.
	 */
	String computePBindingInheritance(String cn);

	/**
	 * Load a jorm configuration file.
	 * @param file  the name of the jorm configuration file
	 */
	void loadConfFile(String file, Iterator knownmappers) throws PException;

	/**
	 * Load a jorm configuration file.
	 * @param in  the input stream of the jorm configuration file
	 */
	void loadConfFile(InputStream in, Iterator knownmappers) throws PException;

	PathExplorer getClasspath();

	/**
	 * @return the output directory where the generated will be put
	 */
	String getOutput();

	/**
	 * @return a boolean value indicating if the verbose mode is activated
	 */
	boolean isVerbose();

	Collection getInputFiles();

	boolean isKeepSrc();

	String getJavacName();

	String getProjectName();

	boolean isJavac();

	boolean isParseOnly();

	/**
	 * @return the name of the class inherited by the generated XXXBinding
	 */
	String getBindingInheritance();

	String getClassMappingInheritance();

	boolean isBindingAbstract();

	ArrayList getDtdLocations();

	boolean isGeneratedPDFiles();

	boolean isGeneratedWithMapperPackage();

	void setClasspath(PathExplorer classpath);

	void setOutput(String output);

	void setVerbose(boolean verbose);

	void setKeepSrc(boolean keepSrc);

	void setProjectName(String projectName);

	void setJavac(boolean javac);

	void setBindingInheritance(String bindingInheritance);

	void setClassMappingInheritance(String classMappingInheritance);

	void setBindingAbstract(boolean bindingAbstract);

	void setDtdLocations(ArrayList dtdLocations);

	void setGeneratedPDFiles(boolean generatedPDFiles);

	void setInputFiles(Collection inputFiles);

	void setGeneratedWithMapperPackage(boolean generatedWithMapperPackage);

	void setLogConfFile(String logConfFile);

	Object clone();
}
