/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.compiler.lib;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.JormCompilerConfigurator;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.generator.api.Generator;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.mi2xml.api.Writer;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.jorm.util.io.lib.JavaFileHolder;
import org.objectweb.jorm.verifier.api.Verifier;
import org.objectweb.jorm.xml2mi.api.MappingParser;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public class JormCompiler {
	/**
	 * The logger used by the compiler to log messages
	 */
	protected Logger logger;

	/**
	 * The logger factory used by the compiler to log messages
	 */
	protected LoggerFactory loggerFactory = null;

	/**
	 * The manager is in charge of the meta information management.
	 */
	protected Manager manager;

	/**
	 * The parser object is able to parse XML files and build an internal
	 * representation of its content.
	 */
	protected Parser parser;

	/**
	 * The verifier is in charge to check the semantical validity of the meta
	 * information.
	 */
	protected Verifier verifier;

	/**
	 * generator is a simple JormGenerator object, which is able to generate
	 * java file.
	 */
	protected Generator generator;

	/**
	 * domWriter is a DomWriter object that generates .pd files from org.w3c.dom.Document
	 * objects representing Class and CompositeName objects.
	 */
	protected Writer writer;

	/**
	 * The compiler parameters
	 */
	protected JormCompilerParameter compilerParameter;

	/**
	 * The compiler configurator
	 */
	protected JormCompilerConfigurator compilerConfigurator;

	/**
	 * It builds a JormCompiler with the default JormCompilerParameter and
	 * JormCompilerParameter.
	 */
	public JormCompiler() {
		compilerParameter = new JormCompilerParameterImpl();
		compilerConfigurator = new JormCompilerConfiguratorImpl();
	}

	/**
	 * It builds a JormCompiler with the specified JormCompilerParameter
	 * instance.
	 */
	public JormCompiler(JormCompilerParameter cp, JormCompilerConfigurator cc) {
		this.compilerParameter = cp;
		this.compilerConfigurator = cc;
	}

	/**
	 * It retrieves the CompilerParameter associated to the compiler
	 * @return the compiler parameter
	 */
	public JormCompilerParameter getCompilerParameter() {
		return compilerParameter;
	}

	/**
	 * It retrieves the CompilerParameter associated to the compiler
	 * @return the compiler parameter
	 */
	public JormCompilerConfigurator getCompilerConfigurator() {
		return compilerConfigurator;
	}

	/**
	 * It assigns a LoggerFactory and a Logger if it is not already done.
	 */
	public void setupLogger() throws PException {
		if (loggerFactory != null) {
			logger.log(BasicLevel.DEBUG, "The log system is already setup");
			return;
		}
		loggerFactory = compilerConfigurator.getLoggerFactory();
		logger = loggerFactory.getLogger("org.objectweb.jorm.compiler");
		logger.log(BasicLevel.DEBUG, "Logger factory assigned");
	}

	public void setupMIManager() throws PException {
		if (manager != null) {
			logger.log(BasicLevel.DEBUG, "The meta information manager is already defined");
			return;
		}
		logger.log(BasicLevel.DEBUG, "Instanciate the meta-information manager");
		manager = compilerConfigurator.getMIManager();
		((Loggable) manager).setLoggerFactory(loggerFactory);
		((Loggable) manager).setLogger(loggerFactory.getLogger("org.objectweb.jorm.metainfo"));
		manager.init();
		Iterator it = compilerConfigurator.knownMappers();
		while (it.hasNext()) {
			String mn = (String) it.next();
			logger.log(BasicLevel.DEBUG, "Register specific meta-information management for mapper [" + mn + "]");
			manager.addMappingFactory(mn, compilerConfigurator.getMIFactory(mn));
		}
	}

	public void setupParser() throws PException {
		if (parser != null) {
			logger.log(BasicLevel.DEBUG, "The parser is already defined");
			return;
		}
		logger.log(BasicLevel.DEBUG, "Instanciate the parser");
		parser = compilerConfigurator.getParser();
		((Loggable) parser).setLoggerFactory(loggerFactory);
		((Loggable) parser).setLogger(loggerFactory.getLogger("org.objectweb.jorm.xml2mi"));
		parser.setMetaInfoManager(manager);
		logger.log(BasicLevel.DEBUG, "Assign the path explorer to parser");
		parser.setPathExplorer(compilerParameter.getClasspath());
		parser.init(true, compilerParameter.getDtdLocations());
		Iterator it = compilerConfigurator.knownMappers();
		while (it.hasNext()) {
			String mn = (String) it.next();
			logger.log(BasicLevel.DEBUG, "Register specific parser for mapper [" + mn + "]");
			MappingParser mp = compilerConfigurator.getMappingParser(mn);
			((Loggable) mp).setLoggerFactory(loggerFactory);
			((Loggable) mp).setLogger(loggerFactory.getLogger("org.objectweb.jorm.xml2mi." + mn));
			parser.addMappingParser(mn, mp);
		}
		logger.log(BasicLevel.DEBUG, "Parser setup terminated.");
	}

	public void setupVerifier() throws PException {
		if (verifier != null) {
			logger.log(BasicLevel.DEBUG, "The verifier is already defined");
			return;
		}
		logger.log(BasicLevel.DEBUG, "Instanciate the verifier");
		verifier = compilerConfigurator.getVerifier();
		((Loggable) verifier).setLoggerFactory(loggerFactory);
		((Loggable) verifier).setLogger(loggerFactory.getLogger("org.objectweb.jorm.verifier"));
		verifier.setMetaInfoManager(manager);
		verifier.setProjectName(compilerParameter.getProjectName());
		Iterator it = compilerConfigurator.knownMappers();
		while (it.hasNext()) {
			String mn = (String) it.next();
			logger.log(BasicLevel.DEBUG, "Register specific verifier for mapper [" + mn + "]");
			verifier.addMappingVerifier(mn, compilerConfigurator.getMappingVerifier(mn));
		}
	}

	public void setupGenerator() throws PException {
		if (generator != null) {
			logger.log(BasicLevel.DEBUG, "The generator is already defined");
			return;
		}
		logger.log(BasicLevel.DEBUG, "Instanciate the generator");
		generator = compilerConfigurator.getGenerator();
		((Loggable) generator).setLogger(loggerFactory.getLogger("org.objectweb.jorm.generator"));
		((Loggable) generator).setLoggerFactory(loggerFactory);
		generator.setPathExplorer(compilerParameter.getClasspath());
		generator.setMetaInfoManager(manager);
		generator.setCompilerConfigurator(compilerConfigurator);
		generator.setCompilerParameter(compilerParameter);
		generator.init();
		Iterator it = compilerConfigurator.knownMappers();
		while (it.hasNext()) {
			String mn = (String) it.next();
			logger.log(BasicLevel.DEBUG, "Register specific generator for mapper [" + mn + "]");
			generator.addMappingGenerator(mn, compilerConfigurator.getMOPFactory(mn));
		}
	}

	/**
	 * Creates class instances for the XML parser, the Meta Information Manager
	 * and manages the JormGenerator system.
	 */
	public void process() throws PException {
		setupLogger();
		// parser part of the Jorm Compiler System
		logger.log(BasicLevel.DEBUG, "process with jormc; files:" + compilerParameter.getInputFiles());
		Collection metaObjects = parseFiles(compilerParameter.getInputFiles());
		// generates the .pd files
		logger.log(BasicLevel.DEBUG, "generates " + metaObjects.size() + "jorm files");
		if (compilerParameter.isGeneratedPDFiles()) {
			generateJormFiles(metaObjects);
		}
		//verifyMetaInfo(metaObjects);
		if (!compilerParameter.isParseOnly()) {
			Collection javafiles = generateFiles(metaObjects);
			if (compilerParameter.isJavac()) {
				compileFiles(javafiles);
			}
		}
	}

	/**
	 * Parses the input files and build the meta information.
	 */
	public Collection parseFiles(Collection fns) throws PException {
		setupLogger();
		setupMIManager();
		setupParser();
		logger.log(BasicLevel.DEBUG, "parsing of files");
		return parser.parse(fns.iterator());
	}

	/**
	 * Verifies if the metainformation is in a clean state.
	 */
	public void verifyMetaInfo(Collection mos) throws PException {
		setupLogger();
		setupVerifier();
		verifier.verify(mos);
	}

	/**
	 * Generates java files for all meta object specified in the Collection
	 * parameter. If the parameter is null, all classes described in the meta
	 * information manager are generated.
	 * @param c is the collection of meta objects
	 * @return a Collection of String. The elements are the file names of the
	 * generated files
	 * @exception PException
	 */
	public Collection generateFiles(Collection c) throws PException {
		setupLogger();
		setupMIManager();
		setupGenerator();
		TargetHolder targetHolder;
		logger.log(BasicLevel.DEBUG, "Output directory=" + compilerParameter.getOutput());
		try {
			targetHolder = new JavaFileHolder(compilerParameter.getOutput());
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Impossible to create a JavaFileHolder");
		}
		Iterator mos = (c != null
				? c.iterator()
				: manager.getJormObjects().iterator());
		while (mos.hasNext()) {
			MetaObject o = (MetaObject) mos.next();
			if (o instanceof CompositeName) {
				logger.log(BasicLevel.INFO, "Generation for the [" + ((CompositeName) o).getName() + "] composite name");
				generator.generate(((CompositeName) o), targetHolder, compilerParameter);
			} else if (o instanceof Class) {
				logger.log(BasicLevel.INFO, "Generation for the [" + ((Class) o).getName() + "] class");
				try {
					generator.generate((Class) o, targetHolder, compilerParameter, compilerConfigurator);
				} catch (Exception e) {
					logger.log(BasicLevel.ERROR,
							   "Error during the generation of the class "
							   + ((Class) o).getFQName(), e);
					if (e instanceof PException) {
						throw (PException) e;
					} else {
						throw new PExceptionCompiler(e,
													 "Error during the generation of the class "
													 + ((Class) o).getFQName());
					}
				}
			} else {
				throw new PExceptionCompiler("Meta Object is not a class or a composite name : " + o);
			}
		}
		ArrayList al = new ArrayList();
		for (Iterator it = targetHolder.iterateFile(); it.hasNext();) {
			try {
				al.add(((File) it.next()).getCanonicalPath());
			} catch (IOException e) {
				throw new PException(e, "Cannot build the list of generated files.");
			}
		}
		return al;
	}

	/**
	 * Compiles with javac the java files generated by the generator system.
	 * @param javafiles All java files to be compiled with javac.
	 */
	public void compileFiles(Collection javafiles) throws PException {
		setupLogger();
		// TODO
		throw new UnsupportedOperationException("NOT YET IMPLEMENTED!");
	}

	/**
	 * Retrieves the MIanager instance. It can be null.
	 */
	public Manager getMIManager() {
		return manager;
	}

	/**
	 * Assignes the Meta info manager
	 */
	public void setMIManager(Manager manager) {
		this.manager = manager;
	}

	public void setupWriter() throws PException {
		if (writer != null) {
			logger.log(BasicLevel.DEBUG, "The writer is already defined");
			return;
		}
		logger.log(BasicLevel.DEBUG, "Instanciate the jorm writer");
		writer = compilerConfigurator.getWriter();
		((Loggable) writer).setLogger(loggerFactory.getLogger("org.objectweb.jorm.mi2xml"));
		((Loggable) writer).setLoggerFactory(loggerFactory);
		//writer.setPathExplorer(compilerParameter.classpath);
		writer.init(compilerConfigurator);
	}

	/**
	 * Generates .files for all meta object specified in the Collection
	 * parameter. If the parameter is null, all classes described in the meta
	 * information manager are generated.
	 * @param c is the collection of meta objects
	 * @return a Collection of String. The elements are the file names of the
	 * generated files
	 * @exception PException
	 */
	public Collection generateJormFiles(Collection c) throws PException {
		setupLogger();
		setupMIManager();
		setupWriter();
		TargetHolder targetHolder;
		logger.log(BasicLevel.DEBUG, "Output directory=" + compilerParameter.getOutput());
		try {
			targetHolder = new JavaFileHolder(compilerParameter.getOutput());
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Impossible to create a JavaFileHolder");
		}
		Iterator mos = (c != null
				? c.iterator()
				: manager.getJormObjects().iterator());
		while (mos.hasNext()) {
			MetaObject o = (MetaObject) mos.next();
			if (o instanceof CompositeName) {
				logger.log(BasicLevel.INFO,
						   ".pd file generation for the [" + ((CompositeName) o).getName() + "] composite name");
				writer.write(o, targetHolder);

			} else if (o instanceof Class) {
				logger.log(BasicLevel.INFO, ".pd file generation for the [" + ((Class) o).getName() + "] class");
				try {
					writer.write(o, targetHolder);
				} catch (Exception e) {
					logger.log(BasicLevel.ERROR,
							   "Error during the .pd file generation of the class "
							   + ((Class) o).getFQName(), e);
					if (e instanceof PException) {
						throw (PException) e;
					} else {
						throw new PExceptionCompiler(e,
													 "Error during the .pd file generation of the class "
													 + ((Class) o).getFQName());
					}
				}
			} else {
				throw new PExceptionCompiler("Meta Object is not a class or a composite name : " + o);
			}
		}
		ArrayList al = new ArrayList();
		for (Iterator it = targetHolder.iterateFile(); it.hasNext();) {
			try {
				al.add(((File) it.next()).getCanonicalPath());
			} catch (IOException e) {
				throw new PExceptionCompiler(e, "Impossible to retrieve File descriptors");
			}
		}
		return al;
	}
}
