/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.facility.naming.rdbsequence;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.util.monolog.api.Logger;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.ResultSet;

/**
 * This helper permits to create and consult a RDB sequence. It requires a
 * RdbAdapter.
 *
 * @author S.Chassande-Barrioz
 */
public class RdbSequenceHelper {

	/**
	 * The SQL request for asking a new value
	 */
	private String nextValRequest;

	/**
	 * The Rdb Adapter permitting to choose the SQL syntax corresponding to
	 * the relational database.
	 */
	private RdbAdapter adapter;

	/**
	 * The sequence name
	 */
	private String seqName;

	/**
	 * the sequence start. A null value means that no value has been specified.
	 * In this case the default sequence is built with the default sequence
	 * start.
	 */
	private Integer seqStart = null;

	/**
	 * The sequence increment. A null value means that no value has been
	 * specified. In this case the default sequence is built with the default
	 * sequence increment.
	 */
	private Integer seqIncrement = null;

	/**
	 * The sequence cache. A null value means that no value has been
	 * specified. In this case the default sequence is built with the default
	 * sequence cache.
	 */
	private Integer seqCache = null;

	private boolean created;

	private Logger logger;

	public RdbSequenceHelper() {
	}

	/**
	 * @param adapter is the Rdb Adapter permitting to choose the SQL syntax corresponding to
	 * the relational database. (not null)
	 * @param seqName is the sequence name (not null)
	 */
	public RdbSequenceHelper(RdbAdapter adapter,
							 String seqName,
							 boolean sequenceCreated) {
		this();
		setAdapter(adapter);
		setSequenceName(seqName);
		setSequenceCreated(sequenceCreated);
	}

	public void setSequenceCreated(boolean sequencecreated) {
		this.created = sequencecreated;
	}

	public boolean isSequenceCreated() {
		return created;
	}

	public void setSequenceName(String seqName) {
		this.seqName = seqName;
	}

	public String getSequenceName() {
		return seqName;
	}

	public RdbAdapter getAdapter() {
		return adapter;
	}

	public void setAdapter(RdbAdapter adapter) {
		this.adapter = adapter;
	}

	public Integer getSequenceStart() {
		return seqStart;
	}

	public void setSequenceStart(Integer seqStart) {
		this.seqStart = seqStart;
	}

	public Integer getSequenceIncrement() {
		return seqIncrement;
	}

	public void setSequenceIncrement(Integer seqIncrement) {
		this.seqIncrement = seqIncrement;
	}

	public Integer getSequenceCache() {
		return seqCache;
	}

	public void setSequenceCache(Integer seqCache) {
		this.seqCache = seqCache;
	}

	public Logger getLogger() {
		return logger;
	}

	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	/**
	 * Create the sql sequence on the relational database
	 * @param c is the connection for the acces to the database
	 * @return true if the sequence has been create, false if the sequence
	 * already exists.
	 * @throws PException if an error occurs during the SQL request
	 */
	public synchronized boolean createSequence(Object c) throws PException {
		if (created) {
			return false;
		}
		if (seqName == null) {
			throw new PException("No sequence name specified");
		}
		if (adapter == null) {
			throw new PException("No RdbAdapter specified");
		}
		Connection conn = RdbConnectionWrapper.narrow2SQL(c);
		created = true;
		Statement s = null;
		try {
			if (!adapter.existSequence(conn, seqName)) {
				s = conn.createStatement();
				s.execute(adapter.getCreateSequence(
					seqName, seqStart, seqIncrement, seqCache));
				return true;
			} else {
				return false;
			}
		} catch (SQLException e) {
			created = false;
			throw new PExceptionIO(e, "Impossible to create the sequence '"
				+ seqName + "'.");
		} finally {
			if (s != null) {
				try {
					s.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	public long allocateId(Object c) throws PException {
		if (nextValRequest == null) {
			if (seqName == null) {
				throw new PException("No sequence name specified");
			}
			if (adapter == null) {
				throw new PException("No RdbAdapter specified");
			}
			if (!created) {
				throw new PException("Protocol Error: You have to ask the sequence creation before id allocation");
			}
			nextValRequest = adapter.getNextValInSequence(seqName);
		}
		Connection conn = RdbConnectionWrapper.narrow2SQL(c);
		Statement s = null;
		ResultSet rs = null;
		long res = -1;
		try {
			s = conn.createStatement();
			rs = s.executeQuery(nextValRequest);
			if (rs.next()) {
				res = rs.getLong(1);
			} else {
				throw new PExceptionIO("Strong case: The nextval operation does " +
					"not return any value!");
			}
		} catch (SQLException e) {
			throw new PExceptionIO(e, "Impossible to allocate a new identifier"
				+ " on the sequence: " + seqName);
		} finally {
			try {
				if (rs != null) {
					rs.close();
				}
				if (s != null) {
					s.close();
				}
			} catch (SQLException e) {
			}
		}
		return res;
	}
}
