/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.api;

import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.api.JormCompilerConfigurator;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.jorm.util.io.api.PathExplorer;
import org.objectweb.jorm.api.PException;

/**
 * Generation is a simple interface which allows to generate object file
 * depending on their nature. The compiler uses Meta Object to build the
 * input source tree.
 *
 * @author	P. Dechamboux
 */
public interface Generator {
	/**
	 * Initializes the generator. Should be called after setCompilerParameter
	 * and setCompilerConfigurator.
	 * @throws PException
	 */
	void init() throws PException;

	/**
	 * Assigns the compiler parameter to this generator.
	 * @param jcp   the compiler parameter.
	 */
	void setCompilerParameter(JormCompilerParameter jcp);

	/**
	 * Assigns the compiler configurator to this generator.
	 * @param jcc   the compiler configurator.
	 */
	void setCompilerConfigurator(JormCompilerConfigurator jcc);

	/**
	 * Assigns the meta information manager to this generator.
	 * @param mim   the meta information manager to use.
	 */
	void setMetaInfoManager(Manager mim);

	/**
	 * Assigns a PathExplorer object for locating files that have to be parsed.
	 * @param pathexpl	the PathExplorer to be used for file location
	 */
	void setPathExplorer(PathExplorer pathexpl);

	/**
	 * Adds a mapping generator which generates a specific mapping part.
	 * @param mappername		The name of the mapper.
	 * @param mappinggenerator	The Mapping Verifier object.
	 */
	void addMappingGenerator(String mappername, MOPFactory mappinggenerator);

	/**
	 * Generates code for the given Class.
	 * @param mo	The class meta-object.
	 * @param th	The target holder which allows to create files.
	 * @param cp	This parameter permits to reach the compilation parameters.
	 * @param jcc	The actual configuration of the compiler.
	 */
	void generate(Class mo, TargetHolder th, JormCompilerParameter cp, JormCompilerConfigurator jcc) throws PException;

	/**
	 * Generates code for the given CompositeName.
	 * @param mo	The composite name meta-object.
	 * @param th	The target holder which allows to create files.
	 * @param cp	This parameter permits to reach the compilation parameters.
	 */
	void generate(CompositeName mo, TargetHolder th, JormCompilerParameter cp) throws PException;
}
