/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.generator.lib;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.ClassProject;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.compiler.api.JormCompilerConfigurator;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.generator.api.MOP;
import org.objectweb.util.monolog.api.BasicLevel;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;

import java.io.FileWriter;
import java.io.File;
import java.util.Collection;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * Mapping generator for abstract classes
 *
 * @author S.Chassande-Barrioz
 */
public class AbstractMappingGenerator extends CommonGenerator {

	/**
	 * The mapper generation specializations (some MOP objects)
	 * key = mapper name
	 * value = a MOP
	 */
	protected HashMap mmops = new HashMap();

	/**
	 * This constrcutor permits to specify which BindingMOP must be used
	 */
	public AbstractMappingGenerator(JormCompilerConfigurator jcc,
									JormCompilerParameter cp) throws PException {
		super();
		Iterator it = jcc.knownMappers();
		while (it.hasNext()) {
			String mn = (String) it.next();
			mmops.put(mn, jcc.getMOPFactory(mn).createMappingMOP(cp, mn));
		}
	}

	public String[][] getTemplateLibraries() {
		String[][] res = new String[mmops.size()][];
		Iterator it = mmops.values().iterator();
		int i = 0;
		while (it.hasNext()) {
			res[i] = ((MOP) it.next()).getTemplateLibraries();
			i++;
		}
		return res;
	}

	public void generate(Class clazz,
						 TargetHolder holder,
						 JormCompilerParameter cp,
						 JormCompilerConfigurator jcc) throws PException {
		if (!clazz.isAbstract()) {
			return;
		}
		Collection nds = clazz.getNameDefs();
		Class classWithNd = clazz;
		while(nds.size() < 1 && classWithNd.getInheritedClassNumber() > 0) {
			classWithNd = (Class) classWithNd.getSuperClasses().iterator().next();
			nds = classWithNd.getNameDefs();
		}
		if (nds.size() < 1) {
			logger.log(BasicLevel.WARN, "AbstractMapping: No NameDef defined for the class " + clazz.getFQName());
			return;
		}
		if (nds.size() == 1) {
			generate(clazz, holder, cp, (NameDef) nds.iterator().next(), null);
		} else {
			Iterator it = nds.iterator();
			while(it.hasNext()) {
				NameDef nd = (NameDef) it.next();
				generate(clazz, holder, cp, nd,
				    (nd.getName() != null && nd.getName().length() == 0
				    ? null
				    : nd.getName()));
			}
		}

	}
	public void generate(Class clazz,
	                     TargetHolder holder,
	                     JormCompilerParameter cp,
	                     NameDef nd,
	                     String ndName) throws PException {

		String packName = ((Package) clazz.getParent()).getName();
		// File name
		String fileName = clazz.getName() + PMapper.PCLASSMAPPINGAPPENDER;
		if (ndName != null) {
			if (packName != null && packName.length() > 0) {
				packName += "." + ndName;
			} else {
				packName = ndName;
			}
		}
		if (packName.length() > 0) {
			fileName = (packName + '.').replace('.', File.separatorChar) + fileName;
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName + " class");
		Context ctx = new VelocityContext();
		ctx.put("class", clazz);
		if (packName.length() > 0) {
			ctx.put("package", packName);
		}
		ctx.put("cparam", cp);
		CommonHelper ch = new CommonHelper();
		ch.setLogger(logger);
		ctx.put("tools", ch);
		ctx.put("mappername", "");
		ctx.put("noBindingCreation", Boolean.TRUE);
		ctx.put("abstractMapping", Boolean.TRUE);
		ctx.put("header", GEN_TEMPLATE_DIR + "Header.vm");
		ctx.put("mappingTools", this);
		ClassProject classproject = clazz.getClassProject(cp.getProjectName());
		ArrayList dependencies = new ArrayList();
		if (classproject != null) {
			Collection ms = classproject.getMappings();
			if (!ms.isEmpty()) {
				Iterator it = ms.iterator();
				while(it.hasNext()) {
					Mapping m = (Mapping) it.next();
					ClassMapping cm = m.getClassMapping();
					if (cm != null) {
						dependencies.addAll(cm.getDependencies());
					}
				}
			}
		}
		//mmop.initContext(ctx, clazz, cp.getProjectName(), null);
		ctx.put("dependencies", dependencies);
		try {
			// Fetch a block with the fileName
			FileWriter fw = holder.getFileWriter(fileName + ".java");
			velocityEngine.getTemplate(GEN_TEMPLATE_DIR + "Mapping.vm").merge(ctx, fw);
			fw.flush();
			fw.close();
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Problem while writing Mapping java file: " + fileName + ".java");
		}

	}
}
