/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.Collection;
import java.util.HashMap;
import java.util.ArrayList;

/**
 * This class is a generator of XXXBinder objects. It is generic either the
 * mapper type. This generator use the velocity tools. The used template is
 * Binder.vm . Associated to this template this generator builds a velocity
 * context which contains the following information:<br/>
 * <table border=1>
 * 	<tr><td>Key</td><td>Value</td></tr>
 * 	<tr><td>"compositename"</td><td>The reference to the compositename meta object</td></tr>
 * 	<tr>
 * 		<td>"tools"</td>
 * 		<td>The reference to the <a href="CommonHelper.html">CommonHelper<a/></td>
 * </tr>
 * 	<tr>
 * 		<td>"header"</td>
 * 		<td>The file name of the common template which contains the header of
 * 			the generate files.</td>
 * </tr>
 * </table>
 */
public class BinderGenerator extends CommonGenerator {

	HashMap autoCalculatedFieldInfos;

	/**
	 * This method generates a XPBinder file corresponding to the co
	 * parameter in the directory parameter.
	 * @param co		The meta object interface.
	 * @param holder	The target holder which allows to create files.
	 * @param cp		This parameter permits to reach the compilation parameters.
	 */
	public void generate(CompositeName co,
						 TargetHolder holder,
						 JormCompilerParameter cp) throws PException {
		// Calculate fileName
		String packName = ((Package) co.getParent()).getName();
		String fileName = co.getName() + "Binder";
		if (packName != null && packName.length() > 0) {
			fileName = packName + "." + fileName;
			fileName = fileName.replace('.', File.separatorChar);
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName + " Binder");
		// Fetch a block with the fileName
		Context ctx = new VelocityContext();
		ctx.put("compositename", co);
		ctx.put("tools", this);
		ctx.put("bindertools", this);
		ctx.put("header", GEN_TEMPLATE_DIR + "Header.vm");
		Collection allFields = co.getAllFields();
		Iterator it = allFields.iterator();
		PrimitiveElement firstField = (PrimitiveElement) it.next();
		//If the composite name is composed by an unique field that the type is
		// different to String, specify a uniqueField in order to support the
		// direct coding. The String coding is always supported !!
		if (!it.hasNext()) {
			ctx.put("uniqueField", firstField);
			if (firstField.getType().getTypeCode() != PType.TYPECODE_STRING) {
				ctx.put("defaultStringEncoding", "toto");
			}
		} else {
			ctx.put("defaultStringEncoding", "toto");
		}

		//Auto calculated field.
		it = allFields.iterator();
		boolean allAutoCalculated = true;
		while (it.hasNext()) {
			PrimitiveElement pe = (PrimitiveElement) it.next();
			if (pe.isAutoCalculated()) {
				if (autoCalculatedFieldInfos == null) {
					autoCalculatedFieldInfos = new HashMap();
				}
				AutoCalculatedFieldInfo acfi = new AutoCalculatedFieldInfo();
				autoCalculatedFieldInfos.put(pe.getName(), acfi);
				acfi.pe = pe;
				acfi.name = pe.getName() + "JormLongGen";
				acfi.type = "org.objectweb.jorm.facility.naming.generator.LongGen";
				acfi.value = pe.getName() + "JormLongGen.genId($)";
				acfi.getter = "getACF" + upperFL(pe.getName());
				acfi.setter = "setACF" + upperFL(pe.getName());
				switch (pe.getType().getTypeCode()) {
				case PType.TYPECODE_INT:
					acfi.value = "(int) " + acfi.value;
					break;
				case PType.TYPECODE_OBJINT:
					acfi.value = "new Integer((int)" + acfi.value + ")";
					break;
				case PType.TYPECODE_LONG:
					//nothing more to do
					break;
				case PType.TYPECODE_OBJLONG:
					acfi.value = "new Long(" + acfi.value + ")";
					break;
				default:
					throw new PException(
						"Auto calculated field not supported for the field type: "
						+ pe.getType().getJavaName());
				}
			} else {
				allAutoCalculated = false;
			}
		}
		if (autoCalculatedFieldInfos != null) {
			ctx.put("autoCalculatedFieldInfos", autoCalculatedFieldInfos.values());
		}
		ctx.put("allAutoCalculated", Boolean.valueOf(allAutoCalculated));

		try {
			FileWriter fw = holder.getFileWriter(fileName + ".java");
			if (template == null) {
				template = velocityEngine.getTemplate(GEN_TEMPLATE_DIR + "Binder.vm");
			}
			template.merge(ctx, fw);
			fw.flush();
			fw.close();
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Problem while generating PBinder.");
		}
	}

	public boolean containsObject(CompositeName cn) throws PException {
		for (Iterator it = cn.getAllFields().iterator(); it.hasNext();) {
			if (canBeNullValue(
					((ScalarField) it.next()).getType())) {
				return true;
			}
		}
		return false;
	}

	public boolean containsPrimitive(CompositeName cn) throws PException {
		for (Iterator it = cn.getAllFields().iterator(); it.hasNext();) {
			if (!canBeNullValue(
					((ScalarField) it.next()).getType())) {
				return true;
			}
		}
		return false;
	}

	public String getNullValue(PrimitiveElement pe) {
		if (isObjectType(pe.getType())) {
			return "null";
		} else if (pe.getType().getTypeCode() == PType.TYPECODE_CHAR) {
			return "'0'";
		} else {
			return "(" + pe.getType().getJavaName() + ") -1";
		}
	}

	public String getPNameFromSpecificPNG(CompositeName cn,
										  String pngVar,
										  String ctxVar) throws PException {
		return getPNameFromSpecificPNG(cn, pngVar, ctxVar, null);
	}
	public String getPNameFromSpecificPNG(CompositeName cn,
										  String pngVar,
										  String ctxVar,
										  String connVar) throws PException {
		StringBuffer result = new StringBuffer();
		result.append("new ");
		result.append(cn.getName());
		result.append("PName(");
		Iterator fs = cn.getAllFields().iterator();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			if (connVar!= null && f.isAutoCalculated()) {
				result.append(((AutoCalculatedFieldInfo)
					autoCalculatedFieldInfos.get(f.getName()))
					.getValue(connVar));
				result.append(", ");
			} else {
				result.append("((");
				result.append(cn.getName());
				result.append("PNG)");
				result.append(pngVar);
				result.append(").pnGet");
				result.append(upperFL(f.getName()));
				result.append("(");
				result.append(ctxVar);
				result.append("), ");
			}
		}
		result.append(" this)");
		return result.toString();
	}

	public String getPNameFromGenericPNG(CompositeName cn,
										 String pngVar,
										 String ctxVar) throws PException {
		return getPNameFromGenericPNG(cn, pngVar, ctxVar, null);
	}

	public String getPNameFromGenericPNG(CompositeName cn,
										 String pngVar,
										 String ctxVar,
										 String connVar) throws PException {
		StringBuffer result = new StringBuffer();
		result.append("new ");
		result.append(cn.getName());
		result.append("PName(");
		Iterator fs = cn.getAllFields().iterator();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			if (connVar!= null && f.isAutoCalculated()) {
				result.append(((AutoCalculatedFieldInfo)
					autoCalculatedFieldInfos.get(f.getName()))
					.getValue(connVar));
				result.append(", ");
			} else {
				result.append("((PNameGetter)");
				result.append(pngVar);
				result.append(").");
				result.append(getPNameGetterGetFunction(f.getType()));
				result.append("(\"");
				result.append(f.getName());
				result.append("\", ");
				result.append(ctxVar);
				result.append("), ");
			}
		}
		result.append(" this)");
		return result.toString();
	}

	public String getTestNullValues(CompositeName cn, boolean generic, String ctx)
			throws PException {
		StringBuffer res = new StringBuffer("");
		String sep = "";
		Iterator fs = cn.getAllFields().iterator();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			res.append(sep);
			sep = " && ";
			res.append("(");
			String v1;
			if (generic) {
				v1 = "png." + getPNameGetterGetFunction(f.getType()) + "(\""
					+ f.getName() + "\", " + ctx + ")";
			} else {
				v1 = "png.pnGet" + upperFL(f.getName() + "(" + ctx + ")");
			}
			res.append(comparePE(v1, "jorm" + f.getName(), f.getType()));
			res.append(")");
		}
		return res.toString();
	}

	public String getCoderSetter(PType t) {
		if (t.getTypeCode() == PType.TYPECODE_BYTEARRAY) {
			return "putByteArray(";
		} else {
			return "put" + getCoderName(t) + "(";
		}
	}

	public String getCoderGetter(PType t) {
		if (t.getTypeCode() == PType.TYPECODE_BYTEARRAY) {
			return "getByteArray()";
		} else {
			return "get" + getCoderName(t) + "()";
		}
	}

	public class AutoCalculatedFieldInfo {
		public PrimitiveElement pe;
		public String type;
		public String name;
		private String value;
		public String getter;
		public String setter;

		public String getType() {
			return type;
		}

		public String getName() {
			return name;
		}

		public String getGetter() {
			return getter;
		}

		public String getSetter() {
			return setter;
		}

		public String getValue(String connVar) {
			int idx = value.indexOf('$');
			if (idx == -1) {
				return value;
			} else {
				return value.substring(0, idx) + connVar
					+  value.substring(idx + 1);
			}
		}
	}
}
