/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import java.io.File;
import java.io.FileWriter;
import java.util.Collection;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.JormCompilerConfigurator;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class is a generator of XXXBinding objects. This generator use the 
 * velocity tools. The used template is Binding.vm . Associated to this template
 * this generator builds a velocity context which contains the following
 * information:<br/>
 * <table border=1>
 * 	<tr><td>Key</td><td>Value</td></tr>
 * 	<tr><td>"class"</td><td>The reference to the Class meta object</td></tr>
 * 	<tr><td>"classND"</td><td>The reference to the used NameDef</td></tr>
 * 	<tr><td>"tools"</td>
 * 		<td>The reference to the <a href="CommonHelper.html">CommonHelper<a/></td>
 * </tr>
 * 	<tr><td>"header"</td>
 * 		<td>The file name of the common template which contains the header of
 * 			the generate files.</td>
 * </tr>
 * 	<tr><td>"cparam"</td>
 * 		<td>The reference to a CompilerParameter instance. This instance
 * 		describes the user compilation parameter.</td>
 * </tr>
 * </table>
 */
public class BindingGenerator extends CommonGenerator {

	/**
	 * This method generates a XPBinding file corresponding to the pod
	 * parameter in the directory parameter
	 * @param clazz		the class meta object interface
	 * @param holder	The target holder which allows to create files
	 * @param cp		This parameter permits to reach the compilation parameters
	 * @param jcc		the JormCompilerConfigurator
	 */
	public void generate(Class clazz, 
			TargetHolder holder,
			JormCompilerParameter cp,
			JormCompilerConfigurator jcc) throws PException {
		String pack = ((Package) clazz.getParent()).getName();
		// File name
		String fileName = clazz.getName() + "Binding";
		if (pack.length() > 0) {
			fileName = (pack + '.').replace('.', File.separatorChar) + fileName;
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName + " class");
		Collection nds = clazz.getNameDefs();
		Class classWithNd = clazz;
		while(nds.size() < 1 && classWithNd.getInheritedClassNumber() > 0) {
			classWithNd = (Class) classWithNd.getSuperClasses().iterator().next();
			nds = classWithNd.getNameDefs();
		}
		if (nds.size() < 1) {
			logger.log(BasicLevel.WARN, "Binding generation: No NameDef defined for the class " + clazz.getFQName());
			return;
		}
		NameDef nd = (NameDef) nds.iterator().next();
		Context ctx = new VelocityContext();
		ctx.put("class", clazz);
		ctx.put("classND", nd);
		if (pack.length() > 0) {
			ctx.put("package", pack);
		}
		ctx.put("cparam", cp);
		ctx.put("tools", this);
		ctx.put("header", GEN_TEMPLATE_DIR + "Header.vm");
		try {
			FileWriter fw = holder.getFileWriter(fileName + ".java");
			velocityEngine.getTemplate(GEN_TEMPLATE_DIR + "Binding.vm").merge(ctx, fw);
			fw.flush();
			fw.close();
		} catch (Exception e) {
			throw new PExceptionCompiler(e,
					"Problem while writing Binding java file: " 
					+ e.getMessage());
		}
	}
}
