/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.apache.velocity.app.Velocity;
import org.apache.velocity.app.VelocityEngine;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.JormCompilerConfigurator;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.generator.api.Generator;
import org.objectweb.jorm.generator.api.MOPFactory;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.util.io.api.PathExplorer;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.jorm.util.io.lib.DirJavaExplorer;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.wrapper.velocity.VelocityLogger;

import java.io.File;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * This class start the generations of the jorm objects and user object. The
 * jorm objects are the mapping, the binding and the accessor. It is needed to
 * specify a MOPFactory before start the generation.The generate method
 * starts the generation. To generate user objects it is necessary to specify
 * their GeneratorFactory by the addGeneratorFactory method.
 */
public class JormGenerator implements Generator, Loggable {
	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_DESCRIPTION_PROP
			= "class.resource.loader.description";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_DESCRIPTION_VALUE
			= "Velocity Classpath Resource Loader";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_CLASS_PROP
			= "class.resource.loader.class";

	/**
	 * Velocity constant.
	 */
	private static final String CLASS_RESOURCE_LOADER_CLASS_VALUE
			= "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader";

	private Manager metaInfoManager = null;
	private Logger logger = null;
	private LoggerFactory loggerFactory = null;
	/**
	 * The mopFactories which specialize jorm for different supports
	 */
	private HashMap mopFactories = new HashMap();
	private PathExplorer pathExplorer = null;

	private AccessorGenerator accessorGenerator = null;
	private BindingGenerator bindingGenerator = null;
	private MappingGenerator mappingGenerator = null;
	private AbstractMappingGenerator abstractMappingGenerator = null;
	private SerializableGenerator serializableGenerator = null;
	private PNameGetterGenerator pNameGetterGenerator = null;
	private PNameGenerator pNameGenerator = null;
	private BinderGenerator binderGenerator = null;
	private FPNCGenerator fpncGenerator = null;
	private KFPNCGenerator kfpncGenerator = null;
	private VelocityEngine velocityEngine = null;

	private JormCompilerParameter compilerParameter = null;
	private JormCompilerConfigurator compilerConfigurator = null;

	/**
	 * Initializes the generator. Should be called after setCompilerParameter
	 * and setCompilerConfigurator.
	 * @throws PException
	 */
	public void init() throws PException {
		velocityEngine = new VelocityEngine();
		velocityEngine.setProperty(Velocity.RESOURCE_LOADER, "class");
		velocityEngine.setProperty(CLASS_RESOURCE_LOADER_DESCRIPTION_PROP,
								   CLASS_RESOURCE_LOADER_DESCRIPTION_VALUE);
		velocityEngine.setProperty(CLASS_RESOURCE_LOADER_CLASS_PROP,
								   CLASS_RESOURCE_LOADER_CLASS_VALUE);
		// Connect the velocity log system to monolog
		velocityEngine.setProperty(Velocity.RUNTIME_LOG_LOGSYSTEM, new VelocityLogger(
				(loggerFactory == null
				 ? logger
				 : loggerFactory.getLogger(logger.getName() + ".velocity"))
		));
		StringBuffer strlibs = new StringBuffer();
		String libs[][][];
		libs = setupClassGenerators(compilerConfigurator, compilerParameter);
		String sep = "";
		for (int i = 0; i < libs.length; i++) {
			for (int j = 0; j < libs[i].length; j++) {
				for (int k = 0; k < libs[i][j].length; k++) {
					strlibs.append(sep);
					sep = ",";
					strlibs.append(libs[i][j][k]);
					logger.log(BasicLevel.DEBUG, "VM Library: " + libs[i][j][k]);
				}
			}
		}
		libs = setupCompositeNameGenerators();
		for (int i = 0; i < libs.length; i++) {
			for (int j = 0; j < libs[i].length; j++) {
				for (int k = 0; k < libs[i][j].length; k++) {
					strlibs.append(sep);
					sep = ",";
					strlibs.append(libs[i][j][k]);
					logger.log(BasicLevel.DEBUG, "VM Library: " + libs[i][j][k]);
				}
			}
		}
		velocityEngine.setProperty(Velocity.VM_LIBRARY, strlibs.toString());
		try {
			logger.log(BasicLevel.DEBUG, "initializes the velocity engine");
			velocityEngine.init();
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Problem while initializing velocity");
		}
	}

	private String[][][] setupClassGenerators(JormCompilerConfigurator jcc, JormCompilerParameter cp) throws PException {
		logger.log(BasicLevel.DEBUG, "setupClassGenerators");
		Map generators = new HashMap();
		if (accessorGenerator == null) {
			accessorGenerator = new AccessorGenerator();
			generators.put("accessor", accessorGenerator);
		}
		if (bindingGenerator == null) {
			bindingGenerator = new BindingGenerator();
			generators.put("binding", bindingGenerator);
		}
		if (mappingGenerator == null) {
			mappingGenerator = new MappingGenerator(jcc, cp);
			generators.put("mapping", mappingGenerator);
		}
		if (serializableGenerator == null) {
			serializableGenerator = new SerializableGenerator();
			generators.put("serializable", serializableGenerator);
		}

		if (fpncGenerator == null) {
			fpncGenerator = new FPNCGenerator();
			generators.put("fpnc", fpncGenerator);
		}

		if (kfpncGenerator == null) {
			kfpncGenerator = new KFPNCGenerator();
			generators.put("kfpnc", kfpncGenerator);
		}

		if (abstractMappingGenerator == null) {
			abstractMappingGenerator = new AbstractMappingGenerator(jcc, cp);
			generators.put("abstractmapping", abstractMappingGenerator);
		}
		return setupGenerators(generators);
	}

	private String[][][] setupCompositeNameGenerators() {
		logger.log(BasicLevel.DEBUG, "setupCompositeNameGenerators");
		Map generators = new HashMap();
		if (pNameGetterGenerator == null) {
			pNameGetterGenerator = new PNameGetterGenerator();
			generators.put("pnamegetter", pNameGetterGenerator);
		}
		if (pNameGenerator == null) {
			pNameGenerator = new PNameGenerator();
			generators.put("pname", pNameGenerator);
		}
		if (binderGenerator == null) {
			binderGenerator = new BinderGenerator();
			generators.put("binder", binderGenerator);
		}
		return setupGenerators(generators);
	}

	private String[][][] setupGenerators(Map generators) {
		int size = generators.size();
		String[][][] templateLibs = new String[size][][];
		Iterator it = generators.entrySet().iterator();
		int i = 0;
		while(it.hasNext()) {
			Map.Entry me = (Map.Entry) it.next();
			CommonGenerator gen = (CommonGenerator) me.getValue();
			gen.setVelocityEngine(velocityEngine);
			if (loggerFactory != null) {
				gen.setLogger(
						loggerFactory.getLogger(logger.getName() + "." + me.getKey()));
				gen.setLoggerFactory(loggerFactory);
			} else {
				gen.setLogger(logger);
			}
			templateLibs[i] = gen.getTemplateLibraries();
			i++;
		}
		return templateLibs;
	}

	private JormCompilerParameter overloadCompilerParameter(String filename, JormCompilerParameter jcp) {
		logger.log(BasicLevel.DEBUG, "overloadCompilerParameter");
		JormCompilerParameter res;
		File fn = new File(filename);
		try {
			logger.log(BasicLevel.DEBUG, "look for parameter file: " + fn.getParent() + File.separator + "jormcOpts.properties");
			res = (JormCompilerParameter) jcp.clone();
			InputStream is = pathExplorer.getInputStream(fn.getParent() + File.separator + "jormcOpts.properties");
			if (is == null) {
				logger.log(BasicLevel.DEBUG, "no overloading");
				res = jcp;
			} else {
				res.loadConfFile(is, compilerConfigurator.knownMappers());
				logger.log(BasicLevel.DEBUG, "parameter overloaded");
			}
		} catch (PException e) {
			logger.log(BasicLevel.DEBUG, "no overloading");
			res = jcp;
		}
		return res;
	}

	// IMPLEMENTATION OF METHODS FROM THE Generator INTERFACE

	/**
	 * Assigns the compiler parameter to this generator.
	 * @param jcp   the compiler parameter.
	 */
	public void setCompilerParameter(JormCompilerParameter jcp) {
		compilerParameter = jcp;
	}

	/**
	 * Assigns the compiler configurator to this generator.
	 * @param jcc   the compiler configurator.
	 */
	public void setCompilerConfigurator(JormCompilerConfigurator jcc) {
		compilerConfigurator = jcc;
	}

	/**
	 * Assigns the meta information manager to this generator.
	 * @param mim   the meta information manager to use.
	 */
	public void setMetaInfoManager(Manager mim) {
		metaInfoManager = mim;
	}

	/**
	 * Assigns a PathExplorer object for locating files that have to be parsed.
	 *
	 * @param pathexpl	the PathExplorer to be used for file location
	 */
	public void setPathExplorer(PathExplorer pathexpl) {
		pathExplorer = pathexpl;
		if (pathExplorer == null) {
			pathExplorer = new DirJavaExplorer();
		}
		logger.log(BasicLevel.DEBUG, "jorm classpath=" + pathExplorer.getClassPath());
	}

	/**
	 * Adds a mapping generator which generates a specific mapping part.
	 * @param mappername		The name of the mapper.
	 * @param mappinggenerator	The Mapping Verifier object.
	 */
	public void addMappingGenerator(String mappername, MOPFactory mappinggenerator) {
		mopFactories.put(mappername, mappinggenerator);
	}

	/**
	 * This method generates the jorm objects by delegation to the
	 * AccessorGenerator, the BindingGenerator and the MappingGenerator. Its
	 * second task is to launch the user objects generator.
	 * @param mo		This meta object is a Class.
	 * @param holder	The target holder which allows to create files.
	 * @param cp		The compiler parameter.
	 * @exception PException if an error occurs during the generation.
	 */
	public void generate(Class mo, TargetHolder holder,
						 JormCompilerParameter cp, JormCompilerConfigurator jcc)
			throws PException {
		JormCompilerParameter ocp = overloadCompilerParameter(mo.getFileName(), cp);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
		logger.log(BasicLevel.DEBUG, "class generation: " + mo.getFQName()
				+ "\nfilename=[" + mo.getFileName() + "]"
				+ "\nClassMappingInheritance = " + ocp.getClassMappingInheritance()
				+ "\nBindingInheritance = " + ocp.getBindingInheritance()
				+ "\nProject Name = " + ocp.getProjectName());
		}
		bindingGenerator.generate(mo, holder, ocp, jcc);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, mo.getName() + "Bindings generated");
		}
		mappingGenerator.generate(mo, holder, ocp, jcc);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, mo.getName() + "Mappings generated");
		}
		fpncGenerator.generate(mo, holder, ocp);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, mo.getName() + "FPNCs generated");
		}
		try {
			if (mo.generateKFPNC()) {
				kfpncGenerator.generate(mo, holder, ocp);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, mo.getName() + "KFPNCs generated");
				}
			}
		} catch (ExpressionException e) {
			throw new PException("Problem with call to generateKFNC() method." + e.getMessage());
		}
		// generate the accessor
		accessorGenerator.generate(mo, holder, ocp);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, mo.getName() + "Accessor generated");
		}
		//generate the serializable data
		serializableGenerator.generate(mo, holder, ocp);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, mo.getName() + "SerializableData generated");
		}
	}

	/**
	 * This method generates the jorm objects by delegation to the
	 * PNameGenerator, BinderGenerator and CompositeNameGenerator.
	 * @param mo		This meta object is a CompositeName.
	 * @param holder	The target holder which allows to create files.
	 * @param cp		The compiler parameter.
	 * @exception PException if an error occurs during the generation.
	 */
	public void generate(CompositeName mo,
						 TargetHolder holder,
						 JormCompilerParameter cp) throws PException {
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, "composite name generation: " + mo.getFQName());
		}
		// generate the pnamegetter
		pNameGetterGenerator.generate(mo, holder, cp);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, "XXXpnamegetter generated");
		}
		// generate the pname
		pNameGenerator.generate(mo, holder, cp);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, "XXXpname generated");
		}
		// generate the binder
		binderGenerator.generate(mo, holder, cp);
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, "XXXbinder generated");
		}
	}

	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}
}
