/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.generator.lib;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.naming.lib.MapPNameGetter;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.io.File;
import java.io.FileWriter;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class KFPNCGenerator extends CommonGenerator {
	public void generate(Class clazz,
	                     TargetHolder holder,
	                     JormCompilerParameter cp) throws PException {

		//Generate only for the ancestor class
		if (!clazz.getSuperClasses().isEmpty()) {
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "KFPNC: No generation done because "
					+ clazz.getFQName() + " is not the ancestor");
			}
			return;
		}

		Collection nds = clazz.getNameDefs();
		if (nds.size() < 1) {
			logger.log(BasicLevel.WARN, "FPNC: No NameDef defined for the class " + clazz.getFQName());
			return;
		}
		if (nds.size() == 1) {
			NameDef nd = (NameDef) nds.iterator().next();
			generate(clazz, nd, null, holder);
		} else {
			Iterator it = nds.iterator();
			while(it.hasNext()) {
				NameDef nd = (NameDef) it.next();
				String ndName = (nd.getName() != null
				    && nd.getName().length() == 0
				    ? null
				    : nd.getName());
				generate(clazz, nd, ndName, holder);
			}
		}
	}

	/**
	 * This method generates a XFPNC file corresponding to the clazz
	 * parameter in the directory parameter.
	 * @param clazz		The clazz requiring the generation of a PNamingContext.
	 * @param nd		The NameDef to use
	 * @param ndName    The name def name to use fot the package name
	 * (can be null)
	 * @param holder	The target holder which allows to create files.
	 */
	private void generate(Class clazz,
	                      NameDef nd,
	                      String ndName,
	                      TargetHolder holder) throws PException {
		if (!mustGeneratedKFPNC(clazz, nd)) {
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "KFPNC: No generation done because "
					+ clazz.getFQName() + " has no inheritance filter based on keys");
			}
			return;
		}
		Expression exp;
		try {
			exp = clazz.getInheritanceFilter(nd);
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		if (exp == null) {
			logger.log(BasicLevel.WARN, "KFPNC: No generation done because "
					+ clazz.getFQName() + " has no inheritance filter");
			return;
		}
		generate(clazz, nd, ndName, exp, holder);
	}


	/**
	 * This method generates a XFPNC file corresponding to the clazz
	 * parameter in the directory parameter.
	 * @param clazz		The clazz requiring the generation of a PNamingContext.
	 * @param nd		The NameDef to use
	 * @param ndName    The name def name to use fot the package name
	 * (can be null)
	 * @param holder	The target holder which allows to create files.
	 */
	private void generate(Class clazz,
	                      NameDef nd,
	                      String ndName,
						  Expression exp,
	                      TargetHolder holder) throws PException {
		// Calculate fileName
		String packName = ((Package) clazz.getParent()).getName();
		if (ndName != null) {
			if (packName != null && packName.length() > 0) {
				packName += "." + ndName;
			} else {
				packName = ndName;
			}
		}
		String className = clazz.getName() + "KFPNC";
		String fileName = className;
		if (packName != null && packName.length() > 0) {
			fileName = packName + "." + fileName;
			fileName = fileName.replace('.', File.separatorChar);
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName);


		Context ctx = new VelocityContext();
		ctx.put("class", clazz);
		ctx.put("helper", this);
		ctx.put("header", GEN_TEMPLATE_DIR + "Header.vm");
		ctx.put("className", className);
		ctx.put("packName", packName);

		PType expressionType = exp.getType();
		ctx.put("expressionType", expressionType.getProgName());

		//nullValue & coding type
		String codingType;
		String nullValueStr;
		CompositeName cn = null;
		ctx.put("isComposite", nd.isNameRef() ? Boolean.TRUE : Boolean.FALSE);
		if (nd.isNameRef()) {
			cn = nd.getNameRef().getCompositeName();
			ctx.put("compositeName", cn);
			Collection fields = nd.getNameRef().getCompositeName().getAllFields();
			StringBuffer sb = new StringBuffer();
			sb.append("new ");
			sb.append(cn.getName());
			sb.append("PName(");
			Iterator it = fields.iterator();
			it = fields.iterator();
			while(it.hasNext()) {
				ScalarField sf = (ScalarField) it.next();
				String nullValue = sf.getNullValue();
				if (nullValue == null) {
					nullValue = getStrNullValue(sf.getType());
				}
				sb.append(nullValue);
				sb.append(", ");
			}
			sb.append("this)");
			nullValueStr = sb.toString();
			codingType = "CTCOMPOSITE";
		} else if (nd.isFieldName()) {
			TypedElement te = clazz.getTypedElement(nd.getFieldName());
			nullValueStr = getStrNullValueAsObject(te.getType());
			codingType = getCTDeclaration(te.getType());
		} else {
			return;
		}
		ctx.put("nullValue", nullValueStr);
		ctx.put("codingType", codingType);

		// generate the evaluate method
		NamingFilterExpressionHelper mi;
		try {
			mi = new NamingKeyFilterExpressionHelper(exp, cn, logger);
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		ctx.put("mi", mi);

		//Generate the file
		try {
			FileWriter fw = holder.getFileWriter(fileName + ".java");
			if (template == null) {
				template = velocityEngine.getTemplate(GEN_TEMPLATE_DIR + "KFPNC.vm");
			}
			template.merge(ctx, fw);
			fw.flush();
			fw.close();
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Problem while generating " + fileName + ".java");
		}
	}

	private String getStrNullValueAsObject(PType type) {
		StringBuffer sb = new StringBuffer();
		sb.append("new org.objectweb.jorm.facility.naming.basidir.BasidName(");
		sb.append(getStrAsObject(getStrNullValue(type), type));
		sb.append(", new org.objectweb.jorm.facility.naming.basidir.BasidBinder(");
		sb.append(getCTDeclaration(type));
		sb.append("))");
		return sb.toString();
	}

	private String getStrAsObject(String str, PType type) {
		switch (type.getTypeCode()) {
		case PType.TYPECODE_BYTE:
			return "new Byte(" + str +")";
		case PType.TYPECODE_CHAR:
			return "new Character(" + str +")";
		case PType.TYPECODE_SHORT:
			return "new Short(" + str +")";
		case PType.TYPECODE_INT:
			return "new Integer(" + str +")";
		case PType.TYPECODE_LONG:
			return "new Long(" + str +")";
		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_OBJCHAR:
		case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_OBJINT:
		case PType.TYPECODE_OBJLONG:
		case PType.TYPECODE_DATE:
		case PType.TYPECODE_CHARARRAY:
		case PType.TYPECODE_BYTEARRAY:
		case PType.TYPECODE_STRING:
		case PType.TYPECODE_BIGDECIMAL:
		case PType.TYPECODE_BIGINTEGER:
		default:
			return str;
		}
	}

	private String getStrNullValue(PType type) {
		switch (type.getTypeCode()) {
		case PType.TYPECODE_BYTE:
			return "(byte) -1";
		case PType.TYPECODE_CHAR:
			return "(char) 0";
		case PType.TYPECODE_SHORT:
			return "(short) -1";
		case PType.TYPECODE_INT:
			return "(int) -1";
		case PType.TYPECODE_LONG:
			return "(long) -1";
		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_OBJCHAR:
		case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_OBJINT:
		case PType.TYPECODE_OBJLONG:
		case PType.TYPECODE_DATE:
		case PType.TYPECODE_CHARARRAY:
		case PType.TYPECODE_BYTEARRAY:
		case PType.TYPECODE_STRING:
		case PType.TYPECODE_BIGDECIMAL:
		case PType.TYPECODE_BIGINTEGER:
			return "null";
		default:
			return null;
		}

	}

	private boolean mustGeneratedKFPNC(Class clazz, NameDef nd) {
		if (!clazz.getSuperClasses().isEmpty()) {
			logger.log(BasicLevel.DEBUG, "class " + clazz.getName() + " has parent => no KFPNC");
			return false;
		}
		try {
			if (clazz.getInheritanceFilter(nd) == null) {
				logger.log(BasicLevel.DEBUG, "class " + clazz.getName() + " has no filter => no KFPNC");
				return false;
			}
		} catch (ExpressionException e) {
			logger.log(BasicLevel.ERROR,
				"Error during the fetching of the key naming inheritance filter: ", e);
			return false;
		}
		return hasNamingKey(clazz, nd);
	}

	private boolean hasNamingKey(Class clazz, NameDef nd) {
		if (clazz.isAbstract()) {
			logger.log(BasicLevel.DEBUG, "class " + clazz.getName() + " is abstract");
			//looking for children
			Iterator it = clazz.getSubClasses().iterator();
			while(it.hasNext()) {
				Class subclazz = (Class) it.next();
				if (hasNamingKey(subclazz, nd)) {
					return true;
				}
			}
			return false;
		} else {
			Object key = clazz.getInheritanceNamingKey(nd);
			logger.log(BasicLevel.DEBUG, "class " + clazz.getName() + " key: " + key);
			return key != null;
		}
	}

	public class NamingKeyFilterExpressionHelper extends NamingFilterExpressionHelper {

		private PType filterType;
		private CompositeName co;
		public NamingKeyFilterExpressionHelper(Expression e,
											   CompositeName co,
											   Logger logger)
			throws PException, ExpressionException {
			super(logger);
			this.filterType = e.getType();
			this.co = co;
			fillMatchInfo(e);
		}

		public String getFilter() {
			return getStrAsObject(super.getFilter(), filterType);
		}

		protected void fillMatchInfo(ParameterOperand po) throws PException, ExpressionException {
			logger.log(BasicLevel.DEBUG, "ParameterOperand " + po.getName());
			if (field2declarations.get(po.getName()) == null) {
				StringBuffer declaration = new StringBuffer();
				declaration.append(po.getType().getJavaName());
				declaration.append(' ');
				declaration.append(po.getName());
				declaration.append(" = ");
				if (co == null) {
					declaration.append("((PNameGetter) png).");
					declaration.append(getPNameGetterGetFunction(po.getType()));
					declaration.append("(\"");
					declaration.append(po.getName());
					declaration.append("\", ctx)");
				} else {
					declaration.append("(isSpecific ? ((");
					declaration.append(co.getName());
					declaration.append("PNG) png).pnGet");
					declaration.append(upperFL(po.getName()));
					declaration.append("(ctx) : ((PNameGetter) png).");
					declaration.append(getPNameGetterGetFunction(po.getType()));
					declaration.append("(\"");
					declaration.append(po.getName());
					declaration.append("\", ctx))");
				}
				declaration.append(";");
				field2declarations.put(po.getName(), declaration);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, "Add declaration: " + declaration);
				}
			}
			filter.append(po.getName());
		}
	}


}
