/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.generator.lib;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.api.PExceptionCompiler;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.util.io.api.TargetHolder;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.File;
import java.io.FileWriter;
import java.util.Iterator;

/**
 * This class is a generator of XXXBinder objects. It is generic either the
 * mapper type. This generator use the velocity tools. The used template is
 * Binder.vm . Associated to this template this generator builds a velocity
 * context which contains the following information:<br/>
 * <table border=1>
 * 	<tr><td>Key</td><td>Value</td></tr>
 * 	<tr><td>"compositename"</td><td>The reference to the compositename meta object</td></tr>
 * 	<tr>
 * 		<td>"tools"</td>
 * 		<td>The reference to the <a href="CommonHelper.html">CommonHelper<a/></td>
 * </tr>
 * 	<tr>
 * 		<td>"header"</td>
 * 		<td>The file name of the common template which contains the header of
 * 			the generate files.</td>
 * </tr>
 * </table>
 */
public class PNameGenerator extends CommonGenerator {
	public CommonHelper helper = null;

	/**
	 * This method generates a XPName file corresponding to the pod
	 * parameter in the directory parameter.
	 * @param co		The meta object interface.
	 * @param holder	The target holder which allows to create files.
	 * @param cp		This parameter permits to reach the compilation parameters.
	 */
	public void generate(CompositeName co,
						 TargetHolder holder,
						 JormCompilerParameter cp) throws PException {
		// Calculate fileName
		String packName = ((Package) co.getParent()).getName();
		String fileName = co.getName() + "PName";
		if (packName != null && packName.length() > 0) {
			fileName = packName + "." + fileName;
			fileName = fileName.replace('.', File.separatorChar);
		}
		logger.log(BasicLevel.DEBUG, "Generate the " + fileName + " PName");
		Context ctx = new VelocityContext();
		ctx.put("compositename", co);
		helper = new CommonHelper();
		helper.setLogger(logger);
		ctx.put("tools", helper);
		ctx.put("nametools", this);
		Boolean hasca = Boolean.FALSE;
		Boolean hasba = Boolean.FALSE;
		StringBuffer constructParam = new StringBuffer();
		Iterator it = co.getAllFields().iterator();
		while (it.hasNext()) {
			TypedElement te = (TypedElement) it.next();
			int tc = te.getType().getTypeCode();
			if (tc == PType.TYPECODE_BYTEARRAY) {
				hasba = Boolean.TRUE;
			} else if (tc == PType.TYPECODE_CHARARRAY) {
				hasca = Boolean.TRUE;
			}
			constructParam.append(te.getType().getJavaName());
			constructParam.append(" ");
			constructParam.append(te.getName());
			constructParam.append(", ");
		}
		constructParam.append("PNameManager pnc");
		ctx.put("consparam", constructParam.toString());

		ctx.put("containsCharArray", hasca);
		ctx.put("containsByteArray", hasba);
		ctx.put("header", GEN_TEMPLATE_DIR + "Header.vm");
		try {
			// Fetch a block with the fileName
			FileWriter fw = holder.getFileWriter(fileName + ".java");
			if (template == null) {
				template = velocityEngine.getTemplate(GEN_TEMPLATE_DIR + "PName.vm");
			}
			template.merge(ctx, fw);
			fw.flush();
			fw.close();
		} catch (Exception e) {
			throw new PExceptionCompiler(e, "Problem while generating PName.");
		}
	}

	/**
	 * It compute the hashcode method for a composite pname.
	 */
	public String getCnHashCode(CompositeName cn) throws PException {
		String result = "0", plus = " + ";
		Iterator fs = cn.getAllFields().iterator();
		// compositename type can be : byte | char | short | int | long | string
		// tthe hashcode value for the pname is equal to the hashcode of the first field belonging to the pname.
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			result += plus;
			switch (f.getType().getTypeCode()) {
			case PType.TYPECODE_BYTE:
			case PType.TYPECODE_CHAR:
			case PType.TYPECODE_SHORT:
			case PType.TYPECODE_INT:
				result += f.getName();
				break;
			case PType.TYPECODE_LONG:
				result += "((int) " + f.getName() + ")";
				break;
			case PType.TYPECODE_OBJCHAR:
			case PType.TYPECODE_OBJBYTE:
			case PType.TYPECODE_OBJSHORT:
			case PType.TYPECODE_OBJINT:
			case PType.TYPECODE_OBJLONG:
			case PType.TYPECODE_DATE:
			case PType.TYPECODE_BIGDECIMAL:
			case PType.TYPECODE_BIGINTEGER:
			case PType.TYPECODE_STRING:
				result += "(" + f.getName() + " == null ? 0 :" + f.getName() + ".hashCode())";
				break;
			case PType.TYPECODE_BYTEARRAY:
			case PType.TYPECODE_CHARARRAY:
			default:
				result += "0";
			}
		}
		return result;
	}

	/**
	 * It compute the equality test for the equals method for pname.
	 */
	public String getCnEquals(CompositeName cn) throws PException {
		String result = "", and = " && ";
		boolean addAnd = false;
		Iterator fs = cn.getAllFields().iterator();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			if (addAnd) {
				result += and;
			} else {
				addAnd = true;
			}
			switch (f.getType().getTypeCode()) {
			case PType.TYPECODE_BYTE:
			case PType.TYPECODE_CHAR:
			case PType.TYPECODE_SHORT:
			case PType.TYPECODE_INT:
			case PType.TYPECODE_LONG:
				result += "(" + f.getName() + " == pn." + f.getName() + ")";
				break;
			case PType.TYPECODE_OBJBYTE:
			case PType.TYPECODE_OBJCHAR:
			case PType.TYPECODE_OBJSHORT:
			case PType.TYPECODE_OBJINT:
			case PType.TYPECODE_OBJLONG:
			case PType.TYPECODE_DATE:
			case PType.TYPECODE_BIGDECIMAL:
			case PType.TYPECODE_BIGINTEGER:
			case PType.TYPECODE_STRING:
				result += "(" + f.getName() + " == null" +
						" ? (pn." + f.getName() + "== null)" +
						" : " + f.getName() + ".equals(pn." + f.getName() + "))";
				break;
			case PType.TYPECODE_BYTEARRAY:
			case PType.TYPECODE_CHARARRAY:
				result += "equal(" + f.getName() + ", pn." + f.getName() + ")";
				break;
			default:
				throw new PExceptionTyping(
						"Unsupported PType: " + f.getType().getJormName());
			}
		}
		return result;
	}

	/**
	 * It compute the test used to know if a pname is null.
	 */
	public String getPNameNullTest(CompositeName cn) throws PException {
		String result = "";
		Iterator fs = cn.getAllFields().iterator();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			String nullvalue = getNullValue(cn, f);
			if (helper.canBeNullValue(f.getType())) {
				//To avoid NullPointerException we must check if the field name
				// is not null
				result += "(" + f.getName() + " == null"
						+ "? null == " + nullvalue
						+ ": " + f.getName() + ".equals(" + nullvalue + "))";
			} else {
				result += f.getName() + " == " + nullvalue;
			}
			if (fs.hasNext()) result += " && ";
		}
		return result;
	}

	/**
	 * It compute the constructor parameter used to instanciate a null pname.
	 */
	public String getPNameNullValue(CompositeName cn, String pncVal) throws PException {
		String result = "";
		Iterator fs = cn.getAllFields().iterator();
		while (fs.hasNext()) {
			ScalarField f = (ScalarField) fs.next();
			result = result + getNullValue(cn, f) + ", ";
		}
		return result + pncVal;
	}

	public String getNullValue(CompositeName cn, ScalarField f)
			throws PException {
		String nullValue = f.getNullValue();
		if (nullValue == null) {
			nullValue = (helper.canBeNullValue(f.getType())
					? "NULLOVALUE"
					: "NULLVALUE");
		} else {
			if (f.getType().getTypeCode() == PType.TYPECODE_STRING)
				nullValue = "\"" + nullValue + "\"";
			else if (f.getType().getTypeCode() == PType.TYPECODE_CHAR)
				nullValue = "\'" + nullValue + "\'";
		}
		return "(" + f.getType().getJavaName() + ")" + nullValue;
	}
}
