/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.lib;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Iterator;

import org.objectweb.jorm.api.ClassMappedEvent;
import org.objectweb.jorm.api.JormConfigurator;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapCluster;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMapperListener;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PMappingRequiredEvent;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.lib.PTypeSpacePAAH;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * @author	P. Dechamboux
 */
public abstract class Mapper implements PMapper, Loggable, PMappingCallback {
	/**
	 * This is the initial size of the map used to store the mapped classes.
	 */
	static final int INITIALMAPSIZE = 100;
	/**
	 * This is the name of the kind of mapper used (e.g., file, rdb, etc).
	 */
	String mapperName = null;
	/**
	 * This is the Manager that manages the meta-information associated with
	 * classes thnat are mapped within this mapper. It may stay null.
	 */
	Manager metainfoManager = null;
	/**
	 * This is the Parser that is used to load the mate-information associated
	 * with classes thnat are mapped within this mapper. It may stay null.
	 */
	Parser parser = null;
	/**
	 * This is the TypeSpace associated with this mapper. It is mandatory for
	 * mapping classes.
	 */
	PTypeSpace typeSpace = null;
	/**
	 * This is the structure that registers the classes that have been mapped
	 * within this mapper.
	 */
	HashMap mappedClasses = new HashMap(INITIALMAPSIZE);
	/**
	 * This is the optional prefetch cache that can be associated with a mapper.
	 * It is activated when the dedicated read operation is performed on
	 * bindings.
	 */
	PrefetchCache prefetchCache = null;

	/**
	 * The logger associated with this mapper.
	 */
	protected Logger logger = null;
	/**
	 * The logger factory associated with this mapper.
	 */
	protected LoggerFactory loggerFactory = null;

	/**
	 * The configurator used to configure this mapper.
	 */
	private JormConfigurator jormConfigurator = null;

	private Class gcmClass = null;

	PMappingStructuresManager pMappingStructuresManager = null;

	private ArrayList listenerList = new ArrayList();


	/**
	 * It builds a JormConfigurator with the default JormCompilerParameter and
	 * JormCompilerParameter.
	 */
	public Mapper() throws PException {
		jormConfigurator = new JormConfiguratorImpl();
		jormConfigurator.configure(PMapper.JORMPROPFILE);
		jormConfigurator.configureLog(PMapper.JORMPROPFILE);
	}

	public Mapper(JormConfigurator c) {
		jormConfigurator = c;
	}

    public void setJormConfigurator(JormConfigurator jormConfigurator) {
        this.jormConfigurator = jormConfigurator;
    }

	public JormConfigurator getJormConfigurator() {
		return jormConfigurator;
	}
	// IMPLEMENTATION OF METHODS FROM THE PMapingCallback INTERFACE

	/**
	 * Schedules an mapper-related event.
	 * @param e The event to be scheduled.
	 * @throws PException   Cannot schedule this event.
	 */
	public void scheduleEvent(EventObject e) throws PException {
		Iterator it = listenerList.iterator();
		while (it.hasNext()) {
			PMapperListener l = (PMapperListener) it.next();
			if (e instanceof PMappingRequiredEvent) {
				l.mappingRequired((PMappingRequiredEvent) e);
			} else if (e instanceof ClassMappedEvent) {
				l.classMapped((ClassMappedEvent) e);
			} else {
				throw new PException("Mapper cannot schedule this kind of event: " + e);
			}
		}
	}

	// IMPLEMENTATION OF METHODS FROM THE PMapper INTERFACE

	/**
	 * The name yielded by this method defines what kind of PMapper is used
	 * on a particular DS.
	 * @return	The name defining the PMapper kind.
	 */
	public String getMapperName() {
		return mapperName;
	}

	/**
	 * Retrieves the PMappingStructuresManager associated with this mapper.
	 * @return The associated PMappingStructuresManager.
	 */
	public PMappingStructuresManager getPMappingStructuresManager() {
		return pMappingStructuresManager;
	}

	/**
	 * It looks for the PClassMapping object associated to a JORM class within
	 * this PMapper.
	 * @return	The PClassMapping that maps the corresponding class, or null if
	 * 			none exists
	 */
	public PClassMapping lookup(String classname) {
		return (PClassMapping) mappedClasses.get(classname);
	}

	/**
	 * It registers a PClassMapping associated to a particular JORM class within
	 * this PMapper. It initializes the storage structures if necessary. It
	 * previously removes them if the "dsrem" flag is set. If the mapper is
	 * ready to manage the meta-information, it is loaded for this JORM class.
	 * @param pcm	The class mapping to add to this mapper.
	 * @exception PExceptionIO	It is raised when an I/O problem occurs within
	 * 							the underlying DS.
	 * @exception PExceptionProtocol	It is raised when a problem occurs
	 * 									during initialisation of the class
	 * 									mapping.
	 * @exception PExceptionTyping	It is raised when a typing problem occurs
	 * 								during initialisation of the class mapping.
	 */
	public void map(PClassMapping pcm)
			throws PException {
		map(null, pcm);
	}

	/**
	 * It registers a PClassMapping associated to a particular JORM class within
	 * this PMapper. It initializes the storage structures if necessary. It
	 * previously removes them if the "dsrem" flag is set. If the mapper is
	 * ready to manage the meta-information, it is loaded for this JORM class.
	 * @param conn	The connection to access to the underlying DS.
	 * @param pcm	The class mapping to add to this mapper.
	 * @exception PExceptionIO	It is raised when an I/O problem occurs within
	 * 							the underlying DS.
	 * @exception PExceptionProtocol	It is raised when a problem occurs
	 * 									during initialisation of the class
	 * 									mapping.
	 * @exception PExceptionTyping	It is raised when a typing problem occurs
	 * 								during initialisation of the class mapping.
	 */
	public void map(Object conn, PClassMapping pcm)
			throws PException {
		if (pcm == null)
			throw new PExceptionProtocol(
					"Cannot map null class mapping.");
		PClassMapping ecm
				= (PClassMapping) mappedClasses.get(pcm.getClassName());
		if (ecm == pcm)
			return;
		if (ecm != null)
			throw new PExceptionProtocol(
					"Class already mapped within this mapper.");
		if (!pcm.isConform(mapperName))
			throw new PExceptionProtocol("Class mapping (" + pcm.getClass().getName()
										 + ") cannot be managed by this mapper ("
										 + mapperName + ").");
		if (typeSpace == null)
			typeSpace = new PTypeSpacePAAH();
		((Loggable) pcm).setLogger(loggerFactory.getLogger(
				"org.objectweb.jorm." + mapperName + "." + pcm.getClassName()));
		pcm.init(this, null);
		mappedClasses.put(pcm.getClassName(), pcm);
		scheduleEvent(new ClassMappedEvent(this, pcm.getClassName()));
		if (logger != null)
			logger.log(BasicLevel.INFO,
					   "New mapped class: " + pcm.getClassName());
	}

	/**
	 * It registers a PClassMapping associated to a particular JORM class within
	 * this PMapper. It initializes the storage structures if necessary. It
	 * previously removes them if the "dsrem" flag is set. It also loads the
	 * meta-information related to this class if the "loadmeta" flag is set.
	 * @param conn	The connection to access to the underlying DS.
	 * @param pcm		The class mapping to add to this mapper.
	 * @param loadmeta	It specifies that meta-information should be loaded for
	 * 					this class to map.
	 * @exception PExceptionIO	It is raised when an I/O problem occurs within
	 * 							the underlying DS.
	 * @exception PExceptionProtocol	It is raised when a problem occurs
	 * 									during initialisation of the class
	 * 									mapping.
	 * @exception PExceptionTyping	It is raised when a typing problem occurs
	 * 								during initialisation of the class mapping.
	 */
	public void map(Object conn, PClassMapping pcm, boolean loadmeta)
			throws PException {
		if (loadmeta && ((metainfoManager == null) || (parser == null))) {
			throw new PExceptionProtocol(
					"Not enough means to load meta-information.");
		}
		map(pcm);
		if (!loadmeta)
			return;
		parser.parse(
				pcm.getClassName().replace('.', File.pathSeparatorChar) + ".pd");
	}

	/**
	 * It assigns the name of the mapper that is instanciated. It is mandatory
	 * to assign it at mapper initialization as the load of meta-information
	 * depends on it.
	 * @param mappername	The name of the maping as defined within the XML
	 * 						JORM definition files.
	 */
	public void setMapperName(String mappername) {
		mapperName = mappername;
	}

	public PClassMapping createGenClassMapping() throws PException {
		try {
			return (PClassMapping) gcmClass.newInstance();
		} catch (InstantiationException e) {
			throw new PException(e, "Cannot instanciate gen class mapping Class.");
		} catch (IllegalAccessException e) {
			throw new PException(e, "Cannot access to gen class mapping Class.");
		}
	}

	public Manager getMetaInfoManager() {
		return metainfoManager;
	}

	/**
	 * It returns a PTypeSpace that may be used to create PType
	 * associated to JORM classes mapped within this PMapper.
	 */
	public PTypeSpace getPTypeSpace() {
		return typeSpace;
	}

	/**
	 * Starts this mapper.
	 * @throws PException
	 */
	public void start() throws PException {
		String basemn = mapperName;
		int ind = mapperName.indexOf(".");
		if (ind != -1) {
			basemn = mapperName.substring(0, ind);
		}
		metainfoManager = jormConfigurator.getMIManager();
		parser = jormConfigurator.getParser();
		pMappingStructuresManager = jormConfigurator.getSchMgr(basemn);
		pMappingStructuresManager.setPMapper(this);
		pMappingStructuresManager.setLogger(logger);
		typeSpace = metainfoManager.getPTypeSpace();
		gcmClass = jormConfigurator.getGcmClass(basemn);
	}

	/**
	 * Stops this mapper.
	 * @throws PException
	 */
	public void stop() throws PException {
	}

	/**
	 * It unmaps a particular JORM class within this PMapper.
	 * @param classname	The name of the JORM class to be unmapped.
	 */
	public void unmap(String classname) throws PException {
		mappedClasses.remove(classname);
	}

	/**
	 * Assigns a prefetch cache to this mapper. It is optional.
	 * @exception PExceptionProtocol	It is raised if a prefetch cache
	 * 									has already been assigned.
	 */
	public void setPrefetchCache(PrefetchCache pc) throws PException {
		if (prefetchCache != null) {
			throw new PExceptionProtocol("A prefetch cache has already been assigned to this mapper");
		}
		prefetchCache = pc;
	}

	/**
	 * Returns the prefetch cache associated to this mapper. It can be null.
	 */
	public PrefetchCache getPrefetchCache() {
		return prefetchCache;
	}

	// IMPLEMENTATION OF METHODS FROM THE PMappingStructuresManager INTERFACE

	/**
	 * Asks for the map cluster associated with a JORM class.
	 * @return The relevant map cluster.
	 * @throws PException   This class has not been mapped.
	 */
	public PMapCluster getPMapCluster(String jcname) throws PException {
		if (pMappingStructuresManager == null) {
			throw new PException("No pMappingStructuresManager associated with this mapper");
		}
		return pMappingStructuresManager.getPMapCluster(jcname);
	}

	/**
	 * Asks for all the map clusters defined within this mapper.
	 * @return The Iterator over the collection of map clusters.
	 */
	public Collection getPMapClusters() {
		if (pMappingStructuresManager == null) {
			return new ArrayList();
		}
		return pMappingStructuresManager.getPMapClusters();
	}

	public void addDependency(String jcname1, String jcname2) throws PException {
		if (pMappingStructuresManager == null) {
			throw new PException("No pMappingStructuresManager associated with this mapper");
		}
		pMappingStructuresManager.addDependency(jcname1, jcname2);
	}

	public void declareClass(String jcname) {
		pMappingStructuresManager.declareClass(jcname);
	}

	public void classDefined(String jcname) throws PException {
		if (pMappingStructuresManager == null) {
			throw new PException("No pMappingStructuresManager associated with this mapper");
		}
		pMappingStructuresManager.classDefined(jcname);
	}

	/**
	 * Adds the given mapper listener to this mapper. If it already exists,
	 * does nothing.
	 * @param listener  The corresponding listener.
	 */
	public synchronized void addMapperEventListener(PMapperListener listener) {
		if (listenerList.indexOf(listener) == -1) {
			listenerList.add(listener);
		}
	}

	/**
	 * Removes the given mapper listener from this mapper. If it is not
	 * present among this list, does nothing.
	 * @param listener  The corresponding listener.
	 */
	public synchronized void removeMapperEventListener(PMapperListener listener) {
		if (listenerList.indexOf(listener) != -1) {
			listenerList.remove(listener);
		}
	}

	/**
	 * Assigns a mapper to this schema manager.
	 * @param pm	The concerned mapper.
	 */
	public void setPMapper(PMapper pm) {
	}

	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	/**
	 * It assigns a logger to an component that wants to log things.
	 * @param logger	The logger object.
	 */
	public void setLogger(Logger logger) {
	}

	/**
	 * It assigns a logger factory that allows the creation of new loggers.
	 * @param loggerfactory	The LoggerFactory object to obtain new loggers.
	 */
	public void setLoggerFactory(LoggerFactory loggerfactory) {
		loggerFactory = loggerfactory;
		logger = loggerFactory.getLogger("org.objectweb.jorm." + mapperName);
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public String cn2mn(String cn) {
		String shortMapperName = getMapperName();
		int idx = shortMapperName.indexOf(".");
		if (idx != -1) {
			shortMapperName = shortMapperName.substring(0, idx);
		}
		int ldot = cn.lastIndexOf('.');
		if (ldot == -1)
			return shortMapperName + "." + cn;
		return cn.substring(0, ldot + 1) + shortMapperName + cn.substring(ldot);
	}

	// IMPLEMENTATION OF METHODS FROM THE Object CLASS

	public boolean equals(Object o) {
		if (this == o)
			return true;
		if (!(o instanceof PMapper))
			return false;
		PMapper m = (PMapper) o;
		return m.getMapperName().equals(mapperName);
	}

	public int hashCode() {
		return mapperName.hashCode();
	}
}
