/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.JormConfigurator;

import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.Connection;
import javax.resource.cci.ConnectionSpec;
import javax.resource.ResourceException;

/**
 * This mapper supports JCA CCI connection management.
 * @author	P. Dechamboux
 */
public class MapperJCA extends Mapper {
	private ConnectionFactory connectionFactory;

	public MapperJCA() throws PException {
		super();
	}

	public MapperJCA(JormConfigurator jc) {
		super(jc);
	}

	/**
	 * Closes the given connection which has been allocated previously using
	 * getConnection.
	 * @param conn	The concerned connection.
	 */
	public void closeConnection(Object conn) throws PException {
		try {
			((Connection) conn).close();
		} catch (ResourceException re) {
			throw new PException(re, "Problem while closing connection.");
		}
	}

	/**
	 * Allocates a connection for accessing the underlying DS.
	 */
	public Object getConnection() throws PException {
		try {
			return connectionFactory.getConnection();
		} catch (ResourceException re) {
			throw new PException(re, "Problem while allocating connection.");
		}
	}

	/**
	 * Allocates a connection for accessing the underlying DS knowing additional
	 * context information.
	 * @param ctxt	Context information used to allocate the connection.
	 */
	public Object getConnection(Object ctxt) throws PException {
		try {
			return connectionFactory.getConnection((ConnectionSpec) ctxt);
		} catch (ResourceException re) {
			throw new PException(re, "Problem while allocating connection with SPEC.");
		}
	}

	public Object getConnection(Object connectionContext, Object user) throws PException {
		try {
			return connectionFactory.getConnection((ConnectionSpec) connectionContext);
		} catch (ResourceException re) {
			throw new PException(re, "Problem while allocating connection with SPEC.");
		}
	}

	/**
	 * Assigns a connection factory to this mapper.
	 * @exception PExceptionProtocol	It is raised if a connection factory
	 * 									has already been assigned.
	 */
	public void setConnectionFactory(Object cf) throws PException {
		if (!(cf instanceof ConnectionFactory))
			throw new PException("Requires a JCA connection factory.");
		if (connectionFactory != null)
			throw new PExceptionProtocol("Connection factory assigned once.");
		connectionFactory = (ConnectionFactory) cf;
	}

	/**
	 * Returns the connection factory associated to this mapper.
	 */
	public Object getConnectionFactory() {
		return connectionFactory;
	}
}
