/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.lib;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PException;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * is an implementation of the PNameIterator interface over several
 * PClassMapping instances.
 *
 * @author S.Chassande-Barrioz
 */
public class MultiplePNameIterator implements PNameIterator {

	/**
	 * The array of PClassMapping instance giving all PName to return.
	 */
	private PClassMapping[] pcms;

	/**
	 * The current iterator
	 */
	private Iterator currentIterator;

	/**
	 * The index of the current PClassMapping used
	 */
	private int pcmIdx;

	/**
	 * The connection to the data support permitting the query evaluations on
	 * data support.
	 */
	private Object connection;

	/**
	 * The transaction context that can be used in case of prefetching.
	 */
	private Object txctx;

	/**
	 * is the next PName to return at the next call to the next() method.
	 * A null value means there are no more result in any PClassMapping. This
	 * is the calculateNext() method which compute this value.
	 */
	private Object nextPName;

	/**
	 * indicates if the data prefetching is activated. When the prefetching is
	 * activated, the sub PNameIterator are not closed immediatly. They will
	 * be closed by the prefetching mechanism itself.
	 */
	private boolean prefetching;

	/**
	 * Builds a PNameIterator over a set of PClassMapping instances
	 * @param pcms is the PClassMapping list
	 * @param connection is the connection to the data support
	 * @param prefetching indicates if prefetching is activated
	 * @throws PException if it is not possible to fetch the first element.
	 */
	public MultiplePNameIterator(PClassMapping[] pcms,
	                             Object connection,
	                             boolean prefetching,
								 Object txctx) throws PException {
		this(pcms, null, connection, prefetching, txctx);
	}

	/**
	 * Builds a PNameIterator over an iterator and a set of PClassMapping
	 * instances.
	 *
	 * @param pcms is the PClassMapping list
	 * @param currentIt is the first iterator to use which has not been obtained
	 * through a PClassMapping given in parameter.
	 * @param connection is the connection to the data support
	 * @param prefetching indicates if prefetching is activated
	 * @throws PException if it is not possible to fetch the first element.
	 */
	public MultiplePNameIterator(PClassMapping[] pcms,
	                             Iterator currentIt,
	                             Object connection,
	                             boolean prefetching,
								 Object txctx) throws PException {
		this.pcms = pcms;
		this.prefetching = prefetching;
		this.connection = connection;
		currentIterator = currentIt;
		this.txctx = txctx;
		pcmIdx = -1;
		calculateNext();
	}

	/**
	 * Calculates the next element. The next element is stored into the
	 * nextPName variable. At the end if nextPName == null, that means not next
	 * element has been found.
	 * @throws PException if it is not possible to evaluate a sub PNameIterator
	 */
	private void calculateNext() throws PException {
		if (currentIterator != null && currentIterator.hasNext()) {
			nextPName = currentIterator.next();
		} else {
			nextPName = null;
			//close the previous iterator
			if (!prefetching && currentIterator instanceof PNameIterator) {
				((PNameIterator) currentIterator).close();
			}
			//use the next PClassMapping
			pcmIdx ++;
			if (pcmIdx < pcms.length) {
				//compute the next iterator
				currentIterator = pcms[pcmIdx]
				    .getPNameIterator(connection, true, prefetching, txctx);
				//recall to find the next PName
				calculateNext();
			} // else no next PCM then the iteration is finished
		}
	}

	/**
	 * @return true if there is a next PName
	 */
	public boolean hasNext() {
		return nextPName != null;
	}

	public Object next() {
		if (nextPName == null) {
			throw new NoSuchElementException();
		}
		Object res = nextPName;
		try {
			calculateNext();
		} catch (PException e) {
			nextPName = null;
		}
		return res;
	}

	/**
	 * The remove operation is not supported
	 */
	public void remove() {
		throw new UnsupportedOperationException("Remove operation not supported");
	}

	/**
	 * Close the PNameIterator is the prefetch is not activated.
	 * @throws PException
	 */
	public void close() throws PException {
		nextPName = null;
		if (!prefetching && currentIterator instanceof PNameIterator) {
			((PNameIterator) currentIterator).close();
		}
	}
}
