/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.fos.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.lib.AbstractPMapcluster;
import org.objectweb.perseus.fos.api.FosAccess;
import org.objectweb.perseus.fos.api.FosException;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Defines the PMapCluster used to manage the directory tree for the FOS
 * mappers.
 * @author	P. Dechamboux
 */
public class FosPMapCluster extends AbstractPMapcluster {
	private ArrayList directories;
	private FosPMappingStructuresManager manager;

	/**
	 * Constructs a RdbPMapCluster for an abstract class
	 * @param jcname	The first JORM class name to be assigned to this cluster.
	 * @param man		The manager associated to these mapping structures.
	 */
	FosPMapCluster(String jcname, FosPMappingStructuresManager man) {
		super();
		jcNames.add(jcname);
		directories = new ArrayList(1);
		manager = man;
	}

	/**
	 * Constructs a RdbPMapCluster.
	 * @param jcname	The first JORM class name to be assigned to this cluster.
	 * @param basedir	The directory to be used for this class.
	 * @param man		The manager associated to these mapping structures.
	 */
	FosPMapCluster(String jcname, String basedir, FosPMappingStructuresManager man) {
		this(jcname, man);
		directories.add(basedir);
	}

	/**
	 * Adds a new JORM class name to this cluster.
	 * @param jcname	The JORM class name.
	 */
	void addClass(String jcname) {
		addDependency(jcname);
	}

	/**
	 * Defines a new directory into this map cluster if it does not already
	 * exist.
	 * @param dirname	The directory name.
	 * @throws PException Thrown if the column is redefined with incompatible
	 * information.
	 */
	void addDirName(String dirname) throws PException {
		String curd = null;
		Iterator it = directories.iterator();
		while (it.hasNext()) {
			curd = (String) it.next();
			if (curd.equals(dirname)) {
				break;
			}
			curd = null;
		}
		if (curd == null) {
			// no such directory defined yet: create a new one
			directories.add(dirname);
		}
	}

	/**
	 * Looks for this specified directory to be a sub-directory of the ones
	 * defined into this map cluster.
	 * @param dirname	The table name.
	 * @throws PException Thrown if the column is redefined with incompatible
	 * information.
	 */
	boolean containDirectory(String dirname) throws PException {
		String curd = null;
		Iterator it = directories.iterator();
		while (it.hasNext()) {
			curd = (String) it.next();
			if ((dirname.length() >= curd.length()) && curd.equals(dirname.substring(0, curd.length()))) {
				return true;
			}
			if ((dirname.length() < curd.length()) && dirname.equals(curd.substring(0, dirname.length()))) {
				return true;
			}
			curd = null;
		}
		return false;
	}

	/**
	 * Merges map cluster passed in parameter with this one. Clusters are merged
	 * either because they share a directory.
	 * @param cl
	 */
	void merge(FosPMapCluster cl) {
		jcNames.addAll(cl.jcNames);
		directories.addAll(cl.directories);
	}

	// IMPLEMENTATION OF METHODS FROM THE PMapCluster INTERFACE

	/**
	 * Creates the mapping structures defined by this map cluster. Nothing to do
	 * with FOS.
	 * @throws PException   Thrown if it cannot be performed (especially
	 * if all or some of these mapping structures already exist.
	 */
	public void createMappingStructures(boolean force) throws PException {
		if (structuresActive) {
			throw new PException("Cannot change mapping structures while they are under use");
		}
	}

	/**
	 * Deletes the data that have been inserted into the mapping structures
	 * defined by this map cluster.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 */
	public void deleteData() throws PException {
		Object conn = manager.getConnection();
		String curd = null;
		try {
			Iterator it = directories.iterator();
			while (it.hasNext()) {
				curd = (String) it.next();
				if (((FosAccess) conn).existDir(curd)) {
					Iterator it2 = ((FosAccess) conn).scan(curd);
					while (it2.hasNext()) {
						String oid = (String) it2.next();
						((FosAccess) conn).delete(curd, oid);
					}
				}
			}
		} catch (FosException e) {
			throw new PException(e, "PB while deteling data from FOS directory " + curd);
		} finally {
			manager.closeConnection(conn);
		}
	}

	/**
	 * Deletes all or part of the mapping structures defined by this map
	 * cluster when they exist. If they contain some data, they are also
	 * deleted.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 */
	public void deleteMappingStructures() throws PException {
		if (structuresActive) {
			throw new PException("Cannot change mapping structures while they are under use");
		}
		Object conn = manager.getConnection();
		String curd = null;
		try {
			Iterator it = directories.iterator();
			while (it.hasNext()) {
				curd = (String) it.next();
				if (((FosAccess) conn).existDir(curd)) {
					((FosAccess) conn).deleteDir(curd);
				}
			}
		} catch (FosException e) {
			throw new PException(e, "PB while deteling FOS directory " + curd);
		} finally {
			manager.closeConnection(conn);
		}
	}
}
