/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.fos.xml2mi;

import org.objectweb.jorm.xml2mi.lib.BasicMappingParser;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.mapper.fos.metainfo.FosMapping;
import org.objectweb.jorm.mapper.fos.metainfo.FosClassMapping;
import org.objectweb.jorm.mapper.fos.metainfo.FosGenClassMapping;
import org.objectweb.util.monolog.api.BasicLevel;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import java.util.Iterator;

/**
 * @author	P. Dechamboux
 */
public class FosParser extends BasicMappingParser {
	/**
	 * Parses the mapping information related to a particular mapper and builds
	 * the corresponding Jorm meta-information. It can be class-related or
	 * generic class-related information.
	 * Manager, PathExplorer and currentClass must be set before calling this
	 * method.
	 * @param classMappingElem the current XML node.
	 * @param mapping          a Mapping object.
	 */
	private void parseClassMapping(Element classMappingElem, Mapping mapping) {
		FosClassMapping fosClassMapping = (FosClassMapping) mapping.createClassMapping("");
		fosClassMapping.setLogger(logger);
		Class c = (Class) fosClassMapping.getLinkedMO();
		Iterator it = c.getFields().iterator();
		while (it.hasNext()) {
			Object fd = it.next();
			if (fd instanceof PrimitiveElement) {
				fosClassMapping.addFieldMapping(((PrimitiveElement) fd).getName());
			}
		}
		NodeList children = classMappingElem.getChildNodes();
		String dirname = null;
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String childName = child.getNodeName();
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "begin =<" + childName + ">");
			}
			if (childName.equals("fos-dir-name")) {
				dirname = child.getFirstChild().getNodeValue();
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, "read dirname: " + dirname);
				}
			} else if (childName.equals("id-mapping")) {
				String linkend = ((Element) child).getAttribute("link-end");
				// Gets the NameDef object. It may be inherited from a super-class.
				NameDef nd = getIdNameDef(fosClassMapping, linkend);
				if (nd.isNameRef()) {
					Iterator it2 = nd.iterateField();
					while (it2.hasNext()) {
						fosClassMapping.addFieldMapping((String) it2.next());
					}
				} else {
					fosClassMapping.addFieldMapping(nd.getFieldName());
				}
				fosClassMapping.createIdentifierMapping(nd);
			} else if (childName.equals("fos-ref-mapping")) {
				String linkend = ((Element) child).getAttribute("link-end");
				// Gets the NameDef object, possibly beyond the scope of the current class if
				// this NameDef is inherited from a super-class.
				NameDef nd = getRefNameDef(linkend, childName);
				if (nd.isNameRef()) {
					Iterator it2 = nd.iterateField();
					while (it2.hasNext()) {
						fosClassMapping.addFieldMapping((String) it2.next());
					}
				} else {
					fosClassMapping.addFieldMapping(nd.getFieldName());
				}
				fosClassMapping.createReferenceMapping(null, nd);
			} else if (childName.equals("#text")) {
			} else {
				logger.log(BasicLevel.WARN, "element <" + childName + "> unknown !");
			}
			if (logger.isLoggable(BasicLevel.DEBUG))
				logger.log(BasicLevel.DEBUG, "end   =<" + childName + ">");
		}
		fosClassMapping.setDirName(dirname);
	}

	/**
	 * Parses a relational generic class mapping element and builds the
	 * corresponding Jorm meta-information.
	 * Manager, PathExplorer and currentClass must be set before calling this
	 * method.
	 *
	 * @param genClassMappingElem   the current XML node,
	 * @param mapping    a Mapping object.
	 */
	private void parseGenClassMapping(Element genClassMappingElem,
									  Mapping mapping) {

		String linkend = genClassMappingElem.getAttribute("link-end");
		if (getLogger().isLoggable(BasicLevel.DEBUG)) {
			getLogger().log(BasicLevel.DEBUG,
							"linkend of fos-gen-class-mapping <" + linkend + ">");
		}
		GenClassRef genClassRef = (GenClassRef) idvalue2genclassref.get(linkend);
		if ((genClassRef != null) &&
				(getLogger().isLoggable(BasicLevel.DEBUG))) {
			getLogger().log(BasicLevel.DEBUG,
							"GenClassRef name " + genClassRef.getName());
		}
		// Creates a GenClassMapping object.
		if (getLogger().isLoggable(BasicLevel.DEBUG)) {
			getLogger().log(BasicLevel.DEBUG,
							"create a new BasicFosGenClassMapping for the current " +
							"Class (" + currentClass.getName() + ")");
		}
		FosGenClassMapping fosGenClassMapping = ((FosMapping) mapping)
				.createGenClassMapping("", genClassRef);
		fosGenClassMapping.setLogger(logger);
		Iterator it = genClassRef.getIndexFields().iterator();
		while (it.hasNext()) {
			fosGenClassMapping.addFieldMapping(((TypedElement) it.next()).getName());
		}
		NodeList children = genClassMappingElem.getChildNodes();
		String dirname = null;
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String childName = child.getNodeName();
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "begin =<" + childName + ">");
			}
			if (childName.equals("fos-dir-name")) {
				dirname = child.getFirstChild().getNodeValue();
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, "read dirname: " + dirname);
				}
			} else if (childName.equals("id-mapping")) {
				// The link-end attribute contains the name of the associated name-def.
				String namedefName = ((Element) child).getAttribute("link-end");
				// Gets the corresponding NameDef object.
				NameDef nd = genClassRef.getIdNameDef(namedefName);
				if (nd.isNameRef()) {
					Iterator it2 = nd.iterateField();
					while (it2.hasNext()) {
						fosGenClassMapping.addFieldMapping((String) it2.next());
					}
				} else {
					fosGenClassMapping.addFieldMapping(nd.getFieldName());
				}
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "Fetching the ID name def of the GenClassRef " + namedefName);
					if (nd.isFieldName())
						logger.log(BasicLevel.DEBUG, "fieldName: " + nd.getFieldName());
					else if (nd.isNameRef())
						logger.log(BasicLevel.DEBUG, "NameRef: " + nd.getNameRef());
				}
				// Creates an IdentifierMapping object
				fosGenClassMapping.createIdentifierMapping(nd);
			} else if (childName.equals("fos-ref-mapping")) {
				String namedefName = ((Element) child).getAttribute("link-end");
				NameDef nd = null;
				//TODO: Verifier
				if (genClassRef.isPrimitive()) {
					logger.log(BasicLevel.WARN, "You define a generic class of " +
												"primitive and a useless name-def for the elements: " +
												"generic class id=" + genClassRef.getGenClassId());
					continue;
				} else if (genClassRef.isClassRef()) {
					if (logger.isLoggable(BasicLevel.DEBUG)) {
						logger.log(BasicLevel.DEBUG,
								   "Get the ref name def of the GenClassRef " + namedefName);
					}
					nd = genClassRef.getClassRef().getRefNameDef(namedefName);
					if (nd.isNameRef()) {
						Iterator it2 = nd.iterateField();
						while (it2.hasNext()) {
							fosGenClassMapping.addFieldMapping((String) it2.next());
						}
					} else {
						fosGenClassMapping.addFieldMapping(nd.getFieldName());
					}
				} else if (genClassRef.isGenClassRef()) {
					if (logger.isLoggable(BasicLevel.DEBUG)) {
						logger.log(BasicLevel.DEBUG,
								   "Get the ref name def of the ClassRef " + namedefName);
					}
					nd = genClassRef.getGenClassRef().getRefNameDef(namedefName);
					if (nd.isNameRef()) {
						Iterator it2 = nd.iterateField();
						while (it2.hasNext()) {
							fosGenClassMapping.addFieldMapping((String) it2.next());
						}
					} else {
						fosGenClassMapping.addFieldMapping(nd.getFieldName());
					}
				}
				String ruleName = ((Element) child).getAttribute("rule-name");
				// Creates a ReferenceMapping.
				fosGenClassMapping.createReferenceMapping(ruleName, nd);
			} else if (childName.equals("#text")) {
			} else {
				logger.log(BasicLevel.WARN, "element <" + childName + "> unknown !");
			}
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "end   =<" + childName + ">");
			}
		}
		fosGenClassMapping.setDirName(dirname);
	}

	/**
	 * Parses the mapping information related to a particular mapper and builds
	 * the corresponding Jorm meta-information. It can be class-related or
	 * generic class-related information.
	 * Manager, PathExplorer and currentClass must be set before calling this
	 * method.
	 *
	 * @param mappingElem   the current XML node.
	 * @param mapping       a Mapping object.
	 */
	public void parseMapping(Element mappingElem, Mapping mapping) throws PException {
		if (mappingElem.getNodeName().equals("fos-class-mapping")) {
			parseClassMapping(mappingElem, mapping);
		} else if (mappingElem.getNodeName().equals("fos-gen-class-mapping")) {
			parseGenClassMapping(mappingElem, mapping);
		} else {
			throw new PException("Mapping element not supported by FOS: " + mappingElem.getNodeName());
		}
	}
}
