/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.generator;

import org.apache.velocity.context.Context;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.generator.api.MOP;
import org.objectweb.jorm.generator.lib.CommonGenerator;
import org.objectweb.jorm.generator.lib.NamingFilterExpressionHelper;
import org.objectweb.jorm.mapper.rdb.lib.RdbExtentGenInfos;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.PrimitiveElementMapping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

public class RdbMappingMOP extends RdbCommonHelper implements MOP {

	public RdbMappingMOP(Logger logger) {
		super(logger);
	}

	public boolean isInPK(PrimitiveElementMapping pem) {
		return false;
	}

	// IMPLEMENTATION OF METHODS FROM THE MOP INTERFACE

	public void initContext(Context ctx, Class clazz,
							String projectName,
							String mapperName) throws PException {
		ctx.put("rdbTools", this);
		String mn = (String) ctx.get("mappername");
		ctx.put("adapter", getRdbAdapter(mn));
		ctx.put("G", "\"");
		RdbGenInfos rdbGenInfos = getRdbGenInfos(clazz, projectName, mapperName); 
		ctx.put("genInfos", rdbGenInfos);
        ctx.put("extentGenInfos", new RdbExtentGenInfos(logger));
        
        if (clazz.isPolymorphic()) {
        	NamingFilterExpressionHelper mi = null;
        	try {
        		//get the namedef : it can belong to a super class
        		NameDef nd = clazz.getClassMapping(projectName, mapperName).getIdentifierMapping().getNameDef();
        		//get the class in which looking for the filter
        		Class classToLookForExpression = (Class) nd.getParent();
        		//get the medor expression of the filter
        		Expression exp = classToLookForExpression.getInheritanceFilter(nd);
        		//compile it to compute the type
        		exp.compileExpression();
        		//generate the filter evaluate method
        		mi = new PolymorphicFilterExpressionHelper(exp, logger);
        	} catch (ExpressionException e) {
        		throw new PException(e);
        	}
        	ctx.put("mi", mi);
        }
	}

	public String[] getTemplateLibraries() {
		String[] libs = {
                CommonGenerator.MAPPER_TEMPLATE_DIR + "rdb/generator/RdbMapping.vm",
                CommonGenerator.MAPPER_TEMPLATE_DIR + "rdb/generator/RdbExtentDefinition.vm"};
		return libs;
	}
	
	private String getStrAsObject(String str, PType type) {
		switch (type.getTypeCode()) {
		case PType.TYPECODE_BYTE:
			return "new Byte(" + str +")";
		case PType.TYPECODE_CHAR:
			return "new Character(" + str +")";
		case PType.TYPECODE_SHORT:
			return "new Short(" + str +")";
		case PType.TYPECODE_INT:
			return "new Integer(" + str +")";
		case PType.TYPECODE_LONG:
			return "new Long(" + str +")";
		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_OBJCHAR:
		case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_OBJINT:
		case PType.TYPECODE_OBJLONG:
		case PType.TYPECODE_DATE:
		case PType.TYPECODE_CHARARRAY:
		case PType.TYPECODE_BYTEARRAY:
		case PType.TYPECODE_STRING:
		case PType.TYPECODE_BIGDECIMAL:
		case PType.TYPECODE_BIGINTEGER:
		default:
			return str;
		}
	}
	
	private String getTupleType(PType type) {
		switch (type.getTypeCode()) {
		case PType.TYPECODE_DATE:
			return "Date";
		case PType.TYPECODE_STRING:
			return "String";
		case PType.TYPECODE_CHARARRAY:
			return "CharArray";
		case PType.TYPECODE_BYTEARRAY:
			return "ByteArray";
		case PType.TYPECODE_BIGDECIMAL:
			return "BigDecimal";
		case PType.TYPECODE_BIGINTEGER:
			return "BigInteger";
		case PType.TYPECODE_BYTE:
			return "Byte";
		case PType.TYPECODE_CHAR:
			return "Char";
		case PType.TYPECODE_SHORT:
			return "Short";
		case PType.TYPECODE_INT:
			return "Int";
		case PType.TYPECODE_LONG:
			return "Long";
		case PType.TYPECODE_OBJBYTE:
		case PType.TYPECODE_OBJBOOLEAN:
		case PType.TYPECODE_OBJDOUBLE:
		case PType.TYPECODE_OBJFLOAT:
		case PType.TYPECODE_OBJCHAR:
		case PType.TYPECODE_OBJSHORT:
		case PType.TYPECODE_OBJINT:
		case PType.TYPECODE_OBJLONG:
		default:
			return "Object";
		}
	}
	/**
	 * This class is used to compute the filter for classes being part of 
	 * an inheritance object graph and not having the filter in the primary key.
	 * @author Y.Bersihand
	 */
	public class PolymorphicFilterExpressionHelper extends NamingFilterExpressionHelper {

		private PType filterType;
		public PolymorphicFilterExpressionHelper(Expression e,
											   Logger logger)
			throws PException, ExpressionException {
			super(logger);
			this.filterType = e.getType();
			fillMatchInfo(e);
		}

		public String getFilter() {
			return getStrAsObject(super.getFilter(), filterType);
		}

		/**
		 * Genrates the java code to get the value of a field into a Tuple:
		 * tuple.getXXX(fieldRank);
		 */
		protected void fillMatchInfo(ParameterOperand po) throws PException, ExpressionException {
			logger.log(BasicLevel.DEBUG, "ParameterOperand " + po.getName());
			if (field2declarations.get(po.getName()) == null) {
				StringBuffer declaration = new StringBuffer();
				declaration.append(po.getType().getJavaName());
				declaration.append(' ');
				declaration.append(po.getName());
				declaration.append(" = ");
				declaration.append("tuple.get");
				declaration.append(getTupleType(po.getType()));
				declaration.append("(fieldRank");
				declaration.append(")");
				declaration.append(";");
				field2declarations.put(po.getName(), declaration);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, "Add declaration: " + declaration);
				}
			}
			filter.append(po.getName());
		}
		
	}
}
