/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.lib;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;

import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbJoin;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbTable;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class is used to construct extent mapping information when calculating
 * extents (see getNonFinalClassExtent) and that will then be used to
 * generate the appropriate union of select/from/where clauses
 */
public class RdbExtentMappingInfos {

	public static final String NULL_COLUMN = "NULL";
	
	public static Logger logger;
    public static boolean debug = false;
    
	// class to which the expression is attached
	Class clazz;

	// a table of columns for the projections
	private RdbColumnInfo[] columns;
        
	// joins required to get the desired columns
	HashSet joins = new HashSet(); // {RdbJoin}
	// tables required to get the desired columns
	RdbTable mainTable;

	// the filter set will contain the name of classes from which filtering
	// expressions must be retrieved so as to be added to the where clause.
	HashSet filters = new HashSet();  // {String}
	// if addfilters is true, then filters of classes mapped on the same
	// tables (extended/added inheritance) must be added
	boolean addFilters = false;
	boolean hasUnmappedFields = false;
	private boolean emptyEMI = true;
        
	// we can give a name to EMIs
	private String name;
        
	RdbExtentMappingInfos(Class clazz, String name, int nbColumns) {
		this.clazz = clazz;
        this.name = name;
        this.columns = new RdbColumnInfo[nbColumns];
        //debug = logger != null && logger.isLoggable(BasicLevel.DEBUG);
	}

	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	// getters required by Velocity
	public HashSet getFilters() {
		return filters;
	}

	public RdbTable getMainTable() {
		return mainTable;
	}

	
	public void setMainTable(RdbTable mainTable) {
		this.mainTable = mainTable;
	}
	
	public HashSet getJoins() {
    	return joins;
    }

	public void addJoin(RdbJoin join){
		joins.add(join);
	}
	
	public boolean isEmpty() {
		return emptyEMI;
	}

	public void addProjection(String tableName, String columnName,
                               String aliasName, String className, int position) {
		if (columns[position] != null && columns[position].getName() != null && !columns[position].getName().equals(NULL_COLUMN)) {
        	if (debug) {
        		String newExpr = tableName + "." + columnName;
        		if (!columns[position].getName().equals(newExpr)) {
        			throw new InternalError("expr in select definition (" + newExpr
                                + ") not equal to old one (" + columns[position].getName() + ")");
                }
                if (!columns[position].getAlias().equals(aliasName)) {
                	throw new InternalError("alias in select definition (" + aliasName
                			+ ") not equal to old one (" + columns[position].getAlias() + ")");
                	}
            }
		} else {
			emptyEMI = false;
            //do not add the . if the columnName is NULL, because the tableName is an empty string
            // and we want to have something like that: select t.a, t.b, NULL, t.d from t
            columns[position] = new RdbColumnInfo(columnName.equals(NULL_COLUMN)?columnName:tableName + "." + columnName, 
              					aliasName,
								className);
		}
	}
       
	/**
	 * get a select parameter to give to the getQuery method of adapter
     * this method is called in RdbExtentDefinition.vm when generating the
     * extent definition
     * @param adapterName the name of the adapter in the generated code
     * @return a string to include as the first parameter of the call to
     * getQuery in the generated code
     */
	public String getSelectParameter(String adapterName) {
		StringBuffer sb = new StringBuffer();
        // select parameter
        for (int pos = 0; pos < columns.length; pos++) {
           sb.append("\"" + columns[pos].getName() + "\" + " +
                   adapterName +".getColumnAliasExpr(\"" + columns[pos].getAlias() + "\")");
           if (pos < columns.length - 1) {
               sb.append(" + \",\" + ");
            }
        }
        return sb.toString();
	}

	/**
     * Returns a String of type "x1 as y1, x2 as y2, x3 as y3" for the prefetch case.
     * @param adapter
     * @return
     */
	public String getSelectPrefetch(RdbAdapter adapter, Class cl) {
		RdbColumnInfo[] columnsClone = new RdbColumnInfo[columns.length];
        for(int i = 0; i < columns.length; i++){
        	columnsClone[i] = columns[i];
        }
        //sort the table
       	if (cl.isPolymorphic()) {
       	    //Sort by PE name 
       	    Arrays.sort(columnsClone);
       	} else {
       	    //sort by the PEM
       	    Arrays.sort(columnsClone, new Comparator() {
       	     public int compare(Object o1, Object o2) {
       	         return ((RdbColumnInfo) o1).getName().compareTo(
       	                 ((RdbColumnInfo) o2).getName());
       	     }
       	    });
       	}
        StringBuffer sb = new StringBuffer();
        // select parameter
        for (int pos = 0; pos < columnsClone.length; pos++) {
        	sb.append(columnsClone[pos].getName() + " " +
        			adapter.getColumnAliasExpr(columnsClone[pos].getAlias()));
            if (pos < columnsClone.length - 1) {
            	sb.append(" , ");
            }
        }
        return sb.toString();
	}
        
    /**
     * Returns a String of type "x1 as y1, x2 as y2" for the pk only case, 
     * x1 and x2 composing the primary key.
     * @param adapter
     * @param primaryKey the list of columns belonging to the primaryKey
     * @return
     */
	public String getSelectPKOnly(RdbAdapter adapter, String primaryKey) {
		RdbColumnInfo[] columnsClone = new RdbColumnInfo[columns.length];
        for(int i = 0; i < columns.length; i++){
        	columnsClone[i] = columns[i];
        }
        //sort the table
        Arrays.sort(columnsClone);
        StringBuffer sb = new StringBuffer();
        // select parameter
        for (int pos = 0; pos < columnsClone.length; pos++) {
         	if(primaryKey.indexOf(columnsClone[pos].getName()) != -1){
           		sb.append(columnsClone[pos].getName() + " " +
           				adapter.getColumnAliasExpr(columnsClone[pos].getAlias()));
           		if (pos < columnsClone.length - 1) {
           			sb.append(" , ");
           		}
            }
        }
        return sb.toString();
	}
        
    /**
     * Returns a String of type "x1 as y1, x2 as y2, x3 as y3" for the pk and fields case, 
     * x1 and x2 composing the primary key, x3 being a field that belongs to the class or one of its super classes.
     */
	public String getSelectPKAndFields(RdbAdapter adapter, Class cl) {
     	RdbColumnInfo[] columnsClone = new RdbColumnInfo[columns.length];
       	for(int i = 0; i < columns.length; i++){
       		columnsClone[i] = columns[i];
       	}
       	//sort the table
       	if (cl.isPolymorphic()) {
       	    Arrays.sort(columnsClone);
       	} else {
       	    Arrays.sort(columnsClone, new Comparator() {
       	     public int compare(Object o1, Object o2) {
       	         return ((RdbColumnInfo) o1).getName().compareTo(
       	                 ((RdbColumnInfo) o2).getName());
       	     }
       	    });
       	}
       	StringBuffer sb = new StringBuffer();
       	Collection superClasses = new ArrayList();
        //add the current class name to the list of super classes, 
        //not to forget all the fields belonging to this class
        superClasses.add(cl.getFQName());
        //get all the ancestors of the class cl
        Iterator it = cl.getAncestors().iterator();
        //and add the names of these classes to the superClasses list
        while(it.hasNext()){
        	Class c = (Class) it.next();
           	superClasses.add(c.getFQName());
        }
        for (int pos = 0; pos < columnsClone.length; pos++) {
          	//add the field to the list only if it belongs to one of the super classes
           	//including the class cl
           	if (superClasses.contains(columnsClone[pos].getClassName())) {
           		sb.append(columnsClone[pos].getName() + " " +
           				adapter.getColumnAliasExpr(columnsClone[pos].getAlias()));
           		if (pos < columnsClone.length - 1) {
           			sb.append(" , ");
           		}
           	}
        }
        return sb.toString();
	}
        
    public String getWhereParameter(PClassMapping pcm) {
    	StringBuffer sb = new StringBuffer();
        if(pcm != null && pcm instanceof RdbPPolymorphicClass){
         	if (this.hasJoins()) {  
                sb.append(" AND (");
                ((RdbPPolymorphicClass)pcm).appendExtentFilters(this.name, sb);
                sb.append(")");
            } else {
                sb.append(" WHERE ");
                ((RdbPPolymorphicClass)pcm).appendExtentFilters(this.name, sb);
            }
         }
        return sb.toString();
    }
        
    public String getAliasName(int position){
    	if (position >= 0 && position < columns.length && columns[position] != null)
        	return columns[position].getAlias();
        else 
        	return null;
    }
        
    /**
     * 
     * @param prefetch
     * @param primaryKey : the String representing the fields composing the primary key
     * @param cl : if prefetch is true, this parameter is not used and the getColumnAliasesPrefetch method is called
     * 				else 
     * 					if cl is null the getColumnAliasesPKOnly method is called
     * 					else the getColumnAliasesPKAndFields method is called
     * @return
     */
    public Collection getColumnAliases(boolean prefetch, String primaryKey, Class cl){
     	if (prefetch) {
       		return getColumnAliasesPrefetch();
       	} else {
       		if(cl == null)
       			return getColumnAliasesPKOnly(primaryKey);
       		else
       			return getColumnAliasesPKAndFields(cl);
       	}
    }
        
    private Collection getColumnAliasesPrefetch(){
       	RdbColumnInfo[] columnsClone = new RdbColumnInfo[columns.length];
       	for(int i = 0; i < columns.length; i++){
       		columnsClone[i] = columns[i];
       	}
       	//sort the table
       	Arrays.sort(columnsClone);
       	Collection c = new ArrayList();
       	for(int i=0; i< columnsClone.length; i++)
       		c.add(columnsClone[i].getAlias());
       	return c;
    }
        
    private Collection getColumnAliasesPKOnly(String primaryKey){
       	RdbColumnInfo[] columnsClone = new RdbColumnInfo[columns.length];
       	for(int i = 0; i < columns.length; i++){
       		columnsClone[i] = columns[i];
       	}
       	//sort the table
       	Arrays.sort(columnsClone);
       	Collection c = new ArrayList();
       	for(int i=0; i< columnsClone.length; i++)
       		if(primaryKey.indexOf(columnsClone[i].getName()) != -1)
       			c.add(columnsClone[i].getAlias());
       	return c;
    }
        
    public Collection getColumnAliasesPKAndFields(Class cl){
      	RdbColumnInfo[] columnsClone = new RdbColumnInfo[columns.length];
       	for(int i = 0; i < columns.length; i++){
       		columnsClone[i] = columns[i];
       	}
       	//sort the table
       	Arrays.sort(columnsClone);
       	Collection c = new ArrayList();
       	Collection superClasses = new ArrayList();
        //add the current class name to the list of super classes, 
        //not to forget all the fields belonging to this class
        superClasses.add(cl.getFQName());
        //get all the ancestors of the class cl
        Iterator it = cl.getAncestors().iterator();
        //and add the names of these classes to the superClasses list
        while(it.hasNext()){
        	Class currentClass = (Class) it.next();
           	superClasses.add(currentClass.getFQName());
        }
        for (int pos = 0; pos < columnsClone.length; pos++) {
          	//add the field to the list only if it belongs to one of the super classes
           	//including the class cl
           	if (superClasses.contains(columnsClone[pos].getClassName())) {
        		c.add(columnsClone[pos].getAlias());
           	}
        }
        return c;
    }
        
    public Collection getColumnClasses(){
    	RdbColumnInfo[] columnsClone = new RdbColumnInfo[columns.length];
       	for(int i = 0; i < columns.length; i++){
       		columnsClone[i] = columns[i];
       	}
       	//sort the table
       	Arrays.sort(columnsClone);
       	ArrayList columnClasses = new ArrayList();
       	for(int i = 0; i < columnsClone.length; i++){
         	columnClasses.add(columnsClone[i].getClassName());
        }
        return columnClasses;
    }
        
    public boolean mappingDone(int pos) {
        return (columns[pos] != null && columns[pos].getName() != null);
    }
        
    public boolean mappingNull(int pos) {
        return (columns[pos] == null || columns[pos].getName().equals(NULL_COLUMN));
    }
        
    public boolean hasJoins() {
        return (!joins.isEmpty());
    }
}