/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.lib.AbstractPMapcluster;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.HashSet;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Defines the PMapCluster used to manage relational schema for the RDB
 * mappers.
 * @author	P. Dechamboux
 */
public class RdbPMapCluster extends AbstractPMapcluster {

    
	/**
	 * The list of table managed by this PMapCluster. It contains Table instance.
	 */
	private ArrayList tables;
    
	private RdbPMappingStructuresManager manager;
    private RdbAdapter adapter;
	/**
	 * Constructs a RdbPMapCluster for an abstract class.
	 * @param jcname	The first JORM class name to be assigned to this cluster.
	 * @param man		The manager associated to these mapping structures.
	 */
	RdbPMapCluster(String jcname, RdbPMappingStructuresManager man) {
		super();
		manager = man;
		jcNames = new HashSet(1);
		unresolvedDependencies.add(jcname);
		tables = new ArrayList(1);
		structuresActive = false;
        if (this.manager.getPMapper() != null) {
            adapter = ((PMapperRdb) this.manager.getPMapper()).getRdbAdapter();
        }
	}

	/**
	 * Constructs a RdbPMapCluster.
	 * @param jcname	The first JORM class name to be assigned to this cluster.
	 * @param tname		The first table name to be assigned to this cluster.
	 * @param man		The manager associated to these mapping structures.
	 */
	RdbPMapCluster(String jcname, String tname, RdbPMappingStructuresManager man) {
		this(jcname, man);
		tables.add(new Table(tname, this));
	}


	/**
	 * Defines a new table column into this map cluster if it does not already
	 * exist. If it exist, it verifies that the redefinition is compatible with
	 * existing information.
	 * @param tname		The table name.
	 * @param cname		The column name.
	 * @param type		The SQL type of this column.
	 * @param notnull	The notnull status of this column.
	 * @throws PException Thrown if the column is redefined with incompatible
	 * information.
	 */
	void addTableColumn(String tname, String cname, String type,
						boolean notnull, boolean ispkcol, boolean isMaster) throws PException {
		Table curt = null;
		Iterator it = tables.iterator();
		while (it.hasNext()) {
			curt = (Table) it.next();
			if (curt.name.equals(tname)) {
				break;
			}
			curt = null;
		}
		if (curt == null) {
			// no such table defined yet: create a new one
			curt = new Table(tname, this);
			tables.add(curt);
		}
		Column c = curt.getColumn(cname);
		if (c == null) {
			curt.addColumn(cname, type, notnull, ispkcol, isMaster);
		} else if (isMaster) {
			if (c.isMaster) {
				if (!c.type.equals(type)) {
					if (ispkcol && !c.isPkCol) {
						c.notNull = notnull;
						c.isPkCol = ispkcol;
						c.type = type;
					} else if (!ispkcol && c.isPkCol) {
						//nothing to do
					} else {
						throw new PException(
								"Incompatible column definition: existing: " + c
								+ " asked : Column (name:" + cname
								+ ", type:" + type
								+ ", notNull:" + notnull
								+ ", isMaster:" + isMaster
								+ ", isPkCol:" + ispkcol
								+ ")");
					}
				}
			} else {
				c.isMaster = true;
				c.notNull = notnull;
				c.isPkCol = ispkcol;
				c.type = type;
			}
		}
	}
        
   



	/**
	 * Looks for this specified column into this map cluster.
	 * @param tname		The table name.
	 * @param cname		The column name.
	 * @param type		The SQL type of this column.
	 * @param notnull	The notnull status of this column.
	 * @throws PException Thrown if the column is redefined with incompatible
	 * information.
	 */
	boolean containColumn(String tname, String cname, String type,
						  boolean notnull) throws PException {
		Table curt = null;
		Iterator it = tables.iterator();
		while (it.hasNext()) {
			curt = (Table) it.next();
			if (curt.name.equals(tname)) {
				break;
			}
			curt = null;
		}
		if (curt == null) {
			// no such table defined yet : create a new one
			return false;
		}
		return curt.containColumn(cname, type, notnull);
	}

	/**
	 * Looks for this table name into this map cluster.
	 * @param tname	The table name.
	 * @return true if this table name is already assigned to this cluster.
	 */
	boolean containTable(String tname) {
		Iterator it = tables.iterator();
		while (it.hasNext()) {
			Table cur = (Table) it.next();
			if (cur.name.equals(tname))
				return true;
		}
		return false;
	}

	/**
	 * Looks for this table name into this map cluster.
	 * @param tname	The table name.
	 * @return true if this table name is already assigned to this cluster.
	 */
	Table getTable(String tname) {
		Iterator it = tables.iterator();
		while (it.hasNext()) {
			Table cur = (Table) it.next();
			if (cur.name.equals(tname))
				return cur;
		}
		return null;
	}

	/**
	 * Merges map cluster passed in parameter with this one. Clusters are merged
	 * either because they share a table name or a JORM class name.
	 * @param cl
	 */
	void merge(RdbPMapCluster cl) throws PException {
		jcNames.addAll(cl.jcNames);
		unresolvedDependencies.addAll(cl.getUnResolvedDependencies());
		unresolvedDependencies.removeAll(jcNames);
		for (int i = 0; i < cl.tables.size(); i++) {
			Table cur = (Table) cl.tables.get(i);
			Table t = getTable(cur.name);
			if (t == null) {
				tables.add(cur);
			} else {
				t.merge(cur);
			}
		}
	}
    
   


	// IMPLEMENTATION OF METHODS FROM THE PMapCluster INTERFACE


	/**
	 * Creates the mapping structures defined by this map cluster.
	 * @param force	If true, the creation of an existing table leads to an exception.
	 * @throws PException   Thrown if it cannot be performed (especially
	 * if all or some of these mapping structures already exist.
	 */
	public void createMappingStructures(boolean force) throws PException {
		if (structuresActive) {
			throw new PException("Cannot change mapping structures while they are under use");
		}
		if (!isDefined()) {
			throw new PException("Cannot create hosting struture: the " +
								 "following dependencies has not been resolved: "
								 + unresolvedDependencies);
		}
		if ((manager instanceof RdbScriptWriter) 
				&& ((RdbScriptWriter)manager).isWriteScript()) {
			// generate create statements
			Iterator it = tables.iterator();
			while (it.hasNext()) {
				Table table = (Table) it.next();
				table.create(null);
			}
		} else {
			Object conn = manager.getConnection();
			try {
            
				// creates the tables
				Iterator it = tables.iterator();
				while (it.hasNext()) {
					Table table = (Table) it.next();
					if (table.exist(RdbConnectionWrapper.narrow2SQL(conn), adapter)) {
						if (force) {
							throw new PException("Cannot create table " + table.name + ": already exist");
						}
					} else {
						table.create(RdbConnectionWrapper.narrow2SQL(conn));
					}
				}
			
            
			} finally {
				manager.closeConnection(conn);
			}
		}
	}

	/**
	 * Deletes the data that have been inserted into the mapping structures
	 * defined by this map cluster.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 */
	public void deleteData() throws PException {
		if (!isDefined()) {
			throw new PException("Cannot remove data: the " +
								 "following dependencies has not been resolved: "
								 + unresolvedDependencies);
		}
		if ((manager instanceof RdbScriptWriter) 
				&& ((RdbScriptWriter)manager).isWriteScript()) {
			//generate delete statements
			Iterator it = tables.iterator();
			while (it.hasNext()) {
				Table table = (Table) it.next();
				table.deleteData(null);
			}
		} else {
			Object conn = manager.getConnection();
			try {
				Iterator it = tables.iterator();
				while (it.hasNext()) {
					Table table = (Table) it.next();
					if (table.exist(RdbConnectionWrapper.narrow2SQL(conn), adapter)) {
						table.deleteData(RdbConnectionWrapper.narrow2SQL(conn));
					} else {
						manager.logger.log(BasicLevel.WARN,
									   "Impossible to delete the data of the table '"
									   + table.name + "': table does not exist.");
					}
				}
			} finally {
				manager.closeConnection(conn);
			}
		}
	}

	/**
	 * Deletes all or part of the mapping structures defined by this map
	 * cluster when they exist. If they contain some data, they are also
	 * deleted.
	 * @throws PException   Thrown when the data store cannot perform this
	 * operation.
	 */
	public void deleteMappingStructures() throws PException {
		if (structuresActive) {
			throw new PException("Cannot change mapping structures while they are under use");
		}
		if (!isDefined()) {
			throw new PException("Cannot delete hosting struture: the " +
								 "following dependencies has not been resolved: "
								 + unresolvedDependencies);
		}
		if ((manager instanceof RdbScriptWriter) 
				&& ((RdbScriptWriter)manager).isWriteScript()) {
			//generate drop statements
			Iterator it = tables.iterator();
			while (it.hasNext()) {
				Table table = (Table) it.next();
				table.delete(null);
			}
		} else {
			Object conn = manager.getConnection();
			try {
				// deletes tables
				Iterator it = tables.iterator();
				while (it.hasNext()) {
					Table table = (Table) it.next();
					if (table.exist(RdbConnectionWrapper.narrow2SQL(conn), adapter)) {
						table.delete(RdbConnectionWrapper.narrow2SQL(conn));
					}
				}
			} finally {
				manager.closeConnection(conn);
			}
		}
	}

	public RdbPMappingStructuresManager getManager() {
		return manager;
	}
}


/**
 * Describes a table into a map cluster.
 */
class Table {
	String name;
	ArrayList columns;
	RdbPMapCluster cluster;


	Table(String tname, RdbPMapCluster cluster) {
		name = tname;
		this.cluster = cluster;
		columns = new ArrayList();
	}

	void addColumn(String cname, String type, boolean notnull, boolean ispkcol, boolean ismaster)
			throws PException {
		Column cur;
		cur = new Column();
		cur.name = cname;
		cur.type = type;
		cur.notNull = notnull;
		cur.isPkCol = ispkcol;
		cur.isMaster = ismaster;
		columns.add(cur);
	}

	void merge(Table t) throws PException {
		for (int i = 0; i < t.columns.size(); i++) {
			Column cur = (Column) t.columns.get(i);
			Column c = getColumn(cur.name);
			if (c == null) {
				columns.add(cur);
			} else {
				c.merge(cur);
			}
		}
	}

	boolean containColumn(String cname, String type,
						  boolean notnull) throws PException {
		Column cur;
		Iterator it = columns.iterator();
		while (it.hasNext()) {
			cur = (Column) it.next();
			if (cur.name.equals(cname)) {
				return true;
			}
		}
		return false;
	}

	Column getColumn(String cname) {
		Column cur;
		Iterator it = columns.iterator();
		while (it.hasNext()) {
			cur = (Column) it.next();
			if (cur.name.equals(cname)) {
				return cur;
			}
		}
		return null;
	}

	void create(Connection connection) throws PException {
		Statement stmt = null;
		StringBuffer sb = new StringBuffer("CREATE TABLE " + name + " (");
		try {
			String sep = "";
			Iterator it = columns.iterator();
			while (it.hasNext()) {
				sb.append(sep);
				sep = ", ";
				Column col = (Column) it.next();
				sb.append(col.name);
				sb.append(" ");
				sb.append(col.type);
				if (col.notNull && col.type.indexOf("NOT NULL") == -1) {
					sb.append(" NOT NULL");
				}
			}
			sep = "";
			it = columns.iterator();
			while (it.hasNext()) {
				Column col = (Column) it.next();
				if (!col.isPkCol) {
					continue;
				}
				if (sep.equals("")) {
					sb.append(", PRIMARY KEY (");
				}
				sb.append(sep);
				sep = ", ";
				sb.append(col.name);
			}
			if (!sep.equals("")) {
				sb.append(")");
			}
			sb.append(")");
			
			//if generate script, then write into a file
			//else, execute statement on the database
			if(cluster.getManager() instanceof RdbScriptWriter && ((RdbScriptWriter)cluster.getManager()).isWriteScript()) {
				FileWriter writer = new FileWriter(((RdbScriptWriter)cluster.getManager()).getFileName(), true);
				writer.write(sb.toString() + ";\n");
				writer.close();
			} else {
				stmt = connection.createStatement();
				stmt.execute(sb.toString());
			}
		} catch (SQLException se) {
			throw new PExceptionIO(se, "Impossible to create the table '"
					+ name + "' of the cluster " + cluster.getClusterClasses()
					+ " : " + sb.toString());
		} catch(IOException ioe) {
			throw new PExceptionIO(ioe, "Problem while writting create statement in the sql script file");
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	void deleteData(Connection connection) throws PException {
		Statement stmt = null;
		PException resex = null;
		try {
			StringBuffer sb = new StringBuffer("DELETE FROM " + name);
			//if generate script, then write into a file
			//else, execute statement on the database
			if(cluster.getManager() instanceof RdbScriptWriter && ((RdbScriptWriter)cluster.getManager()).isWriteScript()) {
				FileWriter writer = new FileWriter(((RdbScriptWriter)cluster.getManager()).getFileName(), true);
				writer.write(sb.toString() + ";\n");
				writer.close();
			} else {
				stmt = connection.createStatement();
				stmt.execute(sb.toString());
			}
		} catch (SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while deleting data: class <" + name + ">");
		} catch(IOException ioe) {
			throw new PExceptionIO(ioe, "Problem while writting delete statement in the sql script file");
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
					if (resex == null) {
						resex = new PException(e, "Problem while closing statment.");
					}
				}
			}
			if (resex != null) {
				throw resex;
			}
		}
	}

	void delete(Connection connection) throws PException {
		Statement stmt = null;
		PException resex = null;
		try {
			StringBuffer sb = new StringBuffer("DROP TABLE " + name);
			//if generate script, then write into a file
			//else, execute statement on the database
			if(cluster.getManager() instanceof RdbScriptWriter && ((RdbScriptWriter)cluster.getManager()).isWriteScript()) {
				FileWriter writer = new FileWriter(((RdbScriptWriter)cluster.getManager()).getFileName(), true);
				writer.write(sb.toString() + ";\n");
				writer.close();
			} else {
				stmt = connection.createStatement();
				stmt.execute(sb.toString());
			}
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se);
		} catch(IOException ioe) {
			throw new PExceptionIO(ioe, "Problem while writting drop table statement in the sql script file");
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
					if (resex == null) {
						resex = new PException(e, "Problem while closing statment.");
					}
				}
			}
			if (resex != null) {
				throw resex;
			}
		}
	}

	boolean exist(Connection connection, RdbAdapter adapter) throws PException {
		try {
			return adapter.existTable(connection, name);
		} catch (SQLException e) {
			throw new PException(e, "Pb while testing the existence of table " + name);
		}
	}
}

/**
 * Describes a column into a table description into a map cluster.
 */
class Column {
	String name;
	String type;
	boolean notNull;
	boolean isPkCol;
	boolean isMaster;

	void merge(Column c) throws PException {
		if (type != c.type) {
			throw new PException("Impossible to merge the column " + name
								 + ": column type does not match");
		}
		if (c.isMaster) {
			notNull = c.notNull;
			isPkCol |= c.isPkCol;
		}
	}

	public String toString() {
		return "Column (name:" + name
				+ ", type:" + type
				+ ", notNull:" + notNull
				+ ", isMaster:" + isMaster
				+ ", isPkCol:" + isPkCol
				+ ")";
	}
}
