/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.mapper.rdb.metainfo;

import org.objectweb.jorm.naming.api.NamingFilterKeyProvider;
import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.lib.BasicClassMapping;
import org.objectweb.jorm.metainfo.lib.FieldComparator;
import org.objectweb.jorm.metainfo.lib.BasicParentClassMapping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.BasicVariableOperand;
import org.objectweb.medor.expression.lib.Equal;
import org.objectweb.medor.expression.lib.ExpressionPrinter;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Collections;
import java.util.Set;
import java.util.HashSet;
import java.text.DateFormat;
import java.text.ParseException;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * This class defines information on the mapping of a class to relational structures
 * In the case of inheritance, one must defines how the mapping definitions are
 * inherited, and how to distinguish instances of the class to instances of
 * super or sub classes.
 * Following the partitionning schemes in relational database we defines three
 * ways to inherit the mapping of its parent.
 *
 *
 * In the case of multi-inheritance, all combinations are valid. One can map new
 * fields relative to class A to new columns or tables (to-extended or to-added rules)
 * while the resulting stuctures being completely redefined relatively to the ones
 * of another class B (to-new rule). One class can also inherits with a to-added
 * or to-extended rule from class A and also with a to-added or to-exended rule from
 * another class B provided they map their id the same way, which implies that
 * they have the same main table.
 *
 * One limitation is relative to the "abscractness" of the classes. In fact,
 * the inheritance rules are those of Java. That is, a class cannot inherit
 * from more than one concrete class (the 'extends' keyword is limited to only
 * one class). Note that Jorm support abstract classes inheriting from concrete
 * classes, even though this has no sense from a persistance point of view. The
 * reason is that Jorm is used to map Java classes to persistant ones, that
 * Java authorizes abstract classes deriving from concrete classes by defining
 * new abstract methods. Even though methods are not translated in a persistance point
 * of view since they do not represent fields (exception made of accessors),
 * we need to represent the abstract classes since there may exist references
 * of this abstract class.
 *
 */
public class RdbClassMapping extends BasicClassMapping {
    /**
     * Specifies that the structures and the mapping of the parent is inherited,
     * the structures are extended (i.e. new columns are defined in existing tables)
     * and the new fields are mapped to this extension. This merely corresponds to
     * a 'filtered' case but can cover the case of concrete classes where a filter
     * can be defined, and the case of abstract classes where only the mapping
     * is extended and no filter is defined. This implies that concrete classes
     * having this inheritance rule MUST define a filter.
     * Note that when no parent class mapping is defined, a default one is created with this
     * rule, with no new mapping and no filters. That is, the mapping and structures
     * are inherited as is, unchanged. This prevent us from defining a new 'unchanged'
     * rule.
     */
   	public final static String MAP_NEW_FIELDS_TO_EXTENDED_STRUCTURES = "to-extended-structures";

    /**
     * Specifies that the structures and the mapping of the parent is inherited,
     * new structures are added (i.e. new tables are added) and the some of new fields
     * are mapped to this new structures. Note that this rule 'extends' the previous one,
     * that is, some of the new fields may be mapped to new columns of inherited tables
     * This merely corresponds to a 'vertical' case supporting the definition of a filter (for efficiency reasons).
     */
	public final static String MAP_NEW_FIELDS_TO_ADDED_STRUCTURES = "to-added-structures";

    /**
     * Specifies that neither the structures nor the mapping is inherited,
     * structures are completely redefined (i.e. new tables are defined)
     * and ALL the fields (defined in the class or previously defined in super
     * classes are remapped to this new structures. This merely corresponds to
     * a 'horizontal' case.
     */
	public final static String REMAP_FIELDS_TO_NEW_STRUCTURES = "to-new-structures";

    /**
     * ==== NOT USED ANYMORE ====
     * Specifies the structures and the mapping are left unchanged.
     * The class is necessarily abstract, and if it defines new fields, they are
     * not defined here. This is the default case when no mapping rule is defined.
     * Classes that are gathering of other classes (that their extent is the
     * union of its subclasses' extents) comes into this category.
     */
 //   public final static String MAPPING_UNCHANGED = "unchanged";

    /**
     * The (main) table of which the fields are mapped
     */
	RdbTable table = null;

    /**
     *
     */
    RdbFilter filter = null;


    /**
     * 
     */
    RdbInheritanceQuery inheritanceQuery = null;
    
	/**
	 * Builds a new BasicRdbClassMapping object.
	 *
	 * This object contains the mapping structure of the class it refers to.
	 * The parent object is a Mapping object that contains the mapper name.
	 *
	 * @param ruleName  the name of the rule used to map the class,
	 *        linkedMo	the Class object referenced by the current object,
	 *        parent	the parent of the current object.
	 */
	public RdbClassMapping(String ruleName, MetaObject linkedMO,
								MetaObject parent) {
		super(ruleName, linkedMO, parent);
	}

	///////////////////////////////////////////////////////////////////
	//                      from RdbClassMapping interface
	///////////////////////////////////////////////////////////////////
	public List getAllPrimitiveElementMappings() {
		List pems = super.getAllPrimitiveElementMappings();
		List pcms = (List) getParentClassMappings();
		int i;
		for(i = pcms.size()-1; i>=0; i--) {
			ParentClassMapping pcm = (ParentClassMapping) pcms.get(i);
			if ((MAP_NEW_FIELDS_TO_EXTENDED_STRUCTURES.equalsIgnoreCase(pcm.getRuleName())) ||
                    (MAP_NEW_FIELDS_TO_ADDED_STRUCTURES.equalsIgnoreCase(pcm.getRuleName()))) {
				//Add the the pem of the super class
				Class superClass = pcm.getMOClass();
				ClassProject cp = superClass.getClassProject(getProjectName());
				if (cp != null) {
					Mapping m = cp.getMapping(getMapperName());
					if (m != null) {
						//fetch the PEMs of the super class (Recusrive call)
						pems.addAll(m.getClassMapping().getAllPrimitiveElementMappings());
					}
				}
			}
		}
		if (i == -1) {
			Collections.sort(pems, FieldComparator.instance);
		}
		return pems;
	}
	
	public RdbTable createRdbTable(String tableName) {
		if (table == null) {
			table = new RdbTable(this, getLinkedMO(), tableName);
			table.setLogger(getLogger());
			table.setLoggerFactory(getLoggerFactory());
		} else {
			table.setName(tableName);
		}
		return table;
	}

	public RdbTable getRdbTable() {
		return table;
    }
    /**
     * Get the main table (tables ??).
     * OL: Gets the RdbTable meta-objects relative to the main tables of
     * ancestors while it is the same main table (which is the case
     * with 'extended' and 'added' inheritance rules)
     * @param res the list of RdbTable meta-object in ancestors relative to the
     *  the main table of the class
     */
	public void getMainRdbTableOld(ArrayList res) {
        if ( !getParentClassMappings().isEmpty()) {
		    for (Iterator it = getParentClassMappings().iterator(); it.hasNext();) {
			    ParentClassMapping pcm = (ParentClassMapping) it.next();
                String ruleName = pcm.getRuleName();
			    //System.out.println("rule-name = " + ruleName);
                if (MAP_NEW_FIELDS_TO_ADDED_STRUCTURES.equals(ruleName) ||
                        MAP_NEW_FIELDS_TO_EXTENDED_STRUCTURES.equals(ruleName)) {
                    Class parentClass = pcm.getMOClass();
                    RdbClassMapping classmapping =
                            (RdbClassMapping) parentClass.getClassMapping(getProjectName(),getMapperName());
                    classmapping.getMainRdbTableOld(res);
                } else {
                    // new tables at this level (horizontal)
                    res.add(table);
                }
			}
		} else {
            // There is no extension.
            //logger.log(BasicLevel.DEBUG, "no extension");
            res.add(table);
        }
	}

    public RdbTable getMainRdbTable() {
        // if main table is null (this can happen if the classmapping is implicit)
        // gets the main table of the super class
        if (table == null) {
            // get a super class (one among all since they should be the same)
            ParentClassMapping aPcm = (ParentClassMapping)
                    parentClassMappings.get(0);
            Class aSuperClass = aPcm.getMOClass();
            RdbClassMapping aSuperCm =
                    (RdbClassMapping) aSuperClass.getClassMapping(getProjectName(),
                            getMapperName());
            table = aSuperCm.getMainRdbTable();
        }
        return table;
    }

    public RdbFilter createRdbFilter() {
       filter = new RdbFilter(this);
       return filter;
    }

    public RdbFilter getRdbFilter() {
		return filter;
	}
    
	public RdbInheritanceQuery getRdbInheritanceQuery() {
		if(inheritanceQuery == null)
			createRdbInheritanceQuery();
		return inheritanceQuery;
	}

	public RdbInheritanceQuery createRdbInheritanceQuery() {
		if(inheritanceQuery == null)
			inheritanceQuery = new RdbInheritanceQuery(this);
		return inheritanceQuery;
	}
		
	public Expression getMappingFilterFromNamingFilter()
		throws ExpressionException {
		return getMappingFilterFromNamingFilter(null);
	}
	public Expression getMappingFilterFromNamingFilter(NamingFilterKeyProvider nfkp)
			throws ExpressionException {
		if (debug) {
			logger.log(BasicLevel.DEBUG,
				"getMappingFilterFromNamingFilter: class "
				+ ((Class) linkedMO).getFQName());
		}
		NameDef nd = getIdentifierMapping().getNameDef();
		if (nd == null) {
			if (debug) {
				logger.log(BasicLevel.DEBUG,
					"getMappingFilterFromNamingFilter: null NameDef");
			}
			return null;
		}
		Class clazz = getJormClass();
		Expression namingFilter = null;
		namingFilter = clazz.getInheritanceFilter(nd);
		Object key;
		if (nfkp != null) {
			//fetch the key value from the NamingFilterKeyProvider
			// This case happens when the key value is calculated at runtime for
			// example.
			key = nfkp.getNamingFilterKey();
		} else {
			//Fetch the key value from the meta information
			key = clazz.getInheritanceNamingKey(nd);
		}
		if (debug) {
			logger.log(BasicLevel.DEBUG,
				"getMappingFilterFromNamingFilter: key=" + key
				+ " / namingFilter=" + namingFilter);
		}
		if (key != null) { //A key value is specified on the current class ==> FKPNC
			if (namingFilter == null) {
				logger.log(BasicLevel.DEBUG, "Find the naming filter on ancestors");
				ArrayList al = new ArrayList(clazz.getSuperClasses());
				while (namingFilter == null && !al.isEmpty()) {
					Class superClass = (Class) al.remove(0);
					logger.log(BasicLevel.DEBUG, "Ancestor " + superClass.getFQName());
					namingFilter = superClass.getInheritanceFilter(nd);
					if (namingFilter == null) {
						al.addAll(superClass.getSuperClasses());
					}
				}
				if (namingFilter == null) {
					logger.log(BasicLevel.DEBUG, "No the naming filter found on ancestors");
					//not found ==> no filter
					return null;
				} else {
					logger.log(BasicLevel.DEBUG, "Naming filter found on ancestors: " + ExpressionPrinter.e2str(namingFilter));
				}
			} // else the filter is also on the current class

			logger.log(BasicLevel.DEBUG, "Compute the key filter with key value");
			//the filter is based on key principle => add the equality between
			// the key value and the expression
			BasicVariableOperand keyexp = new BasicVariableOperand(namingFilter.getType());
			if (key instanceof String) {
				assignKey(keyexp, (String) key);
			} else {
				keyexp.setValue(key);
			}
			namingFilter = new Equal(keyexp, namingFilter);
			logger.log(BasicLevel.DEBUG, "Naming filter changed into Equal: " + ExpressionPrinter.e2str(namingFilter));

		} else { // no key define on the current class ==> FPNC
			if (namingFilter == null) {
				return null;
			}
		}
		//changes are done here!!
		Expression res = cloneAndReplaceLeaves(namingFilter);
		if (debug) {
			logger.log(BasicLevel.WARN, "Filter=" + ExpressionPrinter.e2str(res));
		}
		return res;
	}

	private void assignKey(BasicVariableOperand keyexp,
						   String str) throws ExpressionException {
		switch (keyexp.getType().getTypeCode()) {
		case PType.TYPECODE_BYTE:
			keyexp.setValue(Byte.parseByte(str));
			break;
		case PType.TYPECODE_CHAR:
			keyexp.setValue(str.charAt(0));
			break;
		case PType.TYPECODE_SHORT:
			keyexp.setValue(Short.parseShort(str));
			break;
		case PType.TYPECODE_INT:
			keyexp.setValue(Integer.parseInt(str));
			break;
		case PType.TYPECODE_LONG:
			keyexp.setValue(Long.parseLong(str));
			break;
		case PType.TYPECODE_OBJBYTE:
			keyexp.setValue(Long.valueOf(str));
			break;
		case PType.TYPECODE_OBJCHAR:
			keyexp.setValue(new Character(str.charAt(0)));
			break;
		case PType.TYPECODE_OBJSHORT:
			keyexp.setValue(Short.valueOf(str));
			break;
		case PType.TYPECODE_OBJINT:
			keyexp.setValue(Integer.valueOf(str));
			break;
		case PType.TYPECODE_OBJLONG:
			keyexp.setValue(Long.valueOf(str));
			break;
		case PType.TYPECODE_DATE:
			try {
				keyexp.setValue(DateFormat.getInstance().parse(str));
			} catch (ParseException e) {
				throw new ExpressionException("Impossible to parse the date " + str, e);
			}
			break;
		case PType.TYPECODE_STRING:
			keyexp.setValue(str);
			break;
		case PType.TYPECODE_BIGDECIMAL:
			keyexp.setValue(new BigDecimal(str));
			break;
		case PType.TYPECODE_BIGINTEGER:
			keyexp.setValue(new BigInteger(str));
			break;
		default:
			throw new ExpressionException("Umanaged key type: "
				+ keyexp.getType().getJavaName());
		}
	}


	/**
	 * Clones an Expression, where leaves which point to naming fields are
	 * replaced with leaves pointing to class fields.
	 * @param exp
	 * @return
	 * @throws ExpressionException
	 */
	private Expression cloneAndReplaceLeaves(Expression exp)
		throws ExpressionException {
		if (exp instanceof ParameterOperand) {
			//field to replace
			NameDef nd = getIdentifierMapping().getNameDef();
			String classFieldName = null;
			if (nd.isFieldName()) {
				if (debug) { 
					logger.log(BasicLevel.DEBUG, "nd.getFieldName = " + nd.getFieldName() 
						+ " while exp.getName = " + ((ParameterOperand) exp).getName());
				}
				return exp;
			} else if (nd.isNameRef()) {
				classFieldName  = (String) nd.getNameRef().getProjection().get(
					((ParameterOperand) exp).getName());
				//if the field is not part of the namedef, just keep the original name
				if (classFieldName == null) {
					return exp;
				}
				if (debug) {
					logger.log(BasicLevel.DEBUG, "nref.getFieldName = " + classFieldName 
						+ " while exp.getName = " + ((ParameterOperand) exp).getName());
				}
			} else {
				throw new ExpressionException("Naming type not managed: " + nd);
			}
			return new BasicParameterOperand(exp.getType(), classFieldName);
		} else if (exp instanceof BasicVariableOperand) {
			//constant
			return new BasicVariableOperand((BasicOperand) exp);
		} else if (exp instanceof BasicOperand) {
			//constant
			return new BasicOperand((BasicOperand) exp);

		} else if (exp instanceof Operator) {
			Operator oldop = (Operator) exp;
			Operator newop = null;
			try {
				newop = (Operator) exp.getClass().newInstance();
			} catch (Exception e) {
				throw new ExpressionException(
					"Impossible to clone the operator "
					+ exp.getClass().getName() + ": ", e);
			}
			for(int i=(oldop.getOperandNumber() - 1); i>=0; i--) {
				//recursion using clone of operator and clones of children
				newop.setExpression(i, cloneAndReplaceLeaves(oldop.getExpression(i)));
			}
			return newop;
		} else {
			throw new ExpressionException("UnManaged expression: " + exp);
		}
	}

	protected Collection getChildren() {
		Collection col = super.getChildren();
		if (table != null) {
			col.add(table);
		}
		return col;
	}

    public ParentClassMapping createImplicitParentClassMapping(Class superClass) {
        BasicParentClassMapping pcm =
                new BasicParentClassMapping(MAP_NEW_FIELDS_TO_EXTENDED_STRUCTURES,
                        superClass, this);
        addParentClassMapping(pcm);
        return pcm;
    }

    /**
     * The creation of tables cannot be done until all columns are known and those
     * columns can be defined in the super and subclasses with wich there is
     * a 'to-extended' or a 'to-added' inheritance rule.
     *
     */
    public void addImplicitDependencies() {

        // treat inheritance with its super classes
        Class clazz = (Class) getLinkedMO();
        for (Iterator itPcm = getParentClassMappings().iterator(); itPcm.hasNext();) {
            ParentClassMapping parentClassMapping = (ParentClassMapping) itPcm.next();
            // The creation the structures of a class requires that the structures are completely
            // defined. In case of 'added' and 'extended' inheritance rules, structures (i.e. table) are 
            // shared, so the definition of the columns mapping fields of its super classes must be done. 
            if (inheritsStructures(parentClassMapping)) {
                Class superClass = (Class) parentClassMapping.getLinkedMO();
                if (debug) {
                    logger.log(BasicLevel.DEBUG, "Add dependency: " +
                            clazz.getName() + " -> " + superClass.getName());
                }
                addDependency(superClass.getFQName());
            }

        }

        // treat inheritance with its sub classes
        for (Iterator itClass = clazz.getSubClasses().iterator(); itClass.hasNext();) {
            Class subClass = (Class) itClass.next();
            ClassMapping subClassMapping = subClass.getClassMapping(getProjectName(), getMapperName());
            if (subClassMapping == null) {
                throw new InternalError("Cannot find class mapping of "
                        + subClass.getName() + "<" + getProjectName() + ","
                        + getMapperName() + ">" +
                        " when adding implicit dependencies to " + clazz.getName());
            }
            ParentClassMapping parentClassMapping = subClassMapping.getParentClassMapping(clazz.getFQName());
            // The creation of the structures of a class requires that the structures
            // of ALL its subclasses are created, and, therefore, defined. This is required because 
            // the creation of the extents of its full-extent will required that the
            // structures of the subclasses are defined before. 
            // Hence, we do not test the inheritency rule.
            // String inheritanceRule = parentClassMapping.getRuleName();
            if (debug) {
                logger.log(BasicLevel.DEBUG, "Add dependency: " +
                        clazz.getName() + " -> " + subClass.getName());
            }
            addDependency(subClass.getFQName());
        }
    }
    
    /**
     * Check if any (hidden) field defined in the class (or its supers) has
     * a defined mapping (defined in the class mapping or in it supers)
     * @return true if there is at least one (hidden) field whch is unmapped
     */
    public boolean hasUnmappedPrimitiveElements(Collection pes) {
        Class clazz = (Class) linkedMO;
        // search if there is a field which is not mapped
        for (Iterator itPE = pes.iterator(); itPE.hasNext();) {
                PrimitiveElement pe = (PrimitiveElement) itPE.next();
                PrimitiveElementMapping pem = getPrimitiveElementMapping(pe.getName(), true);
                if (pem == null) {
                    return true;
                }
            }
        return false;
    }

    /**
     * Returns the parent class mappings having a inheritance rule in a
     * set of inheritance rules.
     * @param  inheritanceRules the set of inheritance rule names
     * @return the parent class mappings
     */
    public Set getParentClassMapping(Set inheritanceRules) {
        HashSet res = new HashSet();
        for (Iterator it = parentClassMappings.iterator(); it.hasNext();) {
            ParentClassMapping pcm = (ParentClassMapping) it.next();
            if (inheritanceRules.contains(pcm.getRuleName())) {
                res.add(pcm);
            }
        }
        return res;
    }

    /**
     * Returns the parent class mappings having an extened or added inheritance
     * rule.
     * @return the parent class mappings having an extened or added inheritance
     * rule.
     */
    public Set getAddedOrExtendedParentClassMapping() {
        HashSet res = new HashSet();
        for (Iterator it = parentClassMappings.iterator(); it.hasNext();) {
            ParentClassMapping pcm = (ParentClassMapping) it.next();
            if (inheritsStructures(pcm)) {
                res.add(pcm);
            }
        }
        return res;
    }

    public boolean inheritsStructures(ParentClassMapping pcm) {
        return (RdbClassMapping.MAP_NEW_FIELDS_TO_EXTENDED_STRUCTURES.
                    equalsIgnoreCase(pcm.getRuleName()) ||
                    RdbClassMapping.MAP_NEW_FIELDS_TO_ADDED_STRUCTURES.
                    equalsIgnoreCase(pcm.getRuleName()));
    }

    public boolean inheritsStructures() {
        for(Iterator it= getParentClassMappings().iterator(); it.hasNext();) {
        ParentClassMapping pcm = (ParentClassMapping) it.next();
            if (inheritsStructures(pcm)) {
                return true;
            }
        }
        return false;
    }
    
    public PrimitiveElementMapping getPrimitiveElementMapping(String fieldName,
            boolean searchInSuper) {
        PrimitiveElementMapping pem = getPrimitiveElementMapping(fieldName);
        if ((pem == null) && searchInSuper) {
            Mapping mapping = (Mapping) getParent();
            String mapperName = mapping.getMapperName();
            String projectName = ((ClassProject) mapping.getParent()).getProjectName();
            for (Iterator itPcm = parentClassMappings.iterator(); itPcm.hasNext();) {
                ParentClassMapping pcm = (ParentClassMapping) itPcm.next();
                // remap inheritance rule does no inherits mappings
                if (RdbClassMapping.REMAP_FIELDS_TO_NEW_STRUCTURES.equals(pcm.getRuleName())) {
                    continue;
                }
                Class superClass = (Class) pcm.getLinkedMO();
                BasicClassMapping superClassMapping =
                    (BasicClassMapping) superClass.getClassMapping(projectName, mapperName);
                pem = superClassMapping.getPrimitiveElementMapping(fieldName, searchInSuper);
                if (pem != null) break;
            }
        }
        return pem;
    }

}
