/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.metainfo;

import org.objectweb.jorm.metainfo.api.*;
import org.objectweb.jorm.metainfo.api.Class;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * Implementation of the RdbClassMultiMapping interface.
 *
 * BasicRdbClassMultiMapping contains mapping information referring to:
 * - the primary table,
 * - external tables,
 * - join relationships between the primary table and an external table.
 */
public class RdbClassMultiMapping extends RdbClassMapping
        implements RdbMappingInfos {
	List tableNames = null;
	List tables = null;

	/**
	 * Builds a new BasicRdbClassMultiMapping object.
	 *
	 * This object contains the mapping structure of the class it refers to.
	 * The parent object is a Mapping object that contains the mapper name.
	 *
	 * @param ruleName  the name of the rule used to map the class,
	 *        linkedMo	the Class object referenced by the current object,
	 *        parent	the parent of the current object.
	 */
	public RdbClassMultiMapping(String ruleName, MetaObject linkedMO,
									 MetaObject parent) {
		super(ruleName, linkedMO, parent);
		tableNames = new ArrayList();
		tables = new ArrayList();
	}

	public PrimitiveElementMapping getPrimitiveElementMapping(String fieldName) {
		PrimitiveElementMapping pem = super.getPrimitiveElementMapping(fieldName);
		if (pem == null) {
			Iterator it = tables.iterator();
			while (pem == null && it.hasNext()) {
				pem = ((RdbExternalTable) it.next())
					.getPrimitiveElementMappingByField(fieldName);
			}
		}
		return pem;
	}

	///////////////////////////////////////////////////////////////////
	//                      from RdbClassMultiMapping interface
	///////////////////////////////////////////////////////////////////

	public RdbExternalTable createRdbExternalTable(String tableName) {
		int idx = tableNames.indexOf(tableName);
		RdbExternalTable t;
		if (idx == -1) {
			t = new RdbExternalTable(this, getLinkedMO(), tableName);
			t.setLogger(getLogger());
			t.setLoggerFactory(getLoggerFactory());
			tableNames.add(tableName);
			tables.add(t);
		}
		else {
			t = (RdbExternalTable) tables.get(idx);
		}
		return t;
	}

	public RdbExternalTable removeRdbExternalTable(String tableName) {
		int idx = tableNames.indexOf(tableName);
		if (idx != -1) {
			tableNames.remove(idx);
			return (RdbExternalTable) tables.remove(idx);
		}
		return null;
	}

	public Collection getRdbExternalTables() {
		return tables;
	}


	public void getAllRdbExternalTables(ArrayList res) {
        if ( !getParentClassMappings().isEmpty()) {
		    Iterator it = getParentClassMappings().iterator();
		    while (it.hasNext()) {
			    ParentClassMapping pcm = (ParentClassMapping) it.next();
                if (pcm.getRuleName().equals(REMAP_FIELDS_TO_NEW_STRUCTURES)) {
                    Class parentClass = pcm.getMOClass();
					//System.out.println("rule-name = vertical =<" + parentClass.getFQName() + ">");
                    RdbClassMultiMapping classmapping =
                        (RdbClassMultiMapping) parentClass.getClassMapping(getProjectName(),getMapperName());
                    classmapping.getAllRdbExternalTables(res);
                    // add the main table ?
                    // turns first the main table into an external table
                    //res.add(table);
                    // add the external tables
                    res.addAll(tables);
                }
                else {
                    // There is a filtered or an horizontal mapping.
					//System.out.println("rule-name = " + pcm.getRuleName());
                    res.addAll(getRdbExternalTables());
                }
			}
		}
        else {
            // There is no extension.
            res.addAll(tables);
        }
	}


	public RdbExternalTable getRdbExternalTable(String tableName) {
		int idx = tableNames.indexOf(tableName);
		if (idx != -1) {
			return (RdbExternalTable) tables.get(idx);
		}
		return null;
	}

	protected Collection getChildren() {
		Collection col = super.getChildren();
		col.addAll(tables);
		return col;
	}

    public RdbTable getMainTable() {
        return table;
    }

    public List getExternalTables() {
        return tables;
    }

    public List getExternalTableNames() {
        return tableNames;
    }

}
