/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.rdb.xml2mi;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbClassMultiMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbExternalTable;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbFilter;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbFilterParameterTypeProvider;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbGenClassMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbJoin;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbTable;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.xml2mi.lib.BasicMappingParser;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.lib.ExpressionPrinter;
import org.objectweb.medor.expression.parser.string.ExpressionParser;
import org.objectweb.util.monolog.api.BasicLevel;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.Iterator;


/**
 * RdbParser only parses the relational mapping specification in order to
 * build the corresponding Jorm meta information.
 */
public class RdbParser extends BasicMappingParser {

	//TODO
	public final static String IDVALUE_SEP = ".";
	public final static ExpressionParser expParser = new ExpressionParser();

	/**
	 * Builds a new rdb parser.
	 * This class is loaded by a classloader.
	 */
	public RdbParser() {
		// keep empty
	}

	/**
	 * Parses a relational class mapping element and builds the corresponding
	 * Jorm meta-information.
	 * Manager, PathExplorer and currentClass must be set before calling this
	 * method.
	 *
	 * @param classMappingElem   the current XML node,
	 * @param mapping    a Mapping object.
	 */
	private void parseClassMapping(Element classMappingElem, Mapping mapping) throws PException {
		//String ruleName = classMappingElem.getAttribute("rule-name");
        //System.out.println("currentclass=" + currentClass.getFQName());
		RdbClassMultiMapping rdbClassMapping = null;
		rdbClassMapping = ((RdbMapping) mapping).createClassMultiMapping("");
		rdbClassMapping.setLogger(logger);
		NodeList children = classMappingElem.getChildNodes();
        //link a RdBInheritanceQuery object to the rdbClassMapping
        rdbClassMapping.createRdbInheritanceQuery();
        logger.log(BasicLevel.DEBUG, "link rdbInheritanceQuery to the rdbClassMapping");
        for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String childName = child.getNodeName();
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "begin =<" + childName + ">");
			}
			if (childName.equals("colocated-class")) {
				String str = ((Element) child).getAttribute("class-name");
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, "add dependency: " + str);
				}
				rdbClassMapping.addDependency(str);
			} else if (childName.equals("rdb-table-spec")) {
				String tableName = ((Element) child).getAttribute("table-name");
				if (logger.isLoggable(BasicLevel.DEBUG))
					logger.log(BasicLevel.DEBUG, "Main table: " + tableName);
				RdbTable table = rdbClassMapping.createRdbTable(tableName);
				parseTableSpec((Element) child, table, currentClass);
			} else if (childName.equals("rdb-external-table-spec")) {
				String tableName = ((Element) child).getAttribute("table-name");
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, "External table: " + tableName);
				}
				RdbExternalTable table =
						rdbClassMapping.createRdbExternalTable(tableName);
				parseExternalTableSpec((Element) child, table, currentClass);
			} else if (childName.equals("rdb-filter-spec")) {
				parseFilter(
						(Element) child,
						rdbClassMapping.createRdbFilter(),
						(Class)rdbClassMapping.getLinkedMO());
			} else if (childName.equals("id-mapping")) {

				/* String elementId = ((Element) child).getAttribute("link-end");
				Element nameDefElem =
						child.getOwnerDocument().getElementById(elementId);
				String name = nameDefElem.getAttribute("name");
				*/
				String linkend = ((Element) child).getAttribute("link-end");
				// Gets the NameDef object. It may be inherited from a super-class.
				NameDef nd = getIdNameDef(rdbClassMapping, linkend);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, "link-end=<  \"" + linkend + "\">");
					logger.log(BasicLevel.DEBUG, "nd=<  " + nd + ">");
				}
				rdbClassMapping.createIdentifierMapping(nd);
			} else if ((childName.equals("rdb-class-ref-mapping")) ||
                    (childName.equals("rdb-gen-class-ref-mapping"))) {
				/* String elementId = ((Element) child).getAttribute("link-end");
				Element nameDefElem =
						child.getOwnerDocument().getElementById(elementId);
				String name = nameDefElem.getAttribute("name");
				Node parentNode = nameDefElem.getParentNode();
				String parentNodeName = parentNode.getNodeName();
				String fieldName = getFieldName(nameDefElem);
				if (parentNodeName.equals("class-ref")) {
					nd = getClassRefNameDef(currentClass, fieldName, namedefName);
				} else if (parentNodeName.equals("gen-class-ref")) {
					nd = getGenClassRefNameDef(currentClass, fieldName, name, 0);
				}
				*/
				String linkend = ((Element) child).getAttribute("link-end");
				// Gets the NameDef object, possibly beyond the scope of the current class if
				// this NameDef is inherited from a super-class.
				NameDef nd = getRefNameDef(linkend, childName);
				String ruleName = ((Element) child).getAttribute("rule-name");
				rdbClassMapping.createReferenceMapping(ruleName, nd);
			} else if (childName.equals("rdb-extension-ref-mapping")) {
				String linkend = ((Element) child).getAttribute("link-end");
				// Gets the NameDef object, possibly beyond the scope of the current class if
				// this NameDef is inherited from a super-class.
				NameDef nd = getNameDefFromSuper(linkend);
				String ruleName = ((Element) child).getAttribute("rule-name");
				rdbClassMapping.createReferenceMapping(ruleName, nd);
            } else if (childName.equals("parent-class-mapping")) {
                if (logger.isLoggable(BasicLevel.DEBUG))
					logger.log(BasicLevel.DEBUG, "parent-class-mapping");
                // linkend contains a parent class name.
				String fqcn = ((Element) child).getAttribute("link-end");
				// Gets the corresponding Class object.
				Class superClass = currentClass.getSuperClass(fqcn);
                if (superClass == null) {
                    throw new PException("Cannot find superclass " + fqcn +
                            " for class " + currentClass.getFQName());
                }
				String ruleName = ((Element) child).getAttribute("rule-name");
				if (logger.isLoggable(BasicLevel.DEBUG))
					logger.log(BasicLevel.DEBUG, "fqcn = " + fqcn + " / rule-name=" + ruleName);
				rdbClassMapping.createParentClassMapping(ruleName, superClass);
            } else if (childName.equals("#text")) {
            } else if (childName.equals("#comment")) {
			} else {
				logger.log(BasicLevel.WARN, "element <" + childName + "> unknown !");
			}
			if (logger.isLoggable(BasicLevel.DEBUG))
				logger.log(BasicLevel.DEBUG, "end   =<" + childName + ">");
		}
	}

	/**
	 * Parses a relational generic class mapping element and builds the
	 * corresponding Jorm meta-information.
	 * Manager, PathExplorer and currentClass must be set before calling this
	 * method.
	 *
	 * @param genClassMappingElem   the current XML node,
	 * @param mapping    a Mapping object.
	 */
	private void parseGenClassMapping(Element genClassMappingElem,
									  Mapping mapping) throws PException {

		//String ruleName = genClassMappingElem.getAttribute("rule-name");
		// Gets the referenced gen-class element
		/*String elementId = genClassMappingElem.getAttribute("link-end");
		Element genClassElem =
				genClassMappingElem.getOwnerDocument().getElementById(elementId);
		String fieldName = ((Element) genClassElem.getParentNode().getParentNode()).getAttribute("name");
		int nestingLevel =
				getGenClassNestingLevel(genClassElem, fieldName);
		// Gets the corresponding GenClassRef object
		GenClassRef genClassRef =
				getGenClassRef(currentClass, fieldName, nestingLevel);
		*/
		String linkend = genClassMappingElem.getAttribute("link-end");
		if (getLogger().isLoggable(BasicLevel.DEBUG)) {
			getLogger().log(BasicLevel.DEBUG,
							"linkend of rdb-gen-class-mapping <" + linkend + ">");
		}
		GenClassRef genClassRef = (GenClassRef) idvalue2genclassref.get(linkend);
		if ((genClassRef != null) &&
				(getLogger().isLoggable(BasicLevel.DEBUG))) {
			getLogger().log(BasicLevel.DEBUG,
							"GenClassRef name " + genClassRef.getName());
		}
		// Creates a GenClassMapping object.
		if (getLogger().isLoggable(BasicLevel.DEBUG)) {
			getLogger().log(BasicLevel.DEBUG,
							"create a new BasicRdbGenClassMapping for the current " +
							"Class (" + currentClass.getName() + ")");
		}
		RdbGenClassMapping rdbGenClassMapping = ((RdbMapping) mapping)
				.createGenClassMapping("", genClassRef);
		rdbGenClassMapping.setLogger(logger);
		NodeList children = genClassMappingElem.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String childName = child.getNodeName();
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "begin =<" + childName + ">");
			}
			if (childName.equals("rdb-table-spec")) {
				String tableName = ((Element) child).getAttribute("table-name");
				RdbTable table = rdbGenClassMapping.createRdbTable(tableName);
				parseTableSpec((Element) child, table, genClassRef);
			} else if (childName.equals("id-mapping")) {
				/* Gets the referenced name-def element.
				elementId = ((Element) child).getAttribute("link-end");
				Element nameDefElem =
						child.getOwnerDocument().getElementById(elementId);
				String name = nameDefElem.getAttribute("name");
				nestingLevel =
						getGenClassNestingLevel(nameDefElem, fieldName);
				// Gets the corresponding NameDef object.
				NameDef nd = genClassRef.getIdNameDef(name);
				// Creates an IdentifierMapping object.
				rdbGenClassMapping.createIdentifierMapping(nd);
				*/
				// The link-end attribute contains the name of the associated name-def.
				String namedefName = ((Element) child).getAttribute("link-end");
				// Gets the corresponding NameDef object.
				NameDef nd = genClassRef.getIdNameDef(namedefName);
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG,
							   "Fetching the ID name def of the GenClassRef " + namedefName);
					if (nd.isFieldName())
						logger.log(BasicLevel.DEBUG, "fieldName: " + nd.getFieldName());
					else if (nd.isNameRef())
						logger.log(BasicLevel.DEBUG, "NameRef: " + nd.getNameRef());
				}
				// Creates an IdentifierMapping object
				rdbGenClassMapping.createIdentifierMapping(nd);
            } else if (childName.equals("rdb-external-table-spec")) {
				String tableName = ((Element) child).getAttribute("table-name");
				if (logger.isLoggable(BasicLevel.DEBUG)) {
					logger.log(BasicLevel.DEBUG, "External table: " + tableName);
				}
				RdbExternalTable table =
						rdbGenClassMapping.createRdbExternalTable(tableName);
				parseExternalTableSpec((Element) child, table, genClassRef);
			} else if (childName.equals("rdb-ref-mapping")) {
				/* Gets the referenced name-def element.
				elementId = ((Element) child).getAttribute("link-end");
				Element nameDefElem =
						child.getOwnerDocument().getElementById(elementId);
				String name = nameDefElem.getAttribute("name");
				NameDef nd = null;
				if (genClassRef.isPrimitive()) {
					logger.log(BasicLevel.WARN, "You define a generic class of " +
						"primitive and a useless name-def for the elements: " +
						"generic class id=" + genClassRef.getGenClassId());
					continue;
				} else if (genClassRef.isClassRef()) {
					nd = genClassRef.getClassRef().getRefNameDef(name);
				} else if (genClassRef.isGenClassRef()) {
					nd = genClassRef.getGenClassRef().getRefNameDef(name);
				}
				*/
				String namedefName = ((Element) child).getAttribute("link-end");
				NameDef nd = null;
				//TODO: Verifier
				if (genClassRef.isPrimitive()) {
					logger.log(BasicLevel.WARN, "You define a generic class of " +
												"primitive and a useless name-def for the elements: " +
												"generic class id=" + genClassRef.getGenClassId());
					continue;
				} else if (genClassRef.isClassRef()) {
					if (logger.isLoggable(BasicLevel.DEBUG)) {
						logger.log(BasicLevel.DEBUG,
								   "Get the ref name def of the GenClassRef " + namedefName);
					}
					nd = genClassRef.getClassRef().getRefNameDef(namedefName);
				} else if (genClassRef.isGenClassRef()) {
					if (logger.isLoggable(BasicLevel.DEBUG)) {
						logger.log(BasicLevel.DEBUG,
								   "Get the ref name def of the ClassRef " + namedefName);
					}
					nd = genClassRef.getGenClassRef().getRefNameDef(namedefName);
				}
				String ruleName = ((Element) child).getAttribute("rule-name");
				// Creates a ReferenceMapping.
				rdbGenClassMapping.createReferenceMapping(ruleName, nd);
			} else if (childName.equals("#text")) {
			} else {
				logger.log(BasicLevel.WARN, "element <" + childName + "> unknown !");
			}
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "end   =<" + childName + ">");
			}
		}
	}

	private void parseTableSpec(Element tableSpecElem,
								RdbTable table,
								MetaObject mo) throws PException {
		String str = tableSpecElem.getAttribute("colocated");
		table.setColocated(new Boolean(str).booleanValue());

		str = tableSpecElem.getAttribute("colocated-master");
		table.setColocatedMaster(new Boolean(str).booleanValue());

		str = tableSpecElem.getAttribute("read-only");
		table.setReadOnly(new Boolean(str).booleanValue());

		NodeList children = tableSpecElem.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String childName = child.getNodeName();
			if (childName.equals("rdb-column-spec")) {
				parseColumn(child, table, mo);
			}
		}
	}

	private void parseExternalTableSpec(Element tableSpecElem,
										RdbExternalTable table,
										MetaObject mo) throws PException {
		String str = tableSpecElem.getAttribute("colocated");
		table.setColocated(new Boolean(str).booleanValue());

		str = tableSpecElem.getAttribute("colocated-master");
		table.setColocatedMaster(new Boolean(str).booleanValue());

		str = tableSpecElem.getAttribute("read-only");
		table.setReadOnly(new Boolean(str).booleanValue());

		NodeList children = tableSpecElem.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String childName = child.getNodeName();
			if (childName.equals("rdb-column-spec")) {
				parseColumn(child, table, mo);
			} else if (childName.equals("rdb-join")) {
				str = ((Element) child).getAttribute("name");
				parseJoin((Element) child, table.createRdbJoin(str));
			}
		}
	}

	private void parseColumn(Node child, RdbTable table, MetaObject mo)
			throws PException {
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, "begin =<" + child.getNodeName() + ">");
		}
		/* String elementId = ((Element) child).getAttribute("link-end");
		// The referenced element is either a primitive-type or a
		// a scalar-type.
		Element typeElem =
				child.getOwnerDocument().getElementById(elementId);
		// The parent element is either a field or a scalar-field, or a gen-class !!!
		String fieldName;
		if (((Element) typeElem.getParentNode()).getTagName().equals("gen-class")) {
			fieldName = "";
		} else {
			fieldName = ((Element) typeElem.getParentNode()).getAttribute("name");
		}*/
		// linkend contains a field or scalarfield name or a generic class name.
		String linkend = ((Element) child).getAttribute("link-end");
		PrimitiveElement pe = null;
		if (mo instanceof Class) {
			// pe is either a ScalarField or a PrimitiveElement object.
			//pe = (PrimitiveElement) ((Class) mo).getTypedElement(linkend);
            pe = getPrimitiveElement(linkend);
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "Class field name: " + linkend);
			}
		} else if (mo instanceof GenClassRef) {
			pe = ((GenClassRef) mo).getHiddenField(linkend);
			if (pe == null) {
				pe = ((GenClassRef) mo).getPrimitiveElement();
			}
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "GenClass name: " + linkend);
			}
		}
		String columnName = ((Element) child).getAttribute("column-name");
		if (logger.isLoggable(BasicLevel.DEBUG)) {
			logger.log(BasicLevel.DEBUG, "column name: " + columnName);
		}
		String sqlType = ((Element) child).getAttribute("sql-type");
		boolean notNull = new Boolean(((Element) child).
									  getAttribute("not-null")).booleanValue();
		if (table instanceof RdbExternalTable) {
			String str = ((Element) child).getAttribute("join-name");
			RdbJoin join = null;
			if (str == null) {
				//get the default join
				Iterator it = ((RdbExternalTable) table).getRdbJoins().iterator();
				if (!it.hasNext()) {
					throw new PException("No join defined in the external table ");
				}
				join = (RdbJoin) it.next();
				if (it.hasNext()) {
					throw new PException("Several joins defined in the external"
										 + " table " + table.getName() + ", you must specify"
										 + " a join-name attribute for the column "
										 + columnName);
				}
			} else {
				join = ((RdbExternalTable) table).getRdbJoin(str);
				if (join == null) {
					throw new PException("Impossible to define the column "
										 + columnName + " of the external table "
										 + table.getName()
										 + ": The join '" + str + "' was not defined");
				}
			}
			((RdbExternalTable) table).createPrimitiveElementMapping(
					pe, columnName, sqlType, notNull, join);
		} else {
			table.createPrimitiveElementMapping(pe, columnName, sqlType, notNull);
		}
		if (logger.isLoggable(BasicLevel.DEBUG))
			logger.log(BasicLevel.DEBUG, "end   =<" + child.getNodeName() + ">");
	}

	private void parseJoin(Element rdbJoinElem, RdbJoin rdbJoin) {
		NodeList children = rdbJoinElem.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String childName = child.getNodeName();
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "begin =<" + childName + ">");
			}
			if (childName.equals("rdb-column-match")) {
				String ptColumn = ((Element) child).getAttribute("column1");
				String etColumn = ((Element) child).getAttribute("column2");
				rdbJoin.addJoinColumnNames(ptColumn, etColumn);
			}
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "end   =<" + childName + ">");
			}
		}
	}

	private void parseFilter(
			Element rdbFilterElem,
			RdbFilter rdbFilter,
			Class theClass)
			throws PException {
		NodeList children = rdbFilterElem.getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node child = children.item(i);
			String childName = child.getNodeName();
			if (logger.isLoggable(BasicLevel.DEBUG)) {
				logger.log(BasicLevel.DEBUG, "begin =<" + childName + ">");
			}
			if (childName.equals("rdb-filter")) {
				String filterStr = ((Element) child).getAttribute("value");
				rdbFilter.setStringExpression(filterStr);
				try {
					if (logger.isLoggable(BasicLevel.DEBUG)) {
						logger.log(BasicLevel.DEBUG, "Parsing filter for " + filterStr);
					}
					//TODO delay this call in RdbFilter the first time rdbFilter.getExpression is done.
					Expression e = expParser.parse(
							filterStr,
							new RdbFilterParameterTypeProvider(theClass));
					if (logger.isLoggable(BasicLevel.DEBUG)) {
						logger.log(BasicLevel.DEBUG, "Parsed filter for " + filterStr + " is: " + ExpressionPrinter.e2str(e));
					}
					rdbFilter.setExpression(e);
				}
				catch (ExpressionException e) {
					throw new PException(e);
				}
				if (logger.isLoggable(BasicLevel.DEBUG)) {
				    logger.log(BasicLevel.DEBUG, "end   =<" + childName + ">");
			    }
		    }
        }
	}

	///////////////////////////////////////////////////////////////////
	//                      from MappingParser interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Parses the mapping information related to a particular mapper and builds
	 * the corresponding Jorm meta-information. It can be class-related or
	 * generic class-related information.
	 * Manager, PathExplorer and currentClass must be set before calling this
	 * method.
	 *
	 * @param mappingElem   the current XML node.
	 * @param mapping       a Mapping object.
	 */
	public void parseMapping(Element mappingElem, Mapping mapping) throws PException {
		if (mappingElem.getNodeName().equals("rdb-class-mapping")) {
			parseClassMapping(mappingElem, mapping);
		} else if (mappingElem.getNodeName().equals("rdb-gen-class-mapping")) {
			parseGenClassMapping(mappingElem, mapping);
		} else {
			throw new PException("Mapping element not supported by RDB: " + mappingElem.getNodeName());
		}
	}
}
