/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.api;

import java.util.Iterator;
import java.util.Collection;
import java.util.List;

/**
 * This interface gather methods to the ClassMapping and GenClassMapping
 * interfaces. It defines getter/setter for
 *      - mapping rule name: it defines how the class on the database structures.
 *        it is usefull when a class can be mapped in different ways.
 *        For instance, with a mapping to relational tables, a class
 *        can be mapped to one or more joined tables.
 *        Even though this method is shared by all class mapping implementation,
 *        values set and get are only meaningfull in those implementations
 *      - identifier mapping: it defines how the identifier of the class is mapped
 *      - primitive element mappings (PEMs): they defined how primitive elements are
 *        mapped at this level. Note that the referenced primitive element can
 *        be defined at an upper level in the inheritance hierarchy. Also note
 *        that PEMs are inherited and that inherited PEMs can be retrieved using the
 *        <code> getAllPrimitiveElements() </code> method
 *      - reference mappings (RMs): TODO: description
 *      - dependencies:             TODO: description
 *
 */
public interface CommonClassMapping extends MappingStructure {

	/**
	 * Returns the  name of the rule used to map the class.
	 * @return	 the name of the class mapping rule.
	 */
	String getRuleName();

	/**
	 * Sets the name of the rule used to map the class.
	 * @param rulename the class mapping rule name.
	 */
	void setRuleName(String rulename);

	/**
	 * Returns the IdentifierMapping in this class mapping.
	 * @return	the IdentifierMapping in this class mapping.
	 */
	IdentifierMapping getIdentifierMapping();

	/**
	 * Sets the IdentifierMapping object.
	 * @param idmapping the IdentifierMapping object.
	 */
	void setIdentifierMapping(IdentifierMapping idmapping);

	/**
	 * Returns the collection of PEM of the current class only defined at the
     * inheritance level, that is, without the inherited ones.
	 * @return  a collection.
	 */
	Collection getPrimitiveElementMappings();

	/**
	 * Returns the list of PEM of the current class and its super classes. The
	 * list is ordered by field names.
	 * @return  a collection.
	 */
	List getAllPrimitiveElementMappings();
	
	/**
	 * Returns the PrimitiveElementMapping objects (hidden or not) matching a
     * given fieldName.
	 * @param fieldName the name of the primitivite field (hidden or not) of
	 * the class of which the PEM must be returned
	 * @return  the PEM matching the given field name
	 */
	PrimitiveElementMapping getPrimitiveElementMapping(String fieldName);

	/**
	 * Adds a PrimitiveElementMapping object.
	 * @param  pemapping  the PrimitiveElementMapping object to add to the list.
	 */
	void addPrimitiveElementMapping(PrimitiveElementMapping pemapping);

	/**
	 * Adds a PrimitiveElementMapping object.
	 * @param  pemapping  the PrimitiveElementMapping object to add to the list.
	 */
	void addPrimitiveElementMapping(String fieldName, PrimitiveElementMapping pemapping);

	/**
	 * Returns an Iterator over PrimitiveElementMapping objects.
	 * @return  an Iterator.
	 */
	Iterator primitiveElementMappingsIterator();

	/**
	 * Creates a new IdentifierMapping object.
	 * @param   nd	the NameDef object that defines an object identifier.
	 * @return	an IdentifierMapping object.
	 */
	IdentifierMapping createIdentifierMapping(NameDef nd);

	/**
	 * Creates a new ReferenceMapping object.
	 * @param ruleName  the name of the rule used to map the reference,
	 *        nd	    the NameDef object that defines an object reference.
	 * @return	a ReferenceMapping object.
	 */
	ReferenceMapping createReferenceMapping(String ruleName, NameDef nd);

    /**
     * add a dependency to a given jorm class
     * @param jormClassName the name of the class to which depends the class mapping
     */
	void addDependency(String jormClassName);

    /**
     * remove a dependency to a given jorm class
     * @param jormClassName the name of the class to which the class mapping does
     * not depend anymore
     */
	void removeDependency(String jormClassName);

    /**
     * Returns the set of class names (Strings) of which the class mapping is dependent
     * @return the set of class names (Strings) of which the class mapping is dependent
     */
	Collection getDependencies();
}
