/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.api;

import java.util.Iterator;
import java.util.Collection;

/**
 * Package object defines an object able to gather a set of classes.
 * It is a similar approach to the package keyword from java.
 * @author	X. Spengler
 */
public interface Package extends MetaObject {
	/**
	 * Returns the name of the schema.
	 * @return	the string representation of the name of the schema.
	 *		    Null is returned if there is no schema name else a not empty
	 *          dotted string.
	 */
	String getName();

	/**
	 * Returns a Class created with a class name.
	 * @param className	the string representation of the class name
	 * @return	        a Class corresponding to its name. If the Class does
	 *                  not exist, null is returned.
	 */
	Class getClass(String className);

	/**
	 * Returns a new Class created with a class name.
	 * The MetaObject is added to the list of classes managed by the current
	 * schema. An isAbstract boolean is defined to know if the current class
	 * is an abstract class or not.
	 * @param className     the string representation of the class name
	 * @return	            a Class corresponding to the created class and
	 *                      registered, or an existing one if already defined
	 */
	Class createClass(String className);

	/**
	 * Adds an existing Class to the current Package
	 * @param aClass	the Class object to add
	 */
	void addClass(Class aClass);

	/**
	 * Adds an existing CompositeName to the current Package
	 * @param composite	the CompositeName object to add
	 */
	void addCompositeName(CompositeName composite);

	/**
	 * Allows to know all the registered classes into the current schema.
	 * This method returns an iterator on Class object.
	 * @return	an iterator for Class object. If there is no class, an empty
	 *          iterator is returned.
	 */
	Collection getClasses();

	/**
	 * Allows to know all the registered classes into the current schema.
	 * This method returns an iterator on Class object.
	 * @return	an iterator for Class object. If there is no class, an empty
	 *          iterator is returned.
	 */
	Collection getCompositeNames();

	/**
	 * Returns a compositename identified with its name.
	 * @param cn_Name	    the string representation of the name of the composite name.
	 * @return	            a compositename corresponding to its name. If the
	 *                      compositename does not exist, null is returned.
	 */
	CompositeName getCompositeName(String cn_Name);

	/**
	 * Returns a new compositename identified by its name.
	 * The MetaObject is added to the list of compositename classes managed by the
	 * current schema. If the compositename already exists, it is returned.
	 * @param CN_Name	    the string representation of the name of the compositename
	 * @return	            a compositename corresponding to the created compositename
	 *                      and registered, or an existing one if already
	 *                      defined
	 */
	CompositeName createCompositeName(String CN_Name);

	/**
	 * Allows to know all the registered compositename into the current schema.
	 * This method returns an iterator on compositename object.
	 * @return	an iterator for compositename object. If there is no class, an empty
	 *          iterator is returned.
	 */
	Iterator iterateCompositeName();
}
