/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.ScalarField;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Collection;
import java.util.Collections;

/**
 * The BasicCompositeName provides an implementation of the composite name.
 * Composite Name are used to define composite pname.
 * Composite Name defines the multiple fields describing the structure os a name.
 * @author	N. De Palma
 */
public class BasicCompositeName extends BasicMetaObject implements CompositeName {

	/**
	 * inheritedCompositeNames contains the list of inherited composited name for this composite name.
	 */
	private ArrayList inheritedCompositeNames;

	/**
	 * scalarFields defines a list of field defining the structure of this pname.
	 * key: the name of the field
	 * value: a scalarfield
	 */
	private Map scalarFields;

	/**
	 * the name of this compositename.
	 */
	private String name = null;

	/**
	 * Builds a new CompositeName object.
	 * This object is mainly defined by its name.
	 *
	 * @param className     the name of the current class
	 * @param parent    the parent of the current object
	 */
	public BasicCompositeName(String className,
							  MetaObject parent) {
		super(parent);
		this.name = className;
		scalarFields = new HashMap(5);
		inheritedCompositeNames = new ArrayList();
	}

	/**
	 * Allows to know the name of the current CompositeName object.
	 * @return	the string representation of the name of the CompositeName
	 */
	public String getName() {
		return name;
	}

	/**
	 * Allows to know the fully qualifed name of the current CompositeName object.
	 * @return	the string representation of the name of the CompositeName
	 */
	public String getFQName() {
		String res = ((Package) parent).getName();
		if (res == null || res.length() == 0)
			return name;
		else
			return res + "." + name;
	}

	/**
	 * Add an inherited composite name to the current composite name.
	 * @param cn the name of the composite name to be inherited.
	 */
	public void addInheritedCompositeName(CompositeName cn) {
		if (debug && cn != null)
			logger.log(BasicLevel.DEBUG,
					   "The current compositename (" + name + ") inherits from the composite name (" + cn.getName() + ")");
		if (cn != null)
			inheritedCompositeNames.add(cn);
	}

	public Iterator iterateInheritedCompositeName() {
		return getSuperCompositeNames().iterator();
	}

	public Collection getSuperCompositeNames() {
		return inheritedCompositeNames;
	}

	/**
	 * Retrieve an inherited composite name form its name.
	 * @param name the name of the composite name
	 * @return the composite name
	 */
	public CompositeName getInheritedCompositeName(String name) {

		CompositeName res = null;
		boolean found = false;

		Iterator iter = iterateInheritedCompositeName();
		while (iter.hasNext() && !found) {
			res = (CompositeName) iter.next();
			if (res.getName().equals(name)) found = true;
		}
		if (found) return res;
		return null;

	}

	/**
	 * Retrieve a scalarfield describing the field identified by it name in the current compositename.
	 * @param fieldname the name of the field.
	 * @return the scalarfield describing the field.
	 */
	public ScalarField getScalarField(String fieldname) {
		if (debug) {
			logger.log(BasicLevel.DEBUG,
				   "try to return field (" + fieldname + ")");
		}
		// this method finds in the 3 structures where is the field
		// fieldName could be a PrimitiveElement
		ScalarField res = (ScalarField) scalarFields.get(fieldname);
		if (debug && res == null) {
			// attention, the fieldname does not exist. where is it ?
			logger.log(BasicLevel.DEBUG, "the field (" + fieldname
			    + ") is not defined in the current class");
		}
		return res;
	}

	/**
	 * Returns the number of inherited composite name for the current composite name object.
	 * @return  the number of inherited compositename.
	 */
	public int getInheritedCompositeNameNumber() {
		return inheritedCompositeNames.size();
	}

	/**
	 * Build a new scalarfield that describes a field in the composite name
	 * @param fieldname the name of the field
	 * @param type the ptype of the field
	 * @return the scalarfield
	 */
	public ScalarField createCompositeNameField(String fieldname, PType type, int size, int scale) {
		if (debug)
			logger.log(BasicLevel.DEBUG,
					   "create a new NameField (" + fieldname + ") for the current CompositeName (" + name + ")");

		// verify if the field is not yet in the database
		ScalarField result = (ScalarField) scalarFields.get(fieldname);
		if (result != null) {
			if (logger != null) {
				logger.log(BasicLevel.WARN,
					   "attention, try to create an existing ScalarField (" +
					   fieldname + "), return existing one");
			}
		} else {
			// else create a new one
			result = new BasicScalarField(fieldname, type, size, scale, this);
			setLoggingOnChild(result);
			scalarFields.put(fieldname, result);
		}
		return result;
	}

	/**
	 * Give the number of fields composing the current composite name.
	 * @return the fields number.
	 */
	public int getFieldNumber() {
		return scalarFields.size();
	}

	/**
	 * Provides an iterator over the field of the current composite name.
	 * @return the iterator embeding scalarfield object
	 */
	public Iterator iterateField() {
		return scalarFields.values().iterator();
	}

	public Collection getFields() {
		return scalarFields.values();
	}

	public Collection getAllFields() {
		ArrayList allFields = new ArrayList();
		allFields.addAll(scalarFields.values());

		// and add the fields from the inherited classes
		for (Iterator iter = inheritedCompositeNames.iterator(); iter.hasNext();) {
			CompositeName aCN = (CompositeName) iter.next();
			allFields.addAll(aCN.getAllFields());
		}
		Collections.sort(allFields, FieldComparator.instance);
		return allFields;
	}

	/**
	 * Provides an iterator over the field of the current composite name including the inherited fields.
	 * @return the iterator embeding scalarfield object
	 */
	public Iterator iterateAllField() {
		return getAllFields().iterator();
	}

	/**
	 * @deprecated use getAllFields()
	 */
	public Collection getAllField() {
		return getAllFields();
	}

	protected Collection getChildren() {
		return getAllFields();
	}
}
