/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.api.GenClassMapping;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.MetaObject;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

/**
 * Implementation of the mapping interface.
 */
public abstract class BasicMapping
    extends BasicMetaObject
		implements Mapping {
	/**
	 * The mapper name.
	 */
	private String mapperName;

	/**
	 * The persistent structure resulting from the mapping of a class and its primitive
	 * values is described in a ClassMapping object.
	 */
	private ClassMapping classMapping;

	/**
	 * This HashMap associates a generic class identifier with a GenClassMapping
	 * object.
	 *
	 * Generic class identifiers uniquely identify one or several nested generic
	 * classes accessible from a given class field.
	 *
	 * Example:
	 * class Person {
	 *  private set(set(Equipment) equipments;
	 * ...}
	 * The external set is identified by "equipments.set".
	 * The internal set is identified by "equipments.set.set".
	 */
	private Map genClassMappings;

	/**
	 * Builds a new BasicMapping object.
	 * This object contains the mapping structures of a class.
	 * The parent object is a Project object.
	 * @param mapperName   the mapper name,
	 *        parent	    the parent of the current object.
	 */
	public BasicMapping(String mapperName, MetaObject parent) {
		super(parent);
		this.mapperName = mapperName;
		classMapping = null;
		genClassMappings = new HashMap();
	}

	///////////////////////////////////////////////////////////////////
	//                      from Mapping interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the name of the mapper.
	 * @return	 the mapper name.
	 */
	public String getMapperName() {
		return mapperName;
	}

	/**
	 * Sets the mapper name.
	 * @param mapperName   the mapper name.
	 */
	public void setMapperName(String mapperName) {
		this.mapperName = mapperName;
	}

	/**
	 * Returns the mapping structure of the class.
	 * @return	 a ClassMapping object.
	 */
	public ClassMapping getClassMapping() {
		return classMapping;
	}

	/**
	 * Sets the mapping structure of the class.
	 * @param classMapping	a ClassMapping object.
	 */
	public void setClassMapping(ClassMapping classMapping) {
		this.classMapping = classMapping;
	}

	/**
	 * Returns the GenClassMapping object of a GenClass.
	 * @param  genClassId   a GenClass identifier.
	 * @return a GenClassMapping object.
	 */
	public GenClassMapping getGenClassMapping(String genClassId) {
		return (GenClassMapping) genClassMappings.get(genClassId);
	}

	/**
	 * Returns a collection of GenClassMapping objects.
	 * @return	 a collection.
	 */
	public Collection getGenClassMappings() {
		return genClassMappings.values();
	}

	/**
	 * Adds a GenClassMapping object.
	 * @param  genClassId       a GenClass identifier,
	 *         genClassMapping	a GenClassMapping object.
	 */
	public void addGenClassMapping(String genClassId,
								   GenClassMapping genClassMapping) {
		genClassMappings.put(genClassId, genClassMapping);
	}

	protected Collection getChildren() {
		ArrayList al = new ArrayList();
		al.add(classMapping);
		al.addAll(genClassMappings.values());
		return al;
	}
}
