/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Collection;
import java.util.Iterator;

/**
 * Description
 */
public class BasicMetaObject implements MetaObject, Loggable {

	/**
	 * The parent of the current object.
	 */
	protected MetaObject parent;

	transient protected Logger logger = null;
	transient protected LoggerFactory loggerFactory = null;
	transient protected boolean debug = false;

	/**
	 * Builds a new BasicMetaObject object.
	 *
	 * @param parent    the parent of the current object
	 */
	public BasicMetaObject(MetaObject parent) {
        this.parent = parent;
		if (parent instanceof Loggable) {
			loggerFactory = ((Loggable) parent).getLoggerFactory();
			if (loggerFactory == null) {
				logger = ((Loggable) parent).getLogger();
			} else {
				logger = loggerFactory.getLogger(getClass().getName());
			}
		}
		debug = logger != null && logger.isLoggable(BasicLevel.DEBUG);
    }

	protected Collection getChildren() {
		return null;
	}

	public void setLoggingOnChild(MetaObject mo) {
		if (mo instanceof Loggable) {
			if (loggerFactory != null) {
				((Loggable) mo).setLoggerFactory(loggerFactory);
			} else if (logger != null) {
				((Loggable) mo).setLogger(logger);
			}
		}
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public void setLogger(Logger logger) {
		this.logger = logger;
		Collection children = getChildren();
		if (children != null && !children.isEmpty()) {
			for (Iterator it = children.iterator(); it.hasNext();) {
				Object o = it.next();
				if (o != null && o instanceof Loggable) {
					((Loggable) o).setLogger(logger);
				}
			}
		}
	}

	public void setLoggerFactory(LoggerFactory loggerfactory) {
		this.loggerFactory = loggerfactory;
		Collection children = getChildren();
		if (children != null && !children.isEmpty()) {
			for (Iterator it = children.iterator(); it.hasNext();) {
				Object o = it.next();
				if (o != null && o instanceof Loggable) {
					((Loggable) o).setLoggerFactory(loggerFactory);
				}
			}
		}
		if (loggerFactory != null && logger == null) {
			setLogger(loggerFactory.getLogger(getClass().getName()));
		}
	}

	///////////////////////////////////////////////////////////////////
	//                      from MetaObject interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the parent MetaObject of the current MetaObject.
	 *
	 * @return	the MetaObject corresponding to the parent of the
	 *  		current object. If there is no parent, null is returned.
	 */
	public MetaObject getParent() {
		return parent;
	}

	/**
	 * Sets the parent of the current meta object if it is not yet done
	 * by the constructor of the meta object
	 * @param parent	the parent MetaObject of the current object
	 */
	public void setParent(MetaObject parent) {
        this.parent = parent;
	}

	/**
	 * Returns the Manager object.
	 * @return the Manager object.
	 */
    public Manager getManager() {
		MetaObject tmp = parent;
		MetaObject manager = null;
		while (tmp != null) {
			manager = tmp;
			tmp = tmp.getParent();
		}
        return (Manager) manager;
    }
}