/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.metainfo.lib;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Package;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.Collection;
import java.util.ArrayList;

/**
 * BasicPackage is an implementation of the Package interface.
 * This object defines a set of classes and generic classes declared in a Package
 * tag. Its parent is the Manager.
 * @author	X. Spengler
 */
public class BasicPackage extends BasicMetaObject implements Package {
	/**
	 * This structure contains all declared classes for the
	 * current schema object.
	 * key: the name of the class
	 * value: the Class MetaObject object
	 */
	protected Map classes;

	/**
	 * This structure contains all declared compositename for the
	 * current schema object.
	 * key: the name of the class
	 * value: the Class MetaObject object
	 */
	protected Map compositeNames;

	/**
	 * The name of the schema, this name could be null if
	 * the name is not defined by the user.
	 */
	protected String name;

	/**
	 * Builds a new Package object.
	 * This object is defined by its name and its parent.
	 *
	 * @param name      the name of the schema to create
	 * @param parent    the parent meta-object
	 */
	public BasicPackage(String name, Manager parent) {
		super(parent);
		this.name = name;
		classes = new HashMap();
		compositeNames = new HashMap();
	}

	///////////////////////////////////////////////////////////////////
	//                      from Package interface
	///////////////////////////////////////////////////////////////////

	/**
	 * Returns the name of the schema.
	 *
	 * @return	the string representation of the name of the schema.
	 *		    Null is returned if there is no schema name else a not empty
	 *          dotted string.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns a Class created with a class name.
	 *
	 * @param className	the string representation of the class name
	 * @return	        a Class corresponding to its name. If the Class does
	 *                  not exist, null is returned.
	 */
	public Class getClass(String className) {
		if (debug && classes.get(className) == null) {
			logger.log(BasicLevel.DEBUG, "There is no class with this name ("
			    + className + ") for the current Package (" + name + ")");
		}
		return (Class) classes.get(className);
	}


	/**
	 * Returns a compositename identified with its name.
	 *
	 * @param cn_Name	    the string representation of the name of the composite name.
	 * @return	            a compositename corresponding to its name. If the
	 *                      compositename does not exist, null is returned.
	 */
	public CompositeName getCompositeName(String cn_Name) {
		if (debug && compositeNames.get(cn_Name) == null) {
			logger.log(BasicLevel.DEBUG,
			    "There is no composite name with this name (" + cn_Name
			    + ") for the current Package (" + name + ")");
		}
		return (CompositeName) compositeNames.get(cn_Name);
	}

	/**
	 * Returns a new compositename identified by its name.
	 * The MetaObject is added to the list of compositename classes managed by the
	 * current schema. If the compositename already exists, it is returned.
	 *
	 * @param CN_Name	    the string representation of the name of the compositename
	 * @return	            a compositename corresponding to the created compositename
	 *                      and registered, or an existing one if already
	 *                      defined
	 */
	public CompositeName createCompositeName(String CN_Name) {
		if (debug)
			logger.log(BasicLevel.DEBUG, "create a new CompositeName ("
			    + CN_Name + ") for the current Package (" + name + ")");

		// verify the class is not yet in the current schema
		CompositeName result = (CompositeName) compositeNames.get(CN_Name);
		if (result != null) {
			if (debug) {
				logger.log(BasicLevel.DEBUG,
				    "return the existing CompositeName" + CN_Name);
			}
		} else {
			result = new BasicCompositeName(CN_Name, this);
			setLoggingOnChild(result);
			compositeNames.put(CN_Name, result);
		}
		return result;
	}

	/**
	 * Allows to know all the registered compositename into the current schema.
	 * This method returns an iterator on compositename object.
	 *
	 * @return	an iterator for compositename object. If there is no class, an empty
	 *          iterator is returned.
	 */
	public Iterator iterateCompositeName() {
		return compositeNames.values().iterator();
	}

	/**
	 * Returns a new Class created with a class name.
	 * The MetaObject is added to the list of classes managed by the current
	 * schema. An isAbstract boolean is defined to know if the current class
	 * is an abstract class or not.
	 *
	 * @param className     the string representation of the class name
	 * @return	            a Class corresponding to the created class and
	 *                      registered, or an existing one if already defined
	 */
	public Class createClass(String className) {
		if (debug) {
			logger.log(BasicLevel.DEBUG, "create a new Class (" + className
			    + ") for the current Package (" + name + ")");
		}
		// verify the class is not yet in the current schema
		Class result = (Class) classes.get(className);
		if (result != null) {
			if (debug) {
				logger.log(BasicLevel.DEBUG, "return the existing Class " + className);
			}
		} else {
			result = new BasicClass(className, this);
			setLoggingOnChild(result);
			classes.put(className, result);
		}
		return result;
	}

	/**
	 * Adds an existing Class to the current Package
	 *
	 * @param aClass	the Class object to add
	 */
	public void addClass(Class aClass) {
		String aClassName = aClass.getName();
		if (classes.get(aClassName) != null) {
			if (logger != null) {
				logger.log(BasicLevel.INFO, "try to add an existing Class ("
				    + aClassName + ")");
			}
		} else {
			classes.put(aClassName, aClass);
		}
	}

	/**
	 * Adds an existing CompositeName to the current Package
	 *
	 * @param composite	the CompositeName object to add
	 */
	public void addCompositeName(CompositeName composite) {
		String compositeName = composite.getName();
		if (compositeNames.get(compositeName) != null) {
			if (logger != null) {
				logger.log(BasicLevel.INFO, "try to add an existing CompositeName ("
			            + compositeName + ")");
			}
		} else {
			compositeNames.put(compositeName, composite);
		}
	}


	/**
	 * Allows to know all the registered classes into the current schema.
	 * This method returns an iterator on Class object.
	 *
	 * @return	an iterator for Class object. If there is no class, an empty
	 *          iterator is returned.
	 */
	public Collection getClasses() {
		return classes.values();
	}

	public Collection getCompositeNames() {
		return compositeNames.values();
	}

	protected Collection getChildren() {
		ArrayList res = new ArrayList();
		res.addAll(classes.values());
		res.addAll(compositeNames.values());
		return res;
	}
}
