/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.naming.api;

import java.util.Date;
import java.math.BigInteger;
import java.math.BigDecimal;

/**
 * Represents a PNamingContext able to say if a pname structure (pname getter,
 * a long, ...) match a persistent object represented by this PNamingContext.
 *
 * A FilteredPNamingContext must support the decoding type of its parent. This
 * means that the decodeXXX(xxx) and the match(xxx) methods, where xxx is the
 * parent coding type, must be supported.
 *
 * @author S.Chassande-Barrioz, P.Dechamboux
 */
public interface FilteredPNamingContext extends PNamingContext {

	/**
	 * binds a sub filteredPNamingContext to the current. The implementation of
	 * this method must register the sub PNC and call the setParentCodingType
	 * method on this child.
	 */
	void bindSubFPNC(FilteredPNamingContext fpnc);

	/**
	 * Unbinds a sub PNC from its parent.
	 */
	void unbindSubFPNC(FilteredPNamingContext fpnc);

	/**
	 * Indicates if the identifier represented this byte[] value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(byte[] en);

	/**
	 * Indicates if the identifier represented this Object value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(Object en);

	/**
	 * Indicates if the identifier represented this byte value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(byte en);

	/**
	 * Indicates if the identifier represented this Byte value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(Byte en);

	/**
	 * Indicates if the identifier represented this char value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(char en);

	/**
	 * Indicates if the identifier represented this Character value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(Character en);

	/**
	 * Indicates if the identifier represented this int value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(int en);

	/**
	 * Indicates if the identifier represented this Integer value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(Integer en);

	/**
	 * Indicates if the identifier represented this long value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(long en);

	/**
	 * Indicates if the identifier represented this Long value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(Long en);

	/**
	 * Indicates if the identifier represented this short value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(short en);

	/**
	 * Indicates if the identifier represented this Short value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(Short en);

	/**
	 * Indicates if the identifier represented this String value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(String en);

	/**
	 * Indicates if the identifier represented this char[] value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(char[] en);

	/**
	 * Indicates if the identifier represented this Date value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(Date en);

	/**
	 * Indicates if the identifier represented this BigInteger value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(BigInteger en);

	/**
	 * Indicates if the identifier represented this BigDecimal value matches a
	 * Persistent object manages by the PNamingContext
	 */
	boolean match(BigDecimal en);
}
