/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.naming.lib;

import org.objectweb.jorm.naming.api.KeyFilteredNamingContext;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNameGetterConverter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PType;

import java.util.Map;
import java.util.HashMap;
import java.util.Collections;
import java.util.Date;
import java.util.Arrays;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * This class is a PNamingContext for managing the inheritance. The naming
 * is based on a single filter (Medor expression) for the overall inheritance
 * tree. The medor expression uses naming fields from a PNameGetter. The
 * evalution of the expression on a PNameGetter produces a value. Each non
 * abstract persistent class must define a key value. The comparaison between
 * key values associated to persistent classes and the result of the expression,
 * permits to know the persistent class represented by the PNameGetter. By this
 * way this PNamingContext can delegate the decoding to the binder of the
 * persistent class.
 * This class is abstract. It must be specialized for the evalution of
 * expression.
 *
 * @see org.objectweb.jorm.generator.lib.KFPNCGenerator
 * @author S.Chassande-Barrioz
 */
public abstract class AbstractInheritKeyFilteredPNamingContext
	extends BasicPNamingContext
	implements KeyFilteredNamingContext {

	/**
	 * The PType of the expression which is also the PType of the key of course.
	 */
	protected PType expressionType;

	/**
	 * The PNameGetterConvert to use to convert a primitive value to a
	 * PNameGetter. It can be null only if the coding type is composite.
	 */
	protected PNameGetterConverter pngConverter;

	/**
	 * The map mainting the association between keys and binder
	 */
	protected Map key2binder;

	/**
	 * The codingType of the current PNamingContext
	 */
	protected int codingType;

	/**
	 * An object representing the null pname value. If the coding type is
	 * composite, this field must be a PNameGetter. Otherwise this field is
	 * the java.lang class matching the coding type. At initialization step
	 * it is accepted that the user specify another object. See the
	 * checkNullValue() method for more information on this point.
	 */
	protected Object nullValue;

	protected PName nullPName;

	/**
	 * Builds a PNamingContext. The use of this constructor requires to
	 * configure the expression/key type, the coding type and the pname null
	 * value with the corresponding setter methods, AND to initialize the PNC
	 * througth the init() method..
	 */
	public AbstractInheritKeyFilteredPNamingContext() {
		key2binder = new HashMap();
	}

	public AbstractInheritKeyFilteredPNamingContext(PType expressionType,
													int codingType) throws PException {
		this();
		setExpressionType(expressionType);
		setCodingType(codingType);
	}

	public void init() throws PException {
		checkNullValue();
	}

	public void setCodingType(int codingType) {
		this.codingType = codingType;
		if (this.codingType != PNameCoder.CTCOMPOSITE) {
			pngConverter = new SimplePNGConverter();
		} else {
			pngConverter = null;
		}
	}

	public boolean isNull(Object png, Object ctx) throws PException {
		return false;
	}

	//IMPLEMENTATION OF THE KeyFilteredNamingContext INTERFACE //
	//---------------------------------------------------------//

	public void exportClass(PBinder binder, Object key) throws PException {
		synchronized(key2binder) {
			PBinder oldbinder = (PBinder) key2binder.put(key, binder);
			if (oldbinder != null && oldbinder != binder) {
				key2binder.put(key, oldbinder);
				throw new PException("Another persistent class is alread exported with the same key: "
					+ oldbinder.getBinderClassMapping().getClassName());
			}
		}
	}

	public void unexportClass(Object key) throws PException {
		synchronized(key2binder) {
			Object oldbinder = key2binder.remove(key);
			if (oldbinder == null) {
				throw new PException("No persistent class to unexport with the key: " + key);
			}
		}
	}

	public PBinder lookup(Object key) {
		return (PBinder) key2binder.get(key);
	}

	public Map entries() {
		return Collections.unmodifiableMap(key2binder);
	}

	public void setExpressionType(PType type) {
		this.expressionType = type;
	}

	public PType getExpressionType() {
		return expressionType;
	}

	public abstract Object evaluate(Object png, Object ctx) throws PException;


	//IMPLEMENTATION OF THE PNamingContext INTERFACE //
	//-----------------------------------------------//

	public PName resolve(Object conn, PName pn) throws PException {
		return pn;
	}


	//IMPLEMENTATION OF THE PNamingManager INTERFACE //
	//-----------------------------------------------//

	public PName export(Object conn, Object infoitem) throws PException {
		if (infoitem instanceof PName) {
			return (PName) infoitem;
		}
		throw new PException("Operation not supported by this PNamingContext");
	}

	public PName export(Object conn, Object infoitem, Object hints)
		throws PException {
		throw new PException("Operation not supported by this PNamingContext");
	}

	public void unexport(Object conn, PName pn) throws PException {
		throw new PException("Operation not supported by this PNamingContext");
	}

	public void unexport(Object conn, PName pn, Object hints) throws PException {
		throw new PException("Operation not supported by this PNamingContext");
	}


	//IMPLEMENTATION OF THE PNamingCoder INTERFACE //
	//---------------------------------------------//

	public boolean codingSupported(int codingtype) {
		return codingtype == codingType || codingtype == PNameCoder.CTSTRING;
	}

	public PName decodeAbstract(Object en, Object context)
		throws PExceptionNaming, UnsupportedOperationException {
		Object key;
		try {
			if (isNull(en, context)) {
				return nullPName;
			}
			key = evaluate(en, context);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No");
		}
		return binder.decodeAbstract(en, context);
	}

	public PName decodeByte(byte en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTBYTE) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (((Byte) nullValue).byteValue() == en) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No");
		}
		return binder.decodeByte(en);
	}

	public PName decodeObyte(Byte en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTOBYTE) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (nullValue == null) {
			if (en == null) {
				return getNull();
			}
		} else if (en != null
			&& ((Byte) nullValue).byteValue() == en.byteValue()) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeObyte(en);
	}

	public PName decodeChar(char en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTCHAR) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (((Character) nullValue).charValue() == en) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeChar(en);
	}

	public PName decodeOchar(Character en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTOCHAR) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (nullValue == null) {
			if (en == null) {
				return getNull();
			}
		} else if (en != null
			&& ((Character) nullValue).charValue() == en.charValue()) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeOchar(en);
	}

	public PName decodeInt(int en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTINT) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (((Integer) nullValue).intValue() == en) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeInt(en);
	}

	public PName decodeOint(Integer en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTOINT) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (nullValue == null) {
			if (en == null) {
				return getNull();
			}
		} else if (en != null
			&& ((Integer) nullValue).intValue() == en.intValue()) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeOint(en);
	}

	public PName decodeLong(long en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTLONG) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (((Long) nullValue).longValue() == en) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeLong(en);
	}

	public PName decodeOlong(Long en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTOLONG) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (nullValue == null) {
			if (en == null) {
				return getNull();
			}
		} else if (en != null
			&& ((Long) nullValue).longValue() == en.longValue()) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeOlong(en);
	}

	public PName decodeShort(short en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTSHORT) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (((Short) nullValue).shortValue() == en) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeLong(en);
	}

	public PName decodeOshort(Short en) throws PExceptionNaming, UnsupportedOperationException {
		if (codingType != CTOSHORT) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (nullValue == null) {
			if (en == null) {
				return getNull();
			}
		} else if (en != null
			&& ((Short) nullValue).shortValue() == en.shortValue()) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeOshort(en);
	}

	public PName decodeCharArray(char[] en) throws PExceptionNaming {
		if (codingType != CTCHARARRAY) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (Arrays.equals((char[]) nullValue, en)) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeCharArray(en);
	}

	public PName decode(byte[] en) throws PExceptionNaming {
		if (codingType != CTBYTEARRAY) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (Arrays.equals((byte[]) nullValue, en)) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decode(en);
	}

	public PName decodeDate(Date en) throws PExceptionNaming {
		if (codingType != CTDATE) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (nullValue == null) {
			if (en == null) {
				return getNull();
			}
		} else if (en != null
			&& ((Date) nullValue).getTime() == en.getTime()) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeDate(en);
	}

	public PName decodeBigInteger(BigInteger en) throws PExceptionNaming {
		if (codingType != CTBIGINTEGER) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (nullValue == null) {
			if (en == null) {
				return getNull();
			}
		} else if (((BigInteger) nullValue).compareTo(en) == 0) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeBigInteger(en);
	}

	public PName decodeBigDecimal(BigDecimal en) throws PExceptionNaming {
		if (codingType != CTBIGDECIMAL) {
			throw new PExceptionNaming("Bad coder type: " + codingType);
		}
		if (nullValue == null) {
			if (en == null) {
				return getNull();
			}
		} else if (((BigDecimal) nullValue).compareTo(en) == 0) {
			return getNull();
		}
		Object key;
		try {
			key = evaluate(pngConverter.convert(en), null);
		} catch (PExceptionNaming e) {
			throw e;
		} catch (PException e) {
			throw new PExceptionNaming(e);
		}
		PBinder binder = (PBinder) key2binder.get(key);
		if (binder == null) {
			throw new PExceptionNaming("No binder associated to the key: " + key);
		}
		return binder.decodeBigDecimal(en);
	}


	public PName decodeString(String en) throws PExceptionNaming {

		return null;
	}

	public byte encodeByte(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return ((Byte) nullValue).byteValue();
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeByte(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public char encodeChar(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return ((Character) nullValue).charValue();
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeChar(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public int encodeInt(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return ((Integer) nullValue).intValue();
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeInt(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public long encodeLong(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return ((Long) nullValue).longValue();
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeLong(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public short encodeShort(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return ((Short) nullValue).shortValue();
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeShort(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public Byte encodeObyte(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (Byte) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeObyte(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public Character encodeOchar(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (Character) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeOchar(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public Integer encodeOint(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (Integer) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeOint(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public Long encodeOlong(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (Long) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeOlong(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public Short encodeOshort(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (Short) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeOshort(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public Date encodeDate(PName pn) throws PExceptionNaming {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (Date) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeDate(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public BigInteger encodeBigInteger(PName pn) throws PExceptionNaming {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (BigInteger) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeBigInteger(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public BigDecimal encodeBigDecimal(PName pn) throws PExceptionNaming {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (BigDecimal) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeBigDecimal(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public byte[] encode(PName pn) throws PExceptionNaming {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (byte[]) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encode(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public char[] encodeCharArray(PName pn) throws PExceptionNaming {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return (char[]) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeCharArray(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public Object encodeAbstract(PName pn) throws PExceptionNaming, UnsupportedOperationException {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullPName || pn.isNull()) {
			return nullPName;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeAbstract(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public String encodeString(PName pn) throws PExceptionNaming {
		if (pn == null) {
			throw new PExceptionNaming("Impossible to encode null");
		} else if (pn == nullValue || pn.isNull()) {
			return (String) nullValue;
		} else {
			PNameManager pnm = pn.getPNameManager();
			if (pnm != this && pnm != null) {
				return pn.getPNameManager().encodeString(pn);
			} else {
				throw new PExceptionNaming("This PName should not be encoded in this PNamingContext, pname: " + pn);
			}
		}
	}

	public PName getNull() {
		return nullPName;
	}

	/**
	 * @param o must be the null PName
	 */
	public void setNullPName(Object o) throws PException {
		if (!(o instanceof PName)) {
			throw new PExceptionNaming("A KFPNC accepts only PName instance as parameter to define the null pname");
		}
		nullPName = (PName) o;
	}

	private void checkNullValue() throws PException {
		switch (codingType) {
		case CTCHAR:
			nullValue = new Character(nullPName.encodeChar());
			break;
		case CTOCHAR:
			nullValue = nullPName.encodeOchar();
			break;
		case CTBYTE:
			nullValue = new Byte(nullPName.encodeByte());
			break;
		case CTOBYTE:
			nullValue = nullPName.encodeObyte();
			break;
		case CTSHORT:
			nullValue = new Short(nullPName.encodeShort());
			break;
		case CTOSHORT:
			nullValue = nullPName.encodeOshort();
			break;
		case CTINT:
			nullValue = new Integer(nullPName.encodeInt());
			break;
		case CTOINT:
			nullValue = nullPName.encodeOint();
			break;
		case CTLONG:
			nullValue = new Long(nullPName.encodeLong());
			break;
		case CTOLONG:
			nullValue = nullPName.encodeOlong();
			break;
		case CTSTRING:
			nullValue = nullPName.encodeString();
			break;
		case CTDATE:
			nullValue = nullPName.encodeDate();
			break;
		case CTBIGDECIMAL:
			nullValue = nullPName.encodeBigDecimal();
			break;
		case CTBIGINTEGER:
			nullValue = nullPName.encodeBigInteger();
			break;
		case CTBYTEARRAY:
			nullValue = nullPName.encode();
			break;
		case CTCHARARRAY:
			nullValue = nullPName.encodeCharArray();
			break;
		case CTCOMPOSITE:
			break;
		}
	}
}

