/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.jorm.naming.lib;

import org.objectweb.jorm.api.PMapperListener;
import org.objectweb.jorm.api.PMappingRequiredEvent;
import org.objectweb.jorm.api.ClassMappedEvent;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.lib.PTypePAAH;
import org.objectweb.jorm.naming.api.NamingFilterKeyProvider;
import org.objectweb.jorm.naming.api.KeyFilteredNamingContext;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 * Is a manager of FKPNC instances. It is in charge of the initialization of the
 * couple (binder, key) into KFPNC. This manager keeps an association table
 * between the class name of the root inheritance tree, and the KFPNC instance.
 *
 * @author S.Chassande-Barrioz
 */
public class KFPNCManager
	implements PMapperListener, Loggable {

	private Logger logger;
	private LoggerFactory loggerFactory;
	private boolean debug;

	private Map cn2pnc;
	
	/**
	 * Key=SuperClassName Value=List of Object[]
	 * where Object[0] = binder 
	 * 		 Object[1] = namingKey
	 */
	private Map scn2binderKey = new HashMap();

	public KFPNCManager() {
		cn2pnc = new HashMap();
	}

	public KFPNCManager(Map cn2pnc) {
		this.cn2pnc = cn2pnc;
	}

	public KeyFilteredNamingContext getKFPNC(String className) {
		Object pnc = cn2pnc.get(className);
		return (KeyFilteredNamingContext )
			(pnc instanceof KeyFilteredNamingContext ? pnc : null);
	}


	// IMPLEMENTATION OF THE PMapperListener INTERFACE //
	//-------------------------------------------------//

	public void mappingRequired(PMappingRequiredEvent e) {
		//nothing to do
	}

	/**
	 * If the class is a root of an inheritance class tree, a KFPNC is created.
	 * Otherwise if the PClassMapping of the new persistent class is also
	 * a NamingFilterKeyProvider, the couple (binder, key) is registered into
	 * the KFPNC of the ancestors.
	 */
	public void classMapped(ClassMappedEvent e) {
		String className = e.getJormClassName();
		PMapper mapper = e.getPMapper();
		PType type = mapper.getPTypeSpace().getPType(className);
		if (!(type instanceof PTypePAAH)) {
			return;
		}
		PClassMapping pcm = mapper.lookup(className);
		PTypePAAH classType = (PTypePAAH) type;
		PType[] superTypes = classType.getInheritedPType();
		if (superTypes == null || superTypes.length == 0) { //no parent
			//fetch the KFPNC
			Object pnc = cn2pnc.get(className);
			if (!(pnc instanceof KeyFilteredNamingContext)) {
				return;
			}
			KeyFilteredNamingContext kfpnc = (KeyFilteredNamingContext) pnc;
			// fetch all children
			HashSet subPCMs = new HashSet(); //contains also the current PCM
			findSubPCM(className, subPCMs, mapper);

			//resgister the couple (binder,key) into the PNC
			for(Iterator it = subPCMs.iterator(); it.hasNext();) {
				PClassMapping subpcm = (PClassMapping) it.next();
				if (subpcm instanceof NamingFilterKeyProvider) {
					Object key = ((NamingFilterKeyProvider) subpcm).getNamingFilterKey();
					if (debug) {
						logger.log(BasicLevel.DEBUG,
							"Register the binder of the '"
							+ subpcm.getClassName()
							+ "' sub class with the following key: " + key);
					}
					try {
						kfpnc.exportClass(subpcm.getPBinder(), key);
					} catch (PException e1) {
						logger.log(BasicLevel.ERROR,
							"Impossible to export a Binder into the KFPNC of the class "
							+ className, e1);
					}
				}
			}
			List list = (List) scn2binderKey.remove(className);
			if (list != null) {
				while (!list.isEmpty()) {
					Object[] obj = (Object[]) list.remove(0);
					if (debug) {
						logger.log(BasicLevel.DEBUG,
							"Register the binder of the '"
							+ ((PBinder)obj[0]).getBinderClassMapping().getClassName()
							+ "' sub class with the following key: " + obj[1]);
					}
					try {
						kfpnc.exportClass((PBinder)obj[0], obj[1]);
					} catch (PException e1) {
						logger.log(BasicLevel.ERROR,
							"Impossible to export a Binder into the KFPNC of the class "
							+ className, e1);
					}
				}
			}
		} else if (pcm instanceof NamingFilterKeyProvider) {
			Object key = ((NamingFilterKeyProvider) pcm).getNamingFilterKey();
			if (debug) {
				logger.log(BasicLevel.DEBUG,
					"The '" + pcm.getClassName()
					+ "' provides the following key: " + key);
			}
			//has a parent at least & provide naming filter key
			HashSet superClassNames = new HashSet();

			//find ancestors
			findAncestorNames(className, superClassNames, mapper);
			PBinder binder = pcm.getPBinder();

			//Register the new mapped class into the KFPNC
			for(Iterator it = superClassNames.iterator(); it.hasNext();) {
				String superClassName = (String) it.next();
				KeyFilteredNamingContext kfpnc = getKFPNC(superClassName);
				if (kfpnc == null) {
					List list = (List) scn2binderKey.get(superClassName);
					if (list == null) {
						list = new ArrayList();
						scn2binderKey.put(superClassName, list);
					}
					list.add(new Object[]{binder, key});
					if (debug) {
						logger.log(BasicLevel.DEBUG,
							"No KPNC found for the ancestor " + superClassName
							+ ". The register of the class "
							+ pcm.getClassName() + " will be done later");
					}
				} else {
					if (debug) {
						logger.log(BasicLevel.DEBUG,
							"Register the binder of the '"
							+ pcm.getClassName()
							+ "' class with the following key: " + key
							+ " into the KFPNc of the ancestor " + superClassName);
					}
					try {
						kfpnc.exportClass(binder, key);
					} catch (PException e1) {
						logger.log(BasicLevel.ERROR,
							"Impossible to export a Binder into the KFPNC of the class "
							+ superClassName, e1);
					}
				}
			}
		}
	}

	/**
	 * Find PClassMapping of all sub persistent classes of a persistent class.
	 * The type tree is used to find the sub class.
	 *
	 * @param className is the name of the persistent class
	 * @param result is the result parameter where the PCM instance must be
	 * added.
	 */
	private void findSubPCM(String className, Set result, PMapper mapper) {
		PClassMapping pcm = mapper.lookup(className);
		if (pcm != null && result.add(pcm)) {
			PType classType = mapper.getPTypeSpace().getPType(className);
			PType[] subTypes = classType.getSubTypes();
			for(int i=0; i<subTypes.length; i++) {
				findSubPCM(subTypes[i].getJavaName(), result, mapper);
			}
		} else {
			if (debug) {
				logger.log(BasicLevel.DEBUG, "The sub class "
					+ className + " is not yet mapped");
			}
		}
	}

	/**
	 * Find class name of ancestor of a persistent class
	 * @param className is the name of the persistent class
	 * @param result is the result parameter where the ancestor class name
	 * must be added.
	 */
	private void findAncestorNames(String className, Set result, PMapper mapper) {
		PType type = mapper.getPTypeSpace().getPType(className);
		if (!(type instanceof PTypePAAH)) {
			return;
		}
		PTypePAAH classType = (PTypePAAH) type;
		PType[] superTypes = classType.getInheritedPType();
		if (superTypes == null || superTypes.length == 0) { //no parent
			result.add(className);
		} else { //has Parent
			for(int i=0; i<superTypes.length; i++) {
				findAncestorNames(superTypes[i].getJormName(), result, mapper);
			}
		}
	}


	// IMPLEMENTATION OF THE Loggable INTERFACE //
	//------------------------------------------//

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}

	public void setLoggerFactory(LoggerFactory loggerFactory) {
		this.loggerFactory = loggerFactory;
	}

	public Logger getLogger() {
		return logger;
	}

	public void setLogger(Logger logger) {
		this.logger = logger;
		debug = logger.isLoggable(BasicLevel.DEBUG);
	}
}
