/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.util.lib;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.MatchingTask;
import org.apache.tools.ant.taskdefs.UpToDate;
import org.apache.tools.ant.taskdefs.Javac;
import org.apache.tools.ant.types.DTDLocation;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Reference;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.jorm.compiler.api.JormCompilerConfigurator;
import org.objectweb.jorm.compiler.lib.JormCompiler;
import org.objectweb.jorm.compiler.lib.JormCompilerParameterImpl;
import org.objectweb.jorm.compiler.lib.JormCompilerConfiguratorImpl;
import org.objectweb.jorm.util.io.api.PathExplorer;
import org.objectweb.jorm.util.io.lib.DirJavaExplorer;

import java.io.File;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * The JORM ant task.
 */
public class AntJorm extends MatchingTask {
	private String jormConfFile = "jorm.properties";
	private File pdDir = null;

	private boolean antDtdLocationsSet = false;
	private ArrayList antDtdLocations = new ArrayList();
	private boolean antJavacSet = false;
	private boolean antJavac;
	private boolean classpathSet = false;
	private PathExplorer classpath = new DirJavaExplorer();
	private boolean antOutputSet = false;
	private String antOutput;
	private boolean antLogConfFileSet = false;
	private String antLogConfFile;
	private boolean antProjectNameSet = false;
	private String antProjectName;
	private boolean antBindingInheritanceSet = false;
	private String antBindingInheritance;
	private boolean antClassMappingInheritanceSet = false;
	private String antClassMappingInheritance;
	private boolean antVerboseSet = false;
	private boolean antVerbose;
	private boolean antKeepSrcSet = false;
	private boolean antKeepSrc;
	private boolean antBindingAbstractSet = false;
	private boolean antBindingAbstract;
	private boolean generatedPDFiles = false;
	private String mapperNames = null;

	private JormCompilerParameter compilerParameter = null;
	private JormCompilerConfigurator compilerConfigurator = null;
	private JormCompiler compiler = null;

	public AntJorm() {
		compilerParameter = new JormCompilerParameterImpl();
		compilerConfigurator = new JormCompilerConfiguratorImpl();
		compiler = new JormCompiler(compilerParameter, compilerConfigurator);
	}

	/**
	 * It assigns the directory name where the include and exclude clause has
	 * been based. (REQUIRED)
	 */
	public void setPdDir(File d) {
		pdDir = d;
		classpath.addPath(d.getAbsolutePath());
	}

	/**
	 * Create a DTD location record. This stores the location of a DTD. The
	 * DTD is identified by its public Id. The location may either be a file
	 * location or a resource location.
	 * @return the DTD location object to be configured by Ant
	 */
	public DTDLocation createDTD() {
		DTDLocation dtdLocation = new DTDLocation();
		antDtdLocations.add(dtdLocation);
		antDtdLocationsSet = true;
		return dtdLocation;
	}

	/**
	 * The assigned boolean value indicates if the generated must be compiled.
	 * The compilation is done after the generation of all classes.
	 * true=> no compilation / false=> the classes will be compiled
	 * This parameter can be override for a class if you use a local jorm
	 * configuration file which defines also the property.
	 * (OPTIONAL: default value is false)
	 */
	public void setJavac(boolean l) {
		antJavac = l;
		antJavacSet = true;
	}

	/**
	 * Defines the list of mappers to be used for compilation. This list is in
	 * the form of "m1.sm1,m1.sm2,m2.sm1" where full mapper names are composed
	 * of a mapper name (e.g., rdb) followed by a "." followed by a sub-mapper
	 * name (e.g., mysql). Full mapper names are separated by ",".
	 * @param mapperNames
	 */
	public void setMapperNames(String mapperNames) {
		if (mapperNames != null && mapperNames.length() > 0) {
			this.mapperNames = mapperNames;
		}
	}

	/**
	 * It assigns the classpath where jorm can load .pd files which are
	 * needed (the dependecies).
	 * (OPTIONAL: default value is the PdDir)
	 */
	public void setOtherPdFiles(Path cp) {
		String[] dl = cp.list();
		for (int i = 0; i < dl.length; i++) {
			classpath.addPath(dl[i]);
			classpathSet = true;
		}
	}

	public void setOtherPdFilesRef(Reference r) {
		Path p = new Path(project);
		p.setRefid(r);
		setOtherPdFiles(p);
	}

	/**
	 * The ouput name is the directory name where the generated files will
	 * be created. Futhermmore the classes will be compiled in this directory.
	 * (REQUIRED)
	 * @param s the string representation of the output directory
	 */
	public void setOutput(String s) {
		antOutput = s;
		antOutputSet = true;
	}

	/**
	 * It assigns the jorm configuration file. (REQUIRED)
	 * @param s the string representation of the configuration file.
	 */
	public void setJormConf(String s) {
		jormConfFile = s;
	}

	/**
	 * It assigns the jorm configuration file. (REQUIRED)
	 * @param s the string representation of the configuration file.
	 */
	public void setLogConf(String s) {
		antLogConfFile = s;
		antLogConfFileSet = true;
	}

	/**
	 * It assigns the project name which must be used.
	 * This parameter can be override for a class if you use a local jorm
	 * configuration file which defines also the property.
	 * (OPTIONAL default value = "" empty string)
	 */
	public void setProjectName(String s) {
		antProjectName = s;
		antProjectNameSet = true;
	}

	/**
	 * It assigns the class name which the generated PBinding classes must
	 * extend.
	 * This parameter can be override for a class if you use a local jorm
	 * configuration file which defines also the property.
	 * (OPTIONAL default value is "")
	 */
	public void setBindingInheritance(String s) {
		antBindingInheritance = s;
		antBindingInheritanceSet = true;
	}

	/**
	 * It assigns the class name which the generated PBinding classes must
	 * extend.
	 * This parameter can be override for a class if you use a local jorm
	 * configuration file which defines also the property.
	 * (OPTIONAL default value is "")
	 */
	public void setMappingInheritance(String s) {
		antClassMappingInheritance = s;
		antClassMappingInheritanceSet = true;
	}

	/**
	 * It assigns the verbose mode to the jorm generator.
	 * This parameter can be override for a class if you use a local jorm
	 * configuration file which defines also the property.
	 * (OPTIONAL the default value is false)
	 */
	public void setVerbose(boolean verbose) {
		antVerbose = verbose;
		antVerboseSet = true;
	}

	/**
	 * It assigns a boolean to indicate if the generated .java file must be
	 * kept.
	 * This parameter can be override for a class if you use a local jorm
	 * configuration file which defines also the property.
	 * (OPTIONAL the default value is true)
	 */
	public void setKeepsrc(boolean keepsrc) {
		antKeepSrc = keepsrc;
		antKeepSrcSet = true;
	}

	/**
	 * It assigns the boolean value which indictes if the generated Binding
	 * must be declared abstract.
	 * This parameter can be override for a class if you use a local jorm
	 * configuration file which defines also the property.
	 * This parameter must be defined in the task or in the jorm configuration
	 * file.
	 */
	public void setBindingAbstract(boolean bindingAbstract) {
		antBindingAbstract = bindingAbstract;
		antBindingAbstractSet = true;
	}


	public void setGeneratePDFiles(boolean val) {
		generatedPDFiles = val;
	}

	/**
	 * execute method of the Ant task
	 */
	public void execute() throws BuildException {
		try {
			compilerConfigurator.configure(jormConfFile);
			if (!antLogConfFileSet) {
				compilerConfigurator.configureLog(jormConfFile);
			} else {
				compilerConfigurator.configureLog(antLogConfFile);
			}
			if (mapperNames != null) {
				StringTokenizer st = new StringTokenizer(mapperNames.trim(), ".,", true);
				if (st.hasMoreTokens()) {
					compilerConfigurator.removeAllMappers();
				}
				while (st.hasMoreTokens()) {
					String mn = st.nextToken(), smn;
					if (mn.equals(".") || mn.equals(",")) {
						String errmsg = "Malformed mapperNames string (expect mapper name): " + mapperNames;
						log(errmsg, project.MSG_ERR);
						throw new BuildException(errmsg);
					}
					if (!st.hasMoreTokens()) {
						String errmsg = "Malformed mapperNames string (expect \".\"): " + mapperNames;
						log(errmsg, project.MSG_ERR);
						throw new BuildException(errmsg);
					}
					smn = st.nextToken();
					if (!smn.equals(".")) {
						String errmsg = "Malformed mapperNames string (expect \".\"): " + mapperNames;
						log(errmsg, project.MSG_ERR);
						throw new BuildException(errmsg);
					}
					if (!st.hasMoreTokens()) {
						String errmsg = "Malformed mapperNames string (expect submapper name): " + mapperNames;
						log(errmsg, project.MSG_ERR);
						throw new BuildException(errmsg);
					}
					smn = st.nextToken();
					compilerConfigurator.addSubMapper(mn, smn);
					if (st.hasMoreTokens()) {
						smn = st.nextToken();
						if (!smn.equals(",")) {
							String errmsg = "Malformed mapperNames string (expect \",\"): " + mapperNames;
							log(errmsg, project.MSG_ERR);
							throw new BuildException(errmsg);
						}
					}
				}
			}
			compilerParameter.loadConfFile(compilerConfigurator.getGlobalJormcOptsFile(),
										   compilerConfigurator.knownMappers());
			if (antDtdLocationsSet) {
				compilerParameter.setDtdLocations(antDtdLocations);
			}
			if (antJavacSet) {
				compilerParameter.setJavac(antJavac);
			}
			if (classpathSet) {
				compilerParameter.setClasspath(classpath);
			}
			if (antOutputSet) {
				compilerParameter.setOutput(antOutput);
			}
			if (antLogConfFileSet) {
				compilerParameter.setLogConfFile(antLogConfFile);
			}
			if (antProjectNameSet) {
				compilerParameter.setProjectName(antProjectName);
			}
			if (antBindingInheritanceSet) {
				compilerParameter.setBindingInheritance(antBindingInheritance);
			}
			if (antClassMappingInheritanceSet) {
				compilerParameter.setClassMappingInheritance(antClassMappingInheritance);
			}
			if (antVerboseSet) {
				compilerParameter.setVerbose(antVerbose);
			}
			if (antKeepSrcSet) {
				compilerParameter.setKeepSrc(antKeepSrc);
			}
			if (antBindingAbstractSet) {
				compilerParameter.setBindingAbstract(antBindingAbstract);
			}
			compilerParameter.setGeneratedPDFiles(generatedPDFiles);
		} catch (PException e) {
			Exception current = e;
			while (current instanceof PException
					&& ((PException) current).getNestedException() != null) {
				current = ((PException) current).getNestedException();
			}
			throw new BuildException(current);
		}
		log("Assigns immutable properties", project.MSG_DEBUG);
		// Assigns the properties which CANNOT be overrieded by the jorm
		// configuration file.
		log("pdDir=" + pdDir, project.MSG_DEBUG);
		String[] pdFiles = super.getDirectoryScanner(pdDir).getIncludedFiles();
		log("Generates(" + pdFiles + ")", project.MSG_DEBUG);
		generates(pdFiles);
	}

	public void generates(String[] pdFileNames) throws BuildException {
		//Add the pd file
		ArrayList compileList = new ArrayList();
		for (int i = 0; i < pdFileNames.length; i++) {
			int idx = pdFileNames[i].lastIndexOf(File.separatorChar);
			String packageName;
			String className;
			if (idx == -1) {
				packageName = null;
				className = pdFileNames[i];
			} else {
				packageName = pdFileNames[i].substring(0, idx);
				className = pdFileNames[i].substring(idx + 1);
			}
			className = className.substring(0, className.lastIndexOf('.'));
			log("-packageName=" + packageName, project.MSG_DEBUG);
			log("-className=" + className, project.MSG_DEBUG);
			File pdFile = new File(pdDir, pdFileNames[i]);
			log("-pdfile=" + pdFile, project.MSG_DEBUG);
			FileSet fs = new FileSet();
			fs.setProject(project);
			fs.setDir(pdDir);
			fs.setIncludes(pdFileNames[i]);
			UpToDate u = new UpToDate();
			u.setProject(project);
			u.addSrcfiles(fs);
			// The target is the generated accessor file or pnamegetter file
			String pngFileName = compilerParameter.getOutput() + File.separator
					+ (packageName == null ? "" : packageName + File.separator)
					+ File.separator + className + "PNG.java";
			log("\tpngFileName=" + pngFileName, project.MSG_DEBUG);
			String accFileName = compilerParameter.getOutput() + File.separator
					+ (packageName == null ? "" : packageName + File.separator)
					+ File.separator + className + "Accessor.java";
			log("\taccFileName=" + accFileName, project.MSG_DEBUG);
			boolean mustGenerate;
			File pngFile = new File(pngFileName);
			File accFile = new File(accFileName);
			if (accFile.exists()) {
				u.setTargetFile(accFile);
				mustGenerate = !u.eval();
			} else if (pngFile.exists()) {
				u.setTargetFile(pngFile);
				mustGenerate = !u.eval();
			} else {
				// else no file exists => generate
				mustGenerate = true;
			}
			if (mustGenerate)
				compileList.add(pdFileNames[i]);
		}
		if (compileList.size() > 0) {
			project.log("not upToDate", project.MSG_DEBUG);
			compilerParameter.setInputFiles(compileList);
			try {
				boolean javac = compilerParameter.isJavac();
				compilerParameter.setJavac(false);
				compiler.process();
				if (javac) {
					log("javac compilation", project.MSG_DEBUG);
					// compiles the generated java files
					Javac javacinst = new Javac();
					Path path = new Path(project, compilerParameter.getOutput());
					javacinst.setProject(project);
					javacinst.setSrcdir(path);
					javacinst.setDestdir(new File(compilerParameter.getOutput()));
					javacinst.setDebug(true);
					javacinst.setIncludes("**/*.java");
					//javacinst.setClasspath(classpath);
					javacinst.execute();
				}
			} catch (PException e) {
				Exception current = e;
				while (current instanceof PException
						&& ((PException) current).getNestedException() != null) {
					current = ((PException) current).getNestedException();
				}
				current.printStackTrace();
				throw new BuildException(current);
			}
		} else {
			project.log("upToDate", project.MSG_DEBUG);
		}
	}
}
