/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.lib;

import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PMapperListener;
import org.objectweb.jorm.api.PMapCluster;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.xml2mi.api.Parser;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.genclass.lib.GenClassMapping;
import org.objectweb.perseus.cache.api.CacheManager;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.util.monolog.api.Logger;

import java.util.Collection;
import java.util.Map;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Collections;
import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestJormPathHelper extends TestCase {

    public TestJormPathHelper(String s) throws Exception {
        super(s);
    }

    public static char SEP = JormPathHelper.SEP;

    MyMapper mapper ;

    protected void setUp() throws Exception {
        mapper = new MyMapper();
    }

    protected void tearDown() throws Exception {
        mapper = null;
    }

    public void testClass() {
        PBinder binder = new MyBinder();
        String className = "com.foo.Bar";
        PClassMapping pcm = new MyPCM(className,
                Collections.EMPTY_MAP,
                Collections.EMPTY_MAP,
                binder,
                mapper);
        String path = className;
        Assert.assertEquals("Bad PCM with " + SEP, pcm,
                JormPathHelper.getPClassMapping(SEP + path, mapper));
        Assert.assertEquals("Bad PCM without " + SEP, pcm,
                JormPathHelper.getPClassMapping(path, mapper));
        Assert.assertEquals("Bad PNC with " + SEP, binder,
                JormPathHelper.getPNameCoder(SEP + path, mapper));
        Assert.assertEquals("Bad PNC without " + SEP, binder,
                JormPathHelper.getPNameCoder(path, mapper));
    }

    public void testClassRefInClass() {
        PBinder binder = new MyBinder();
        String className = "com.foo.Bar";
        PNameCoder pncRef1 = new MyBinder();
        new MyPCM(className,
                Collections.singletonMap("ref1", pncRef1),
                Collections.EMPTY_MAP,
                binder,
                mapper);
        String path = className + SEP + "ref1";
        Assert.assertEquals("Bad PNC with " + SEP, pncRef1,
                JormPathHelper.getPNameCoder(SEP + path, mapper));
        Assert.assertEquals("Bad PNC without " + SEP, pncRef1,
                JormPathHelper.getPNameCoder(path, mapper));
    }

    public void testGenClassRefInClass() {
        PBinder binder = new MyBinder();
        String className = "com.foo.Bar";
        PNameCoder pncRef1 = new MyBinder();
        MyGCM gcmRef1 = new MyGCM();
        new MyPCM(className,
                Collections.singletonMap("ref1", pncRef1),
                Collections.singletonMap("ref1", gcmRef1),
                binder,
                mapper);
        String path = className + SEP + "ref1";
        Assert.assertEquals("Bad PCM with " + SEP, gcmRef1,
                JormPathHelper.getPClassMapping(SEP + path, mapper));
        Assert.assertEquals("Bad PCM without " + SEP, gcmRef1,
                JormPathHelper.getPClassMapping(path, mapper));
        Assert.assertEquals("Bad PNC with " + SEP, pncRef1,
                JormPathHelper.getPNameCoder(SEP + path, mapper));
        Assert.assertEquals("Bad PNC without " + SEP, pncRef1,
                JormPathHelper.getPNameCoder(path, mapper));
    }

    public void testClassRefInGenClassRefInClass() {
        String className2 = "com.foo.Bar2";
        PBinder binder2 = new MyBinder();
        new MyPCM(className2,
                Collections.EMPTY_MAP,
                Collections.EMPTY_MAP,
                binder2,
                mapper);

        PBinder binder1 = new MyBinder();
        String className1 = "com.foo.Bar1";
        PNameCoder pncRef1 = new MyBinder();
        MyGCM gcmRef1 = new MyGCM(binder2);
        new MyPCM(className1,
                Collections.singletonMap("ref1", pncRef1),
                Collections.singletonMap("ref1", gcmRef1),
                binder1,
                mapper);

        String path = className1 + SEP + "ref1" + SEP + "java.util.Set"
                + JormPathHelper.ELEMENT;
        Assert.assertEquals("Bad PNC with " + SEP, binder2,
                JormPathHelper.getPNameCoder(SEP + path, mapper));
        Assert.assertEquals("Bad PNC without " + SEP, binder2,
                JormPathHelper.getPNameCoder(path, mapper));
    }

    public class MyMapper implements PMapper {
        Map name2pcm;

        public MyMapper() {
            name2pcm = new HashMap();
        }

        public MyMapper(Map m) {
            name2pcm = m;
        }

		public void setLogger(Logger l) {
		}

		public void setPMapper(PMapper pm) {
		}

        public PClassMapping lookup(String classname) {
            return (PClassMapping) name2pcm.get(classname);
        }

        public void map(PClassMapping pcm) {
            name2pcm.put(pcm.getClassName(), pcm);
        }

        public void map(Object conn, PClassMapping pcm) {
            name2pcm.put(pcm.getClassName(), pcm);
        }

        public void map(Object conn, PClassMapping pcm, boolean loadmeta) {
            name2pcm.put(pcm.getClassName(), pcm);
        }

        public void closeConnection(Object conn) throws PException {
        }

        public String cn2mn(String cn) {
            return null;
        }

        public Object getConnection() throws PException {
            return null;
        }

        public Object getConnection(Object ctxt) throws PException {
            return null;
        }

	    public Object getConnection(Object connectionContext, Object user) throws PException {
		    return null;
	    }

        public String getMapperName() {
            return null;
        }

        public PMappingStructuresManager getPMappingStructuresManager() {
            return null;
        }

        public void setConnectionFactory(Object cf) throws PException {
        }

        public Object getConnectionFactory() {
            return null;
        }

        public void setPrefetchCache(PrefetchCache pc) throws PException {
        }

        public PrefetchCache getPrefetchCache() {
            return null;
        }

        public void setMapperName(String mappername) {
        }

        public Manager getMetaInfoManager() {
            return null;
        }

        public void setMetaInfoManager(Manager m) {
        }

        public void setParser(Parser p) {
        }

        public void setPTypeSpace(PTypeSpace pts) {
        }

        public PTypeSpace getPTypeSpace() {
            return null;
        }

        public void start() throws PException {
        }

        public void stop() throws PException {
        }

        public void unmap(String classname) throws PException {
        }

        public void addMapperEventListener(PMapperListener listener) {
        }

        public void removeMapperEventListener(PMapperListener listener) {
        }

        public PClassMapping createGenClassMapping() throws PException {
            return null;
        }

        public PMapCluster getPMapCluster(String jcname) throws PException {
            return null;
        }

        public Collection getPMapClusters() {
            return null;
        }

        public void addDependency(String jcname1, String jcname2) throws PException {
        }

        public void classDefined(String jcname) throws PException {
        }

		public void declareClass(String jcname) {
		}
    }

    public class MyPCM implements PClassMapping {
        String className;
        Map pncs;
        Map gcms;
        PClassMapping pcm;
        PBinder binder;
        PMapper mapper;

        public MyPCM(String className,
                     Map pncs,
                     Map gcms,
                     PBinder binder,
                     MyMapper mapper) {
            this.className = className;
            this.pncs = pncs;
            this.gcms = gcms;
            this.binder = binder;
            this.mapper = mapper;
            mapper.map(this);
        }

        public PBinding createPBinding() throws PException {
            return null;
        }

        public void init(PMappingCallback mapper, MetaObject metaclass) throws PException {
        }

        public String getClassName() {
            return className;
        }

        public String getProjectName() {
            return null;
        }

		public PNameCoder getClassPNameCoder() {
			return binder;
		}

        public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
            fail("Call the 'getGenClassMapping' method on a PClassMapping of the class '"
                    + className + "' is not authorized");
            return null;
        }

        public PClassMapping getGenClassMapping(String fn)
                throws UnsupportedOperationException {
            PClassMapping pcm = (PClassMapping) gcms.get(fn);
            if (pcm == null) {
                fail("No field '" + fn + "' availlable on the class '" + className + "'");
            }
            return pcm;
       }

        public MetaObject getMetaInfo() {
            return null;
        }

        public PBinder getPBinder() {
            return binder;
        }

        public PMapper getPMapper() {
            return mapper;
        }

        public PNameCoder getPNameCoder()
                throws UnsupportedOperationException {
            fail("Call the 'getPNameCoder' method on a PClassMapping of the class '"
                    + className + "' is not authorized");
            return null;
        }

        public PNameCoder getPNameCoder(String fn)
                throws UnsupportedOperationException {
            PNameCoder pc = (PNameCoder) pncs.get(fn);
            if (pc == null) {
                fail("No field '" + fn + "' availlable on the class '" + className +"'");
            }
            return pc;
        }

        public Iterator getPNameIterator(Object conn) throws PException {
            return null;
        }

	    public PNameIterator getPNameIterator(Object conn,
	                                          boolean withSubType,
	                                          boolean prefetching,
											  Object txctx) throws PException {
		    return null;
	    }

        public PType getPType() {
            return null;
        }

        public boolean isConform(String mappername) {
            return false;
        }

        public void setPBinder(PBinder pb) throws PException {
            binder = pb;
        }

        public void configureRefFields(PClassMapping.ReferenceConfigurator rc)
                throws PException, UnsupportedOperationException {
        }
        
		public boolean exist(PBinding pb, Object conn) throws PException {
			return false;
		}
		public void read(PBinding pb, Object conn, PAccessor pa, Object txctx)
				throws PException {
		}
		public void read(PBinding pb, Object conn, PAccessor pa)
				throws PException {
		}
		public void write(PBinding pb, Object conn, PAccessor pa)
				throws PException {
		}
        public void addAssociation(PClassMapping targetClass, int[] indexes) {
        }
        public HashMap getAssociationTable() {
            return null;
        }
        public int[] getIndexesTable(PClassMapping targetClass) {
            return null;
        }
        public PClassMapping[] getSubPCMs() throws PException {
            return null;
        }
    }

    public class MyGCM extends GenClassMapping {

        /** Contains PrimitiveElement */
        public MyGCM() {
            super();
        }

        /** Contains ClassRef */
        public MyGCM(PNameCoder pnc) {
            super();
            if (pnc != null) {
                try {
                    setPNameCoder(pnc);
                } catch (Exception e) {
                    fail(e.getMessage());
                }
            }
        }

        /** Contains GenClassRef */
        public MyGCM(PClassMapping pcm, PNameCoder pnc) {
            super();
            if (pcm != null) {
                setGenClassMapping(pcm);
            }
            if (pnc != null) {
                try {
                    setPNameCoder(pnc);
                } catch (Exception e) {
                    fail(e.getMessage());
                }
            }
        }

        public PBinding createPBinding() throws PException {
            return null;
        }

        public Iterator getPNameIterator(Object conn) throws PException {
            return null;
        }

	    public PNameIterator getPNameIterator(Object conn,
	                                          boolean withSubType,
	                                          boolean prefetching,
											  Object txctx) throws PException {
		    return null;
	    }

        public boolean isConform(String mappername) {
            return false;
        }
		public boolean exist(PBinding pb, Object conn) throws PException {
			return false;
		}
		public void read(PBinding pb, Object conn, PAccessor pa, Object txctx)
				throws PException {
		}
		public void read(PBinding pb, Object conn, PAccessor pa)
				throws PException {
		}
		public void write(PBinding pb, Object conn, PAccessor pa)
				throws PException {
		}
		
        public void addAssociation(PClassMapping targetClass, int[] indexes) {
        }
        public HashMap getAssociationTable() {
            return null;
        }
        public int[] getIndexesTable(PClassMapping targetClass) {
            return null;
        }
        public PClassMapping[] getSubPCMs() throws PException {
            return null;
        }
    }

    public class MyBinder implements PBinder {
        public PBinding lookup(PName pn) throws PException {
            return null;
        }

        public void bind(PName pn, PBindingCtrl pb) throws PException {
        }

        public PClassMapping getBinderClassMapping() {
            return null;
        }

        public void setPClassMapping(PClassMapping pcm) {
        }

        public void unbind(PBindingCtrl pb) throws PException {
        }

        public CacheManager getCacheManager() {
            return null;
        }

        public void setCacheManager(CacheManager cm) throws PException {
        }

        public PName export(Object conn, Object infoitem) throws PException {
            return null;
        }

        public PName export(Object conn, Object infoitem, Object hints)
                throws PException {
            return null;
        }

        public void unexport(Object conn, PName pn) throws PException {
        }

        public void unexport(Object conn, PName pn, Object hints) throws PException {
        }

        public boolean codingSupported(int codingtype) {
            return false;
        }

        public PName decode(byte[] en) throws PExceptionNaming {
            return null;
        }

        public PName decodeAbstract(Object en, Object context) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeByte(byte en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeObyte(Byte en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeChar(char en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeOchar(Character en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeInt(int en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeOint(Integer en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeLong(long en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeOlong(Long en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeShort(short en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeOshort(Short en) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public PName decodeString(String en) throws PExceptionNaming {
            return null;
        }

        public PName decodeCharArray(char[] en) throws PExceptionNaming {
            return null;
        }

		public PName decodeDate(Date en) throws PExceptionNaming {
			return null;
		}

		public PName decodeBigDecimal(BigDecimal en) throws PExceptionNaming {
			return null;
		}

		public PName decodeBigInteger(BigInteger en) throws PExceptionNaming {
			return null;
		}

        public byte[] encode(PName pn) throws PExceptionNaming {
            return new byte[0];
        }

        public Object encodeAbstract(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public byte encodeByte(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return 0;
        }

        public Byte encodeObyte(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public char encodeChar(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return 0;
        }

        public Character encodeOchar(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public int encodeInt(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return 0;
        }

        public Integer encodeOint(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public long encodeLong(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return 0;
        }

        public Long encodeOlong(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public short encodeShort(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return 0;
        }

        public Short encodeOshort(PName pn) throws PExceptionNaming, UnsupportedOperationException {
            return null;
        }

        public String encodeString(PName pn) throws PExceptionNaming {
            return null;
        }

        public char[] encodeCharArray(PName pn) throws PExceptionNaming {
            return new char[0];
        }

		public Date encodeDate(PName pn) throws PExceptionNaming {
			return null;
		}

		public BigDecimal encodeBigDecimal(PName pn) throws PExceptionNaming {
			return null;
		}

		public BigInteger encodeBigInteger(PName pn) throws PExceptionNaming {
			return null;
		}

        public PName getNull() {
            return null;
        }

        public void setNullPName(Object o) throws PException {
        }

        public boolean supportDynamicComposite() {
            return false;
        }

        public boolean supportCompositeField(String fn, PType ft) {
            return false;
        }

        public boolean supportStaticComposite() {
            return false;
        }

        public PType getPType() {
            return null;
        }

        public void setPType(PType pt) {
        }
    }
}


