/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.mapper.fos.lib;

import junit.framework.TestCase;
import org.objectweb.jorm.api.PException;

/**
 * Code for testing the RDB mapping structures implementation. It only tests
 * the definition of mapping structures and does not test operations that
 * modify the database schema.
 * @author P. Dechamboux
 */
public class TestFosPMappingStructuresManager extends TestCase {
	private final static String JCN1 = "org.objectweb.test.C1";
	private final static String JCN2 = "org.objectweb.test.C2";

	private final static String D1 = "org/d1";
	private final static String D2 = "org/d2long";
	private final static String D11 = "org/d1/d1";

	public TestFosPMappingStructuresManager(String testname) {
		super(testname);
	}

	protected void setUp() {

	}

	protected void tearDown() {

	}

	public void testClusterWith1Class1Dir() {
		try {
			FosPMappingStructuresManager man = new FosPMappingStructuresManager();
			man.addDirName(JCN1, D1);
			FosPMapCluster cl = (FosPMapCluster) man.getPMapCluster(JCN1);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Directory " + D1 + " does not exist in cluster", cl.containDirectory(D1));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith1Class1DirDup() {
		try {
			FosPMappingStructuresManager man = new FosPMappingStructuresManager();
			man.addDirName(JCN1, D1);
			man.addDirName(JCN1, D1);
			FosPMapCluster cl = (FosPMapCluster) man.getPMapCluster(JCN1);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Directory " + D1 + " does not exist in cluster", cl.containDirectory(D1));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith1Class2DirSubBefore() {
		try {
			FosPMappingStructuresManager man = new FosPMappingStructuresManager();
			man.addDirName(JCN1, D1);
			man.addDirName(JCN1, D11);
			FosPMapCluster cl = (FosPMapCluster) man.getPMapCluster(JCN1);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Directory " + D11 + " does not exist in cluster", cl.containDirectory(D11));
			assertTrue("Directory " + D1 + " does not exist in cluster", cl.containDirectory(D1));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith1Class2DirSubAfter() {
		try {
			FosPMappingStructuresManager man = new FosPMappingStructuresManager();
			man.addDirName(JCN1, D1);
			man.addDirName(JCN1, D11);
			FosPMapCluster cl = (FosPMapCluster) man.getPMapCluster(JCN1);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Directory " + D1 + " does not exist in cluster", cl.containDirectory(D1));
			assertTrue("Directory " + D11 + " does not exist in cluster", cl.containDirectory(D11));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith2Class2DirNoMerge() {
		try {
			FosPMappingStructuresManager man = new FosPMappingStructuresManager();
			man.addDirName(JCN1, D1);
			man.addDirName(JCN2, D2);
			FosPMapCluster cl = (FosPMapCluster) man.getPMapCluster(JCN1);
			FosPMapCluster cl2 = (FosPMapCluster) man.getPMapCluster(JCN2);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Cluster has not been created for " + JCN2, cl2 != null);
			assertTrue("The two classes are into the same cluster", cl != cl2);
			assertTrue("Directory " + D1 + " does not exist in cluster", cl.containDirectory(D1));
			assertTrue("Directory " + D2 + " does not exist in cluster", cl2.containDirectory(D2));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testClusterWith2Class3DirMerge() {
		try {
			FosPMappingStructuresManager man = new FosPMappingStructuresManager();
			man.addDirName(JCN1, D1);
			man.addDirName(JCN1, D2);
			man.addDirName(JCN2, D11);
			FosPMapCluster cl = (FosPMapCluster) man.getPMapCluster(JCN1);
			FosPMapCluster cl2 = (FosPMapCluster) man.getPMapCluster(JCN2);
			assertTrue("Cluster has not been created for " + JCN1, cl != null);
			assertTrue("Cluster has not been created for " + JCN2, cl2 != null);
			assertTrue("The two classes are not into the same cluster", cl == cl2);
			assertTrue("Directory " + D1 + " does not exist in cluster", cl.containDirectory(D1));
			assertTrue("Directory " + D2 + " does not exist in cluster", cl.containDirectory(D2));
			assertTrue("Directory " + D11 + " does not exist in cluster", cl.containDirectory(D11));
		} catch (PException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}
}
