/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime;

import junit.framework.TestCase;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapCluster;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.MapperFactory;
import org.objectweb.jorm.ExceptionHelper;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.wrapper.p6spy.P6SpyLogger;
import org.objectweb.medor.eval.prefetch.lib.PrefetchCacheImpl;

import java.util.Iterator;
import java.util.Properties;
import java.util.Date;
import java.util.Calendar;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.io.InputStream;
import java.io.FileInputStream;

/**
 *
 * @author S.Chassande-Barrioz
 */
public abstract class TestRuntimeHelper 
        extends TestCase
        implements PClassMapping.ReferenceConfigurator {

    public final static String LOG_NAME = "org.objectweb.jorm";
    public final static String TESTCONFIGFILE = "TESTCONFIG";

    public PMapper mapper = null;
    public Logger logger = null;
    public LoggerFactory loggerFactory = null;

    private Properties configprop = null;
    private MapperFactory mapperFactory = new MapperFactory();
    private Logger spylogger = null;
    private Map name2pcm = new HashMap();

    public TestRuntimeHelper(String s) throws Exception {
        super(s);
        String fn = System.getProperty(TESTCONFIGFILE);
        if (fn == null) {
            throw new Exception("Cannot read environment variable "
                                + TESTCONFIGFILE + " in order to configure the test case.");
        }
        InputStream is = new FileInputStream(fn);
        if (is == null) {
            throw new Exception("Cannot load file " + fn
                                + " in order to configure the test case.");
        }
        configprop = new Properties();
        configprop.load(is);
    }

    protected abstract String getLoggerName();

    protected void setUp() throws Exception {
        mapper = mapperFactory.createMapper(configprop);
        mapper.start();
        loggerFactory = ((Loggable) mapper).getLoggerFactory();
        logger = loggerFactory.getLogger(getLoggerName());
        P6SpyLogger.logger = logger;
		mapper.setPrefetchCache(new PrefetchCacheImpl(logger));
    }

    protected void tearDown() throws Exception {
        mapper.stop();
        mapper = null;
        logger = null;
        P6SpyLogger.logger = spylogger;
    }

    /**
     * It changes the current logger to a new Logger with the name specified in
     * parameter.
     * @param name the name of the new Logger
     */
    protected void changeLogger(String name) {
        logger = loggerFactory.getLogger(name);
        P6SpyLogger.logger = logger;
    }


    protected PClassMapping getMapping(String cn) {
        PClassMapping pcm = mapper.lookup(cn);
        if (pcm == null) {
            pcm = (PClassMapping) name2pcm.get(cn);
        }
        if (pcm == null) {
            try {
                logger.log(BasicLevel.DEBUG, "Initializing the PCM of the class " + cn);
                pcm = (PClassMapping) Class.forName(
                        mapper.cn2mn(cn) + PMapper.PCLASSMAPPINGAPPENDER)
                        .newInstance();
                PBinder binder = getBinder(cn);
                binder.setPClassMapping(pcm);
                pcm.setPBinder(binder);
                name2pcm.put(cn, pcm);
                pcm.configureRefFields(this);
                mapper.map(pcm);
                name2pcm.remove(cn);
                PMapCluster cl = mapper.getPMapCluster(cn);
                logger.log(BasicLevel.DEBUG, "cluster(" + cn + "):" + cl);
                if (!cl.isDefined()) {
                    Set unresolvedDep = cl.getUnResolvedDependencies();
                    Iterator it = unresolvedDep.iterator();
                    // if there is a dependency left, treat it
                    // the algo stops eventually because at least one element of unresolvedDep is removed
                    // at each recursion
                    if (it.hasNext()) {
                        String className = (String) it.next();
                        logger.log(BasicLevel.DEBUG, "Resolving next dependency: " + className);
                        getMapping(className);
                        logger.log(BasicLevel.DEBUG, "Dependency " + className + " resolved");
                    } 
                } else {
                    logger.log(BasicLevel.DEBUG, "Deleting the hosting structure");
                    cl.deleteMappingStructures();
                    logger.log(BasicLevel.DEBUG, "Creating a hosting structure");
                    cl.createMappingStructures(true);
                }
            } catch (Exception e) {
                Exception inner = ExceptionHelper.getFirstException(e);
                logger.log(BasicLevel.ERROR, logger.getName(), inner);
                fail(logger.getName() + ": " + inner.getMessage());
            }
        }
        return pcm;
    }

    protected abstract PBinder getBinder(String className) throws Exception;

    protected boolean performIO(String cn, PAccessor towrite,
                                PAccessor toread, Object hints) {
        return performIO(cn, towrite, toread, hints, true);
    }
    
    protected boolean performIO(String cn,
                                PAccessor towrite,
                                PAccessor toread, Object hints,
                                boolean doUnexport) {
        Object conn = null;
        boolean res = true;
        try {
            PClassMapping pcm = getMapping(cn);
            conn = mapper.getConnection();
            PBinding pb1 = pcm.createPBinding();
            if (hints == null) {
                pb1.export(conn);
            } else {
                pb1.export(conn, hints);
            }
            pb1.write(conn, towrite);
            pb1.read(conn, toread);
            if (doUnexport) {
                pb1.unexport(conn);
                pb1.write(conn, towrite);
            }
        } catch (Exception e) {
            Exception inner = ExceptionHelper.getFirstException(e);
            logger.log(BasicLevel.ERROR, logger.getName(), inner);
            fail(logger.getName() + ": " + inner.getMessage());
            res = false;
        } finally {
            if (conn != null) {
                try {
                    mapper.closeConnection(conn);
                } catch (Exception e) {
                }
                conn = null;
            }
        }
        return res;
    }

    protected boolean performIO(String cn,
                                PAccessor towrite,
                                PAccessor toread) {
        return performIO(cn, towrite, toread, null);
    }

    public Date getTime() {
        return new Date((Calendar.getInstance().getTime().getTime() / 1000) * 1000);
    }

	protected PBinding export(String cn, Object hints) {
		Object conn = null;
		PBinding pb = null;
		PClassMapping pcm;
		try {
			int fi = cn.indexOf("/");
			if (fi != -1) {
 				int li, nbgc = 0;
				String scn = cn.substring(0, fi);
				String fn;
				li = cn.indexOf("/", fi + 1);
				if (li == -1) {
					fn = cn.substring(fi + 1);
				} else {
					fn = cn.substring(fi + 1, li);
					do {
						nbgc++;
						li = cn.indexOf("/", li + 1);
					} while (li != -1);
				}
                
 				pcm = getMapping(scn);
 				pcm = pcm.getGenClassMapping(fn);
 			} else {
 			   pcm = getMapping(cn);
 			}
 			conn = mapper.getConnection();
 			pb = pcm.createPBinding();
			if (hints == null) {
				pb.export(conn);
			} else {
				pb.export(conn, hints);
			}
		} catch (Exception e) {
			Exception inner = ExceptionHelper.getFirstException(e);
			logger.log(BasicLevel.ERROR, logger.getName(), inner);
			fail(logger.getName() + ": " + inner.getMessage());
		} finally {
			if (conn != null) {
				try {
					mapper.closeConnection(conn);
				} catch (Exception e) {
				}
				conn = null;
			}
		}
		return pb;
	}

	protected PBinding bind(String cn, PName pn) {
		Object conn = null;
		PBinding pb = null;
		PClassMapping pcm;
		try {
			int fi = cn.indexOf("/");
			if (fi != -1) {
				int li, nbgc = 0;
				String scn = cn.substring(0, fi);
				String fn;
				li = cn.indexOf("/", fi + 1);
				if (li == -1) {
					fn = cn.substring(fi + 1);
				} else {
					fn = cn.substring(fi + 1, li);
					do {
						nbgc++;
						li = cn.indexOf("/", li + 1);
					} while (li != -1);
				}
				pcm = getMapping(scn);
				pcm = pcm.getGenClassMapping(fn);
				for (int i = 0; i < nbgc; i++) {
					pcm = pcm.getGenClassMapping();
				}
			} else {
				pcm = getMapping(cn);
			}
			conn = mapper.getConnection();
			pb = pcm.createPBinding();
			pb.bind(pn);
		} catch (Exception e) {
			Exception inner = ExceptionHelper.getFirstException(e);
			logger.log(BasicLevel.ERROR, logger.getName(), inner);
			fail(logger.getName() + ": " + inner.getMessage());
		} finally {
			if (conn != null) {
				try {
					mapper.closeConnection(conn);
				} catch (Exception e) {
				}
				conn = null;
			}
		}
		return pb;
	}

    protected boolean writeRead(PBinding pb, PAccessor towrite,
                                PAccessor toread) {

        Object conn = null;
        boolean res = true;
        try {
            conn = mapper.getConnection();
            pb.write(conn, towrite);
            pb.read(conn, toread);
        } catch (Exception e) {
            Exception inner = ExceptionHelper.getFirstException(e);
            logger.log(BasicLevel.ERROR, logger.getName(), inner);
            fail(logger.getName() + ": " + inner.getMessage());
            res = false;
        } finally {
            if (conn != null) {
                try {
                    mapper.closeConnection(conn);
                } catch (Exception e) {
                }
                conn = null;
            }
        }
        return res;
    }

    protected boolean unexport(PBinding pb, PAccessor pa) {
        Object conn = null;
        boolean res = true;
        try {
            conn = mapper.getConnection();
            pb.unexport(conn);
            pb.write(conn, pa);
        } catch (PException e) {
            Exception inner = ExceptionHelper.getFirstException(e);
            logger.log(BasicLevel.ERROR, logger.getName(), inner);
            fail(logger.getName() + ": " + inner.getMessage());
            res = false;
        } finally {
            if (conn != null) {
                try {
                    mapper.closeConnection(conn);
                } catch (Exception e) {
                }
                conn = null;
            }
        }
        return res;
    }

    public void assertEqual(String msg, byte[] ba1, byte[] ba2) {
        if (ba1 == null) {
            assertNull(msg + "Must be null, found: " + ba2, ba2);
        } else {
            assertNotNull(msg + "Must not be null, expected " + ba1, ba2);
            assertEquals(msg + "Bad length", ba1.length, ba2.length);
            for (int i = 0; i < ba1.length; i++) {
                assertEquals(msg + "Bad " + i + " character", ba1[i], ba2[i]);
            }
        }
    }

    public void assertEqual(String msg, char[] ba1, char[] ba2) {
        if (ba1 == null) {
            assertNull(msg + "Must be null, found: " + ba2.toString(), ba2);
        } else {
            assertNotNull(msg + "Must not be null, expected " + ba1.toString(), ba2);
            assertEquals(msg + "Bad length", ba1.length, ba2.length);
            for (int i = 0; i < ba1.length; i++) {
                assertEquals(msg + "Bad " + i + " character", ba1[i], ba2[i]);
            }
        }
    }

    // IMPLEMENTATION OF THE ReferenceConfigurator INTERFACE //
    //-------------------------------------------------------//

    public boolean isCodingOnly(String className, String refFieldName) {
        return true;
    }

    public PNameCoder getPNameCoder(String sourceclassName,
                                    String refFieldName,
                                    String destclassName) {
        return getMapping(destclassName).getPBinder();
    }

    public PNameCoder getPNameCoder(String sourceclassName,
                                    String refFieldName,
                                    String[] genClassNames) {
        String gcname = sourceclassName + "/" + refFieldName;
        for (int i = 0; i < genClassNames.length - 1; i++) {
            gcname += genClassNames[i];
        }
        try {
            return getBinder(gcname);
        } catch (Exception e) {
            return null;
        }
    }

    public PClassMapping getGenClassMapping(String sourceclassName,
                                            String refFieldName,
                                            String[] genClassNames) {
        String gcname = sourceclassName + "/" + refFieldName;
        for (int i = 0; i < genClassNames.length - 1; i++) {
            gcname += genClassNames[i];
        }
        try {
			PClassMapping pcm = mapper.createGenClassMapping();
            pcm.setPBinder(getBinder(gcname));
            return pcm;
        } catch (Exception e) {
            return null;
        }
    }

    public PClassMapping getGenClassMapping(String sourceclassName,
                                            String refFieldName,
                                            String[] genClassNames,
                                            String destclassName) {
        PClassMapping pcm = getGenClassMapping(sourceclassName, refFieldName, genClassNames);
        try {
            ((PClassMappingCtrl) pcm).setPNameCoder(getBinder(destclassName));
            return pcm;
        } catch (Exception e) {
            return null;
        }
    }
}
