/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.basictype;

import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.naming.api.PBinder;

import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * @author  P. Dechamboux
 */
public class TestBasicType extends TestRuntimeHelper {
    private final static String LOGGER_NAME
            = "test.org.objectweb.jorm.basictype";

    public TestBasicType(String name) throws Exception {
        super(name);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        try {
            return new BasidBinder(mapper,
                    "org.objectweb.jorm.runtime.basictype.TestBasicType",
                    PClassMapping.CLEANUP_REMOVEALL);
        } catch (PException e) {
            Exception current = e;
            while (current instanceof PException
                    && ((PException) current).getNestedException() != null) {
                current = ((PException) current).getNestedException();
            }
            throw current;
        }
    }
    
    // Definition of test units

    // test units for boolean
    private final static String CNBOOLEAN = "org.objectweb.jorm.pobject.basictype.Boolean";

    public void testBooleanTrue() throws Exception {
        changeLogger(LOGGER_NAME + ".boolean");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(true);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(false);
        if (performIO(CNBOOLEAN, acc1, acc2)) {
            assertEquals("Cannot read written boolean value",
                       true, acc2.paGetBooleanValue());
        }
    }

    public void testBooleanFalse() throws Exception {
        changeLogger(LOGGER_NAME + ".boolean");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(false);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(true);
        if (performIO(CNBOOLEAN, acc1, acc2)) {
            assertEquals("Cannot read written boolean value",
                       false, acc2.paGetBooleanValue());
        }
    }

    public void testBooleanGen() throws Exception {
        changeLogger(LOGGER_NAME + ".boolean");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.BOOLEAN, "booleanValue", Boolean.TRUE);
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.BOOLEAN, "booleanValue");
        if (performIO(CNBOOLEAN, acc1, acc2)) {
            assertEquals("Cannot read written boolean value",
                       true, acc2.paGetBooleanField("booleanValue"));
        }
    }

    // test units for oboolean
    private final static String CNOBOOLEAN = "org.objectweb.jorm.pobject.basictype.Oboolean";

    public void testObooleanNull() throws Exception {
        changeLogger(LOGGER_NAME + ".boolean");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Boolean(false));
        if (performIO(CNOBOOLEAN, acc1, acc2)) {
            assertEquals("Cannot read written boolean value",
                         null, acc2.paGetObooleanValue());
        }
    }

    public void testObooleanTrue() throws Exception {
        changeLogger(LOGGER_NAME + ".boolean");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Boolean(true));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Boolean(false));
        if (performIO(CNOBOOLEAN, acc1, acc2)) {
            assertEquals("Cannot read written boolean value",
                       true, acc2.paGetObooleanValue().booleanValue());
        }
    }

    public void testObooleanFalse() throws Exception {
        changeLogger(LOGGER_NAME + ".boolean");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Boolean(false));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Boolean(true));
        if (performIO(CNOBOOLEAN, acc1, acc2)) {
            assertEquals("Cannot read written boolean value",
                       false, acc2.paGetObooleanValue().booleanValue());
        }
    }

    public void test0booleanGen() throws Exception {
        changeLogger(LOGGER_NAME + ".boolean");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.OBJBOOLEAN, "obooleanValue", Boolean.TRUE);
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.OBJBOOLEAN, "obooleanValue");
        if (performIO(CNOBOOLEAN, acc1, acc2)) {
            assertEquals("Cannot read written boolean value",
                       Boolean.TRUE, acc2.paGetObooleanField("obooleanValue"));
        }
    }

    // test units for byte
    private final static String CNBYTE = "org.objectweb.jorm.pobject.basictype.Byte";

    public void testByte0() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        BasicTypeAccessor acc1 = new BasicTypeAccessor((byte) 0);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((byte) -1);
        if (performIO(CNBYTE, acc1, acc2)) {
            assertEquals("Cannot read written byte value",
                         (byte) 0, acc2.paGetByteValue());
        }
    }

    public void testByteMin() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Byte.MIN_VALUE);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((byte) 0);
        if (performIO(CNBYTE, acc1, acc2)) {
            assertEquals("Cannot read written byte value",
                         Byte.MIN_VALUE, acc2.paGetByteValue());
        }
    }

    public void testByteMax() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Byte.MAX_VALUE);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((byte) 0);
        if (performIO(CNBYTE, acc1, acc2)) {
            assertEquals("Cannot read written byte value",
                         Byte.MAX_VALUE, acc2.paGetByteValue());
        }
    }
    public void testByteGen() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.BYTE, "byteValue", new Byte((byte)34));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.BYTE, "byteValue");
        if (performIO(CNBYTE, acc1, acc2)) {
            assertEquals("Cannot read written byte value",
                       (byte)34, acc2.paGetByteField("byteValue"));
        }
    }

    // test units for byte

    // test units for obyte
    private final static String CNOBYTE = "org.objectweb.jorm.pobject.basictype.Obyte";

    public void testObyteNull() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Byte((byte) -1));
        if (performIO(CNOBYTE, acc1, acc2)) {
            assertEquals("Cannot read written Byte value",
                         null, acc2.paGetObyteValue());
        }
    }

    public void testObyte0() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Byte((byte) 0));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOBYTE, acc1, acc2)) {
            assertEquals("Cannot read written Byte value",
                         (byte) 0, acc2.paGetObyteValue().byteValue());
        }
    }

    public void testObyteMin() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Byte(Byte.MIN_VALUE));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOBYTE, acc1, acc2)) {
            assertEquals("Cannot read written Byte value",
                         Byte.MIN_VALUE, acc2.paGetObyteValue().byteValue());
        }
    }

    public void testObyteMax() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Byte(Byte.MAX_VALUE));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOBYTE, acc1, acc2)) {
            assertEquals("Cannot read written Byte value",
                         Byte.MAX_VALUE, acc2.paGetObyteValue().byteValue());
        }
    }

    public void testObyteGen() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.OBJBYTE, "obyteValue", new Byte((byte)34));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.OBJBYTE, "obyteValue");
        if (performIO(CNOBYTE, acc1, acc2)) {
            assertEquals("Cannot read written byte value",
                       new Byte((byte)34), acc2.paGetObyteField("obyteValue"));
        }
    }

    // test units for char
    private final static String CNCHAR = "org.objectweb.jorm.pobject.basictype.Char";

    public void testCharSP() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(' ');
        BasicTypeAccessor acc2 = new BasicTypeAccessor('.');
        if (performIO(CNCHAR, acc1, acc2)) {
            assertEquals("Cannot read written char value",
                         ' ', acc2.paGetCharValue());
        }
    }

    public void testCharDOT() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        BasicTypeAccessor acc1 = new BasicTypeAccessor('.');
        BasicTypeAccessor acc2 = new BasicTypeAccessor(' ');
        if (performIO(CNCHAR, acc1, acc2)) {
            assertEquals("Cannot read written char value",
                         '.', acc2.paGetCharValue());
        }
    }
    public void testCharGen() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.CHAR, "charValue", new Character('a'));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.CHAR, "charValue");
        if (performIO(CNCHAR, acc1, acc2)) {
            assertEquals("Cannot read written char value",
                       'a', acc2.paGetCharField("charValue"));
        }
    }

    // test units for ochar
    private final static String CNOCHAR = "org.objectweb.jorm.pobject.basictype.Ochar";

    public void testOcharNull() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Character(' '));
        if (performIO(CNOCHAR, acc1, acc2)) {
            assertEquals("Cannot read written Char value",
                         null, acc2.paGetOcharValue());
        }
    }

    public void testOcharSP() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Character(' '));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOCHAR, acc1, acc2)) {
            assertEquals("Cannot read written Char value",
                         ' ', acc2.paGetOcharValue().charValue());
        }
    }

    public void testOcharDOT() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Character('.'));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOCHAR, acc1, acc2)) {
            assertEquals("Cannot read written Char value",
                         '.', acc2.paGetOcharValue().charValue());
        }
    }
    public void testOcharGen() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.OBJCHAR, "ocharValue", new Character('a'));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.OBJCHAR, "ocharValue");
        if (performIO(CNOCHAR, acc1, acc2)) {
            assertEquals("Cannot read written char value",
                       new Character('a'), acc2.paGetOcharField("ocharValue"));
        }
    }

    // test units for short
    private final static String CNSHORT = "org.objectweb.jorm.pobject.basictype.Short";

    public void testShort0() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        BasicTypeAccessor acc1 = new BasicTypeAccessor((short) 0);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((short) -1);
        if (performIO(CNSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                         (short) 0, acc2.paGetShortValue());
        }
    }

    public void testShortMin() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Short.MIN_VALUE);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((short) 0);
        if (performIO(CNSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                         Short.MIN_VALUE, acc2.paGetShortValue());
        }
    }

    public void testShortMax() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Short.MAX_VALUE);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((short) 0);
        if (performIO(CNSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                         Short.MAX_VALUE, acc2.paGetShortValue());
        }
    }
    public void testShortGen() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.SHORT, "shortValue", new Short((short) 2));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.SHORT, "shortValue");
        if (performIO(CNSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                       (short) 2, acc2.paGetShortField("shortValue"));
        }
    }

    // test units for Short
    private final static String CNOSHORT = "org.objectweb.jorm.pobject.basictype.Oshort";

    public void testOshortNull() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Short((short) 0));
        if (performIO(CNOSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                         null, acc2.paGetOshortValue());
        }
    }

    public void testOshort0() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Short((short) 0));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                         (short) 0, acc2.paGetOshortValue().shortValue());
        }
    }

    public void testOshortMin() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Short(Short.MIN_VALUE));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                         Short.MIN_VALUE, acc2.paGetOshortValue().shortValue());
        }
    }

    public void testOshortMax() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Short(Short.MAX_VALUE));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                         Short.MAX_VALUE, acc2.paGetOshortValue().shortValue());
        }
    }
    public void testOshortGen() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.OBJSHORT, "oshortValue", new Short((short) 2));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.OBJSHORT, "oshortValue");
        if (performIO(CNOSHORT, acc1, acc2)) {
            assertEquals("Cannot read written short value",
                       new Short((short) 2), acc2.paGetOshortField("oshortValue"));
        }
    }

    // test units for int
    private final static String CNINT = "org.objectweb.jorm.pobject.basictype.Int";

    public void testInt0() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(0);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(-1);
        if (performIO(CNINT, acc1, acc2)) {
            assertEquals("Cannot read written int value",
                         0, acc2.paGetIntValue());
        }
    }

    public void testIntMin() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Integer.MIN_VALUE);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(0);
        if (performIO(CNINT, acc1, acc2)) {
            assertEquals("Cannot read written int value",
                         Integer.MIN_VALUE, acc2.paGetIntValue());
        }
    }

    public void testIntMax() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Integer.MAX_VALUE);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(0);
        if (performIO(CNINT, acc1, acc2)) {
            assertEquals("Cannot read written int value",
                         Integer.MAX_VALUE, acc2.paGetIntValue());
        }
    }
    public void testIntGen() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.INT, "intValue", new Integer(45678));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.INT, "intValue");
        if (performIO(CNINT, acc1, acc2)) {
            assertEquals("Cannot read written int value",
                       45678, acc2.paGetIntField("intValue"));
        }
    }

    // test units for Int
    private final static String CNOINT = "org.objectweb.jorm.pobject.basictype.Oint";

    public void testOintNull() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Integer(0));
        if (performIO(CNOINT, acc1, acc2)) {
            assertEquals("Cannot read written int value",
                         null, acc2.paGetOintValue());
        }
    }

    public void testOint0() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Integer(0));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOINT, acc1, acc2)) {
            assertEquals("Cannot read written int value",
                         0, acc2.paGetOintValue().intValue());
        }
    }

    public void testOintMin() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Integer(Integer.MIN_VALUE));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOINT, acc1, acc2)) {
            assertEquals("Cannot read written int value",
                         Integer.MIN_VALUE, acc2.paGetOintValue().intValue());
        }
    }

    public void testOintMax() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Integer(Integer.MAX_VALUE));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOINT, acc1, acc2)) {
            assertEquals("Cannot read written int value",
                         Integer.MAX_VALUE, acc2.paGetOintValue().intValue());
        }
    }
    public void testOintGen() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.OBJINT, "ointValue", new Integer(45678));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.OBJINT, "ointValue");
        if (performIO(CNOINT, acc1, acc2)) {
            assertEquals("Cannot read written oint value",
                       new Integer(45678), acc2.paGetOintField("ointValue"));
        }
    }


    // test units for long
    private final static String CNLONG = "org.objectweb.jorm.pobject.basictype.Long";

    public void testLong0() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        BasicTypeAccessor acc1 = new BasicTypeAccessor((long) 0);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((long) -1);
        if (performIO(CNLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                         (long) 0, acc2.paGetLongValue());
        }
    }

    public void testLongMin() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Long.MIN_VALUE);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((long) 0);
        if (performIO(CNLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                         Long.MIN_VALUE, acc2.paGetLongValue());
        }
    }

    public void testLongMax() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Long.MAX_VALUE);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((long) 0);
        if (performIO(CNLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                         Long.MAX_VALUE, acc2.paGetLongValue());
        }
    }
    public void testLongGen() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.LONG, "longValue", new Long(100000000000L));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.LONG, "longValue");
        if (performIO(CNLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                       100000000000L, acc2.paGetLongField("longValue"));
        }
    }


    // test units for Long
    private final static String CNOLONG = "org.objectweb.jorm.pobject.basictype.Olong";

    public void testOlongNull() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Long(0));
        if (performIO(CNOLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                         null, acc2.paGetOlongValue());
        }
    }

    public void testOlong0() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Long(0));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                         (long) 0, acc2.paGetOlongValue().longValue());
        }
    }

    public void testOlongMin() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Long(Long.MIN_VALUE));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                         Long.MIN_VALUE, acc2.paGetOlongValue().longValue());
        }
    }

    public void testOlongMax() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Long(Long.MAX_VALUE));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                         Long.MAX_VALUE, acc2.paGetOlongValue().longValue());
        }
    }
    public void testOlongGen() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.OBJLONG, "olongValue", new Long(100000000000L));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.OBJLONG, "olongValue");
        if (performIO(CNOLONG, acc1, acc2)) {
            assertEquals("Cannot read written long value",
                       new Long(100000000000L), acc2.paGetOlongField("olongValue"));
        }
    }

    // test units for float
    private final static String CNFLOAT = "org.objectweb.jorm.pobject.basictype.Float";

    public void testFloat0() throws Exception {
        changeLogger(LOGGER_NAME + ".float");
        BasicTypeAccessor acc1 = new BasicTypeAccessor((float) 0);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((float) -1);
        if (performIO(CNFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                         (float) 0, acc2.paGetFloatValue(), (float) 0);
        }
    }

    public void testFloatMin() throws PException {
        changeLogger(LOGGER_NAME + ".float");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(Float.MIN_VALUE + 1);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((float) 0);
        if (performIO(CNFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                    Float.MIN_VALUE + 1, acc2.paGetFloatValue(), (float) 0);
        }
    }

    public void testFloatMax() throws PException {
        changeLogger(LOGGER_NAME + ".float");
        float f = (float) 1.7E20;
        BasicTypeAccessor acc1 = new BasicTypeAccessor(f);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((float) 0);
        if (performIO(CNFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                    f, acc2.paGetFloatValue(), (float) 0);
        }
    }
    public void testFloatGen() throws Exception {
        changeLogger(LOGGER_NAME + ".float");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.FLOAT, "floatValue", new Float(1.3));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.FLOAT, "floatValue");
        if (performIO(CNFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                       new Float(1.3), new Float(acc2.paGetFloatField("floatValue")));
        }
    }

    // test units for Float
    private final static String CNOFLOAT = "org.objectweb.jorm.pobject.basictype.Ofloat";

    public void testOfloatNull() throws Exception {
        changeLogger(LOGGER_NAME + ".float");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Float(0));
        if (performIO(CNOFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                         null, acc2.paGetOfloatValue());
        }
    }

    public void testOfloat0() throws Exception {
        changeLogger(LOGGER_NAME + ".float");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Float(0));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                         (float) 0, acc2.paGetOfloatValue().floatValue(), (float) 0);
        }
    }

    public void testOfloatMin() throws Exception {
        changeLogger(LOGGER_NAME + ".float");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Float(Float.MIN_VALUE+1));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value", Float.MIN_VALUE + 1,
                    acc2.paGetOfloatValue().floatValue(), (float) 0);
        }
    }

    public void testOfloatMax() throws Exception {
        changeLogger(LOGGER_NAME + ".float");
        float f = (float) 1.7E20;
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Float(f));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNOFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value", f,
                    acc2.paGetOfloatValue().floatValue(), (float) 0);
        }
    }

    public void testOfloatGen() throws Exception {
        changeLogger(LOGGER_NAME + ".float");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.OBJFLOAT, "ofloatValue", new Float(1.3));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.OBJFLOAT, "ofloatValue");
        if (performIO(CNOFLOAT, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                       new Float(1.3), acc2.paGetOfloatField("ofloatValue"));
        }
    }

    // test units for double
    private final static String CNDOUBLE = "org.objectweb.jorm.pobject.basictype.Double";

    public void testDouble0() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        BasicTypeAccessor acc1 = new BasicTypeAccessor((double) 0);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((double) -1);
        if (performIO(CNDOUBLE, acc1, acc2)) {
            assertEquals("Cannot read written double value",
                         (double) 0, acc2.paGetDoubleValue(), (double) 0);
        }
    }

    public void testDoubleMin() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        Double d = new Double(1.79769313486E-300);
        BasicTypeAccessor acc1 = new BasicTypeAccessor(d.doubleValue());
        BasicTypeAccessor acc2 = new BasicTypeAccessor((double) 0);
        if (performIO(CNDOUBLE, acc1, acc2)) {
            assertEquals("Cannot read written double value",
                         d.doubleValue(), acc2.paGetDoubleValue(), (double) 0);
        }
    }

    public void testDoubleMax() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(1.79769313486E300);
        BasicTypeAccessor acc2 = new BasicTypeAccessor((double) 0);
        if (performIO(CNDOUBLE, acc1, acc2)) {
            assertTrue("Cannot read written double value expected: " +
                    "1.79769313486E300, found: " + acc2.paGetDoubleValue(),
                         1.79769313486E300 == acc2.paGetDoubleValue());
        }
    }
    public void testDoubleGen() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.DOUBLE, "doubleValue", new Double(1.3));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.DOUBLE, "doubleValue");
        if (performIO(CNDOUBLE, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                       new Double(1.3), new Double(acc2.paGetDoubleField("doubleValue")));
        }
    }

    // test units for Double
    private final static String CNODOUBLE = "org.objectweb.jorm.pobject.basictype.Odouble";

    public void testOdoubleNull() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(new Double(0));
        if (performIO(CNODOUBLE, acc1, acc2)) {
            assertEquals("Cannot read written double value",
                         null, acc2.paGetOdoubleValue());
        }
    }

    public void testOdouble0() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(new Double(0));
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNODOUBLE, acc1, acc2)) {
            assertEquals("Cannot read written double value [0]",
                         (double) 0, acc2.paGetOdoubleValue().doubleValue(), (double) 0);
        }
    }

    public void testOdoubleMin() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        Double d = new Double(1.79769313486E-300);
        BasicTypeAccessor acc1 = new BasicTypeAccessor(d);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNODOUBLE, acc1, acc2)) {
            assertEquals("Cannot read written double value",
                         d.doubleValue(), acc2.paGetOdoubleValue().doubleValue(), (double) 0);
        }
    }

    public void testOdoubleMax() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        Double d = new Double(1.79769313486E300);
        BasicTypeAccessor acc1 = new BasicTypeAccessor(d);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNODOUBLE, acc1, acc2)) {
            assertEquals("Cannot read written double value",
                         d, acc2.paGetOdoubleValue());
        }
    }
    public void testOdoubleGen() throws Exception {
        changeLogger(LOGGER_NAME + ".double");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.OBJDOUBLE, "odoubleValue", new Double(1.3));
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.OBJDOUBLE, "odoubleValue");
        if (performIO(CNODOUBLE, acc1, acc2)) {
            assertEquals("Cannot read written float value",
                       new Double(1.3), acc2.paGetOdoubleField("odoubleValue"));
        }
    }

    // test units for String
    private final static String CNSTRING = "org.objectweb.jorm.pobject.basictype.String";

    public void testStringNull() throws Exception {
        changeLogger(LOGGER_NAME + ".string");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor("toto");
        if (performIO(CNSTRING, acc1, acc2)) {
            assertEquals("Cannot read written string value",
                         null, acc2.paGetStringValue());
        }
    }

    public void testStringEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".string");
        BasicTypeAccessor acc1 = new BasicTypeAccessor("");
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNSTRING, acc1, acc2)) {
            assertEquals("Cannot read written string value",
                         "", acc2.paGetStringValue());
        }
    }

    public void testStringazerty() throws Exception {
        changeLogger(LOGGER_NAME + ".string");
        BasicTypeAccessor acc1 = new BasicTypeAccessor("azerty");
        BasicTypeAccessor acc2 = new BasicTypeAccessor("toto");
        if (performIO(CNSTRING, acc1, acc2)) {
            assertEquals("Cannot read written string value",
                         "azerty", acc2.paGetStringValue());
        }
    }
    public void testStringGen() throws Exception {
        changeLogger(LOGGER_NAME + ".string");
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.STRING, "stringValue", "toto");
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.STRING, "stringValue");
        if (performIO(CNSTRING, acc1, acc2)) {
            assertEquals("Cannot read written string value",
                       "toto", acc2.paGetStringField("stringValue"));
        }
    }

    // test units for Date
    private final static String CNDATE = "org.objectweb.jorm.pobject.basictype.Date";

    public void testDateNull() throws Exception {
        testDateValue(null);
    }

    public void testDateNanos() throws Exception {
        testDateValue(new Date(1));
    }

    public void testDateNow() throws Exception {
        testDateValue(getTime());
    }

    private void testDateValue(Date d) throws Exception {
        changeLogger(LOGGER_NAME + ".date");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(d);
        BasicTypeAccessor acc2 =
            new BasicTypeAccessor(d == null ? "toto" : null);
        if (performIO(CNDATE, acc1, acc2)) {
            assertEquals("Cannot read written date value",
                         d, acc2.paGetDateValue());
            assertEquals("Cannot read written date value",
                         acc2.paGetDateValue(), d);
        }
    }
    public void testDateSql() throws Exception {
        changeLogger(LOGGER_NAME + ".date");
        java.sql.Time d = new java.sql.Time(getTime().getTime());
        BasicTypeAccessor acc1 = new BasicTypeAccessor(d);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNDATE, acc1, acc2)) {
            assertNotNull("Null date found, expected: " + d, acc2.paGetDateValue());
            assertEquals("Cannot read written date value",
                         d.getTime(), acc2.paGetDateValue().getTime());
        }
    }
    public void testDateGen() throws Exception {
        changeLogger(LOGGER_NAME + ".date");
        Date d = getTime();
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.DATE, "dateValue", d);
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.DATE, "dateValue");
        if (performIO(CNDATE, acc1, acc2)) {
            assertEquals("Cannot read written date value",
                       d.getTime(), acc2.paGetDateField("dateValue").getTime());
        }
    }


    // test units for Date
    private final static String CNSERIALIZED = "org.objectweb.jorm.pobject.basictype.Serialized";

    public void testSerializedNull() throws Exception {
        changeLogger(LOGGER_NAME + ".serialized");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor("toto");
        if (performIO(CNSERIALIZED, acc1, acc2)) {
            assertEquals("Cannot read written serialized value",
                         null, acc2.paGetSerializedValue());
        }
    }

	public void testSerializedAzerty() throws Exception {
		changeLogger(LOGGER_NAME + ".serialized");
		Object o = "azertyuiop";
		BasicTypeAccessor acc1 = new BasicTypeAccessor(o);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNSERIALIZED, acc1, acc2)) {
			assertEquals("Cannot read written serialized value",
						 o, acc2.paGetSerializedValue());
		}
	}

	public void testSerialized10000KB() throws Exception {
		changeLogger(LOGGER_NAME + ".serialized");
		StringBuffer sb = new StringBuffer(10004);
		String s10 = "azertyuiop";
		for (int i = 0; i < 1000; i++) {
			sb.append(s10);
		}
		Object o = sb.toString();
		BasicTypeAccessor acc1 = new BasicTypeAccessor(o);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNSERIALIZED, acc1, acc2)) {
			assertEquals("Cannot read written serialized value",
						 o, acc2.paGetSerializedValue());
		}
	}

    public void testSerializedGen() throws Exception {
        changeLogger(LOGGER_NAME + ".serialized");
        Object o = "azertyuiop";
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.SERIALIZED, "serializedValue", o);
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.SERIALIZED, "serializedValue");
        if (performIO(CNSERIALIZED, acc1, acc2)) {
            assertEquals("Cannot read written serialized value",
                       o, acc2.paGetSerializedField("serializedValue"));
        }
    }

    // test units for Date
    private final static String CNBYTEARRAY = "org.objectweb.jorm.pobject.basictype.Bytearray";

    public void testByteArrayNull() throws Exception {
        changeLogger(LOGGER_NAME + ".bytearray");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor("toto");
        if (performIO(CNBYTEARRAY, acc1, acc2)) {
            assertEquals("Cannot read written byte[] value",
                         null, acc2.paGetSerializedValue());
        }
    }

    public void testByteArrayazerty() throws Exception {
        changeLogger(LOGGER_NAME + ".bytearray");
        byte[] o = "azertyuiop".getBytes();
        BasicTypeAccessor acc1 = new BasicTypeAccessor(o);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNBYTEARRAY, acc1, acc2)) {
            byte[] res = acc2.paGetBytearrayValue();
            assertNotNull("Read written byte[] is null", res);
            assertEquals("Bad size", o.length, res.length);
            for(int i=0; i<o.length; i++) {
                assertEquals("Bad element " + i, o[i], res[i]);
            }
        }
    }

    public void testByteArrayEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".bytearray");
        byte[] o = new byte[0];
        BasicTypeAccessor acc1 = new BasicTypeAccessor(o);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNBYTEARRAY, acc1, acc2)) {
            byte[] res = acc2.paGetBytearrayValue();
            assertNotNull("Read written byte[] is null", res);
            assertEquals("Bad size", 0, res.length);
        }
    }
    public void testByteArrayGen() throws Exception {
        changeLogger(LOGGER_NAME + ".bytearray");
        byte[] o = "azertyuiop".getBytes();
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.BYTEARRAY, "bytearrayValue", o);
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.BYTEARRAY, "bytearrayValue");
        if (performIO(CNBYTEARRAY, acc1, acc2)) {
            byte[] res = acc2.paGetByteArrayField("bytearrayValue");
            assertNotNull("Read written byte[] is null", res);
            assertEquals("Bad size", o.length, res.length);
            for(int i=0; i<o.length; i++) {
                assertEquals("Bad element " + i, o[i], res[i]);
            }
        }
    }

    // test units for Date
    private final static String CNCHARARRAY = "org.objectweb.jorm.pobject.basictype.Chararray";

    public void testCharArrayNull() throws Exception {
        changeLogger(LOGGER_NAME + ".chararray");
        BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
        BasicTypeAccessor acc2 = new BasicTypeAccessor("toto");
        if (performIO(CNCHARARRAY, acc1, acc2)) {
            assertEquals("Cannot read written char[] value",
                         null, acc2.paGetSerializedValue());
        }
    }

    public void testCharArrayazerty() throws Exception {
        changeLogger(LOGGER_NAME + ".chararray");
        char[] o = "azertyuiop".toCharArray();
        BasicTypeAccessor acc1 = new BasicTypeAccessor(o);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNCHARARRAY, acc1, acc2)) {
            char[] res = acc2.paGetChararrayValue();
            assertNotNull("Read written char[] is null", res);
            assertEquals("Bad size", o.length, res.length);
            for(int i=0; i<o.length; i++) {
                assertEquals("Bad element " + i, o[i], res[i]);
            }
        }
    }

    public void testCharArrayEmpty() throws Exception {
        changeLogger(LOGGER_NAME + ".chararray");
        char[] o = new char[0];
        BasicTypeAccessor acc1 = new BasicTypeAccessor(o);
        BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
        if (performIO(CNCHARARRAY, acc1, acc2)) {
            char[] res = acc2.paGetChararrayValue();
            assertNotNull("Read written char[] is null", res);
            assertEquals("Bad size", 0, res.length);
        }
    }
    public void testCharArrayGen() throws Exception {
        changeLogger(LOGGER_NAME + ".chararray");
        char[] o = "azertyuiop".toCharArray();
        GenericAccessor acc1 = new GenericAccessor(
                PTypeSpace.CHARARRAY, "chararrayValue", o);
        GenericAccessor acc2 = new GenericAccessor(
                PTypeSpace.CHARARRAY, "chararrayValue");
        if (performIO(CNCHARARRAY, acc1, acc2)) {
            char[] res = acc2.paGetCharArrayField("chararrayValue");
            assertNotNull("Read written char[] is null", res);
            assertEquals("Bad size", o.length, res.length);
            for(int i=0; i<o.length; i++) {
                assertEquals("Bad element " + i, o[i], res[i]);
            }
        }
    }

	// test units for BigDecimal
	private final static String CNBIGDECIMAL = "org.objectweb.jorm.pobject.basictype.BigDecimal";

	public void testBigDecimalNull() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(new BigDecimal(0));
		if (performIO(CNBIGDECIMAL, acc1, acc2)) {
			assertEquals("Cannot read written bigdecimal value",
						 null, acc2.paGetBigDecimalValue());
		}
	}

	public void testBigDecimal0() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		BigDecimal bd = new BigDecimal("0.0000");
		BigDecimal bd0 = new BigDecimal("0");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGDECIMAL, acc1, acc2)) {
			assertTrue("Cannot read written bigdecimal value expected:<0> but was:<"
					   + acc2.paGetBigDecimalValue() + ">",
					   acc2.paGetBigDecimalValue().equals(bd)
					   || acc2.paGetBigDecimalValue().equals(bd0));
		}
	}

	public void testBigDecimalVP4() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		BigDecimal bd = new BigDecimal("6747645764.8796");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGDECIMAL, acc1, acc2)) {
			assertEquals("Cannot read written bigdecimal value",
						 bd, acc2.paGetBigDecimalValue());
		}
	}

	public void testBigDecimalVN4() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		BigDecimal bd = new BigDecimal("-6747645764.8796");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGDECIMAL, acc1, acc2)) {
			assertEquals("Cannot read written bigdecimal value",
						 bd, acc2.paGetBigDecimalValue());
		}
	}

	public void testBigDecimalVP() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		BigDecimal bd = new BigDecimal("6747645764.8796764453");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGDECIMAL, acc1, acc2)) {
			assertEquals("Cannot read written bigdecimal value",
						 bd, acc2.paGetBigDecimalValue());
		}
	}

	public void testBigDecimalVN() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		BigDecimal bd = new BigDecimal("-6747645764.8796764453");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGDECIMAL, acc1, acc2)) {
			assertEquals("Cannot read written bigdecimal value",
						 bd, acc2.paGetBigDecimalValue());
		}
	}

	public void testBigDecimalGen() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		GenericAccessor acc1 = new GenericAccessor(
				PTypeSpace.BIGDECIMAL, "bigDecimalValue", new BigDecimal("1765765564564.756757657653"));
		GenericAccessor acc2 = new GenericAccessor(
				PTypeSpace.BIGDECIMAL, "bigDecimalValue");
		if (performIO(CNBIGDECIMAL, acc1, acc2)) {
			assertEquals("Cannot read written bigdecimal value",
					   new BigDecimal("1765765564564.756757657653"), acc2.paGetBigDecimalField("bigDecimalValue"));
		}
	}

	// test units for BigInteger
	private final static String CNBIGINTEGER = "org.objectweb.jorm.pobject.basictype.BigInteger";

	public void testBigIntegerNull() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(null);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(new BigInteger("0"));
		if (performIO(CNBIGINTEGER, acc1, acc2)) {
			assertEquals("Cannot read written biginteger value",
						 null, acc2.paGetBigIntegerValue());
		}
	}

	public void testBigInteger0() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		BigInteger bd = new BigInteger("0");
		BigInteger bd0 = new BigInteger("0");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGINTEGER, acc1, acc2)) {
			assertTrue("Cannot read written biginteger value expected:<0> but was:<"
					   + acc2.paGetBigIntegerValue() + ">",
					   acc2.paGetBigIntegerValue().equals(bd)
					   || acc2.paGetBigIntegerValue().equals(bd0));
		}
	}

	public void testBigIntegerVPS() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		BigInteger bd = new BigInteger("6747645764");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGINTEGER, acc1, acc2)) {
			assertEquals("Cannot read written biginteger value",
						 bd, acc2.paGetBigIntegerValue());
		}
	}

	public void testBigIntegerVNS() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		BigInteger bd = new BigInteger("-6747645764");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGINTEGER, acc1, acc2)) {
			assertEquals("Cannot read written biginteger value",
						 bd, acc2.paGetBigIntegerValue());
		}
	}

	public void testBigIntegerVPL() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		BigInteger bd = new BigInteger("67476457648796764453");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGINTEGER, acc1, acc2)) {
			assertEquals("Cannot read written biginteger value",
						 bd, acc2.paGetBigIntegerValue());
		}
	}

	public void testBigIntegerVNL() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		BigInteger bd = new BigInteger("-67476457648796764453");
		BasicTypeAccessor acc1 = new BasicTypeAccessor(bd);
		BasicTypeAccessor acc2 = new BasicTypeAccessor(null);
		if (performIO(CNBIGINTEGER, acc1, acc2)) {
			assertEquals("Cannot read written biginteger value",
						 bd, acc2.paGetBigIntegerValue());
		}
	}

	public void testBigIntegerGen() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		GenericAccessor acc1 = new GenericAccessor(
				PTypeSpace.BIGINTEGER, "bigIntegerValue", new BigInteger("1765765564564756757657653"));
		GenericAccessor acc2 = new GenericAccessor(
				PTypeSpace.BIGINTEGER, "bigIntegerValue");
		if (performIO(CNBIGINTEGER, acc1, acc2)) {
			assertEquals("Cannot read written biginteger value",
					   new BigInteger("1765765564564756757657653"), acc2.paGetBigIntegerField("bigIntegerValue"));
		}
	}
}
