/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package org.objectweb.jorm.runtime.colocated;

import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.jorm.runtime.PolyIdPNG;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdBinder;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestOneMany2T extends TestRuntimeHelper {

    private final static String LOGGER_NAME
        = "test.org.objectweb.jorm.colocated.omemany2t";

    private final static String GenClassRef_SN_C
        = "org.objectweb.jorm.pobject.colocated.GenClassRef_SN_C";
    private final static String GenClassRef_CN_C
        = "org.objectweb.jorm.pobject.colocated.GenClassRef_CN_C";

    private final static String StringReferencedClass_C
        = "org.objectweb.jorm.pobject.colocated.StringReferencedClass_C";
    private final static String PolyIdReferencedClass_C
        = "org.objectweb.jorm.pobject.colocated.PolyIdReferencedClass_C";

    Map cn2binder;

    public TestOneMany2T(String s) throws Exception {
        super(s);
        cn2binder = new HashMap(1);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        PBinder binder = (PBinder) cn2binder.get(className);
        if (binder != null) {
            return binder;
        }
        try {
            if (className.startsWith(GenClassRef_SN_C)) {
                binder = new BasidBinder( PNamingContext.CTLONG);
                binder.setNullPName(new Long(-1));
                ((BasidBinder) binder).setCodingType(PNameCoder.CTLONG);
            } else if (className.startsWith(GenClassRef_CN_C)
                || className.startsWith(PolyIdReferencedClass_C)) {
                binder = new PolymorphIdBinder();
                binder.setNullPName(new PolyIdPNG());
            } else if (className.startsWith(StringReferencedClass_C)) {
                binder = new BasidBinder( PNamingContext.CTSTRING);
                binder.setNullPName(null);
                ((BasidBinder) binder).setCodingType(PNameCoder.CTSTRING);
            }
        } catch (PException e) {
            Exception current = e;
            while (current instanceof PException
                    && ((PException) current).getNestedException() != null) {
                current = ((PException) current).getNestedException();
            }
            throw current;
        }
        if (binder == null) {
            throw new PException("Unmanaged class " + className);
        }
        logger.log(BasicLevel.DEBUG,
            "className=" + className + " / binder=" + binder);
        cn2binder.put(className, binder);
        return binder;
    }

    private void genclassRefTestOrder1(PBinding gcrPb, PBinding gcPb, List rcPbs) {

        //Make persistent the referencedClass instances
        List rcAccws = new ArrayList();
        List rcPns = new ArrayList();
        for(int i=0; i<rcPbs.size(); i++) {
            rcPns.add(((PBinding) rcPbs.get(i)).getPName());
            ReferencedClass_C rcAccw = new ReferencedClass_C();
            rcAccws.add(rcAccw);
            rcAccw.f1 = 10;
            ReferencedClass_C rcAccr = new ReferencedClass_C();
            logger.log(BasicLevel.DEBUG, "create and read referenced class " + i);
            writeRead((PBinding) rcPbs.get(i), rcAccw, rcAccr);
            assertEquals("bad referenced class values", rcAccw, rcAccr);
        }

        //Make persistent the genclass
        GenClass_C gcAccw = new GenClass_C(rcPns);
        GenClass_C gcAccr = new GenClass_C(rcPns.size());
        logger.log(BasicLevel.DEBUG, "create and read genclas");
        writeRead(gcPb, gcAccw, gcAccr);
        assertEquals("bad gen class values", gcAccw, gcAccr);

        //Make persistent the genclassref
        GenClassRef_C gcrAccw = new GenClassRef_C(gcPb.getPName());
        GenClassRef_C gcrAccr = new GenClassRef_C();
        logger.log(BasicLevel.DEBUG, "create and read genclass");
        writeRead(gcrPb, gcrAccw, gcrAccr);
        assertEquals("bad gen class values", gcrAccw, gcrAccr);

        for(int i=0; i<rcPbs.size(); i++) {
            logger.log(BasicLevel.DEBUG, "remove referenced class");
            unexport((PBinding) rcPbs.get(i), (PAccessor) rcAccws.get(i));
        }
        logger.log(BasicLevel.DEBUG, "remove gen class");
        unexport(gcPb, gcAccw);
        logger.log(BasicLevel.DEBUG, "remove gen class ref");
        unexport(gcrPb, gcrAccw);
    }

    private void genclassRefTestOrder2(PBinding gcrPb, PBinding gcPb, List rcPbs) {

        //Make persistent the referencedClass instances
        List rcAccws = new ArrayList();
        List rcPns = new ArrayList();
        for(int i=0; i<rcPbs.size(); i++) {
            rcPns.add(((PBinding) rcPbs.get(i)).getPName());
            ReferencedClass_C rcAccw = new ReferencedClass_C();
            rcAccws.add(rcAccw);
            rcAccw.f1 = 10;
        }

        //Make persistent the genclass
        GenClass_C gcAccw = new GenClass_C(rcPns);
        GenClass_C gcAccr = new GenClass_C(rcPns.size());
        logger.log(BasicLevel.DEBUG, "create and read genclas");
        writeRead(gcPb, gcAccw, gcAccr);
        assertEquals("bad gen class values", gcAccw, gcAccr);

        for(int i=0; i<rcPbs.size(); i++) {
            ReferencedClass_C rcAccw = (ReferencedClass_C) rcAccws.get(i);
            ReferencedClass_C rcAccr = new ReferencedClass_C();
            logger.log(BasicLevel.DEBUG, "create and read referenced class " + i);
            writeRead((PBinding) rcPbs.get(i), rcAccw, rcAccr);
            assertEquals("bad referenced class values", rcAccw, rcAccr);
        }

        //Make persistent the genclassref
        GenClassRef_C gcrAccw = new GenClassRef_C(gcPb.getPName());
        GenClassRef_C gcrAccr = new GenClassRef_C();
        logger.log(BasicLevel.DEBUG, "create and read genclass");
        writeRead(gcrPb, gcrAccw, gcrAccr);
        assertEquals("bad gen class values", gcrAccw, gcrAccr);

        for(int i=0; i<rcPbs.size(); i++) {
            logger.log(BasicLevel.DEBUG, "remove referenced class");
            unexport((PBinding) rcPbs.get(i), (PAccessor) rcAccws.get(i));
        }
        logger.log(BasicLevel.DEBUG, "remove gen class");
        unexport(gcPb, gcAccw);
        logger.log(BasicLevel.DEBUG, "remove gen class ref");
        unexport(gcrPb, gcrAccw);
    }

    public void testGenClassRefSingle1() {
        changeLogger(LOGGER_NAME + ".single1");
        logger.log(BasicLevel.DEBUG, "testGenClassRefSingle2 begin");
        List rcPbs = new ArrayList();
        rcPbs.add(export(StringReferencedClass_C, "azerty"));
        rcPbs.add(export(StringReferencedClass_C, "qsdfgh"));
        genclassRefTestOrder1(
                export(GenClassRef_SN_C, new Long(45)),
                export(GenClassRef_SN_C + "/c", new Long(45)),
                rcPbs);
        logger.log(BasicLevel.DEBUG, "testGenClassRefSingle1 end");
    }

    public void _testGenClassRefSingle2() {
        changeLogger(LOGGER_NAME + ".single2");
        logger.log(BasicLevel.DEBUG, "testGenClassRefSingle2 begin");
        List rcPbs = new ArrayList();
        rcPbs.add(export(StringReferencedClass_C, "azerty"));
        rcPbs.add(export(StringReferencedClass_C, "qsdfgh"));
        genclassRefTestOrder2(
                export(GenClassRef_SN_C, new Long(45)),
                export(GenClassRef_SN_C + "/c", new Long(45)),
                rcPbs);
        logger.log(BasicLevel.DEBUG, "testGenClassRefSingle2 end");
    }

    public void testGenClassRefComposite1() {
        changeLogger(LOGGER_NAME + ".composite1");
        logger.log(BasicLevel.DEBUG, "testGenClassRefComposite1 begin");
        List rcPbs = new ArrayList();
        rcPbs.add(export(PolyIdReferencedClass_C, new PolyIdPNG(78, 90)));
        rcPbs.add(export(PolyIdReferencedClass_C, new PolyIdPNG(13, 24)));
        genclassRefTestOrder1(
            export(GenClassRef_CN_C, new PolyIdPNG(12, 34)),
            export(GenClassRef_CN_C + "/c", new PolyIdPNG(12, 34)),
            rcPbs);
        logger.log(BasicLevel.DEBUG, "testGenClassRefComposite1 end");
    }

    public void _testGenClassRefComposite2() {
        changeLogger(LOGGER_NAME + ".composite1");
        logger.log(BasicLevel.DEBUG, "testGenClassRefComposite1 begin");
        List rcPbs = new ArrayList();
        rcPbs.add(export(PolyIdReferencedClass_C, new PolyIdPNG(78, 90)));
        rcPbs.add(export(PolyIdReferencedClass_C, new PolyIdPNG(13, 24)));
        genclassRefTestOrder2(
            export(GenClassRef_CN_C, new PolyIdPNG(12, 34)),
            export(GenClassRef_CN_C + "/c", new PolyIdPNG(12, 34)),
            rcPbs);
        logger.log(BasicLevel.DEBUG, "testGenClassRefComposite1 end");
    }

}
