/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.gcindex;

import org.objectweb.jorm.api.PGenClassAccessor;
import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.naming.api.PName;

import java.util.Iterator;
import java.util.Date;
import java.util.Map;
import java.util.HashMap;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * @author  P. Dechamboux
 */
public class GcIndexAccessor implements PGenClassAccessor {
    private String[] stringarray_value;
    private Map map_value;
    private int indexTypeCode, nbElem;

    public GcIndexAccessor(int tc, int nbelem) throws PException {
        indexTypeCode = tc;
        nbElem = nbelem;
        switch (indexTypeCode) {
        case PType.TYPECODE_DATE:
        case PType.TYPECODE_STRING:
            map_value = new HashMap();
            break;
        case PType.TYPECODE_BYTE:
        case PType.TYPECODE_CHAR:
        case PType.TYPECODE_SHORT:
        case PType.TYPECODE_INT:
        case PType.TYPECODE_LONG:
        case PType.TYPECODE_OBJBYTE:
        case PType.TYPECODE_OBJCHAR:
        case PType.TYPECODE_OBJSHORT:
        case PType.TYPECODE_OBJINT:
        case PType.TYPECODE_OBJLONG:
            stringarray_value = new String[nbelem];
            break;
        default:
            throw new PException("Wrong index type.");
        }
    }

    public void setElem(int index, String o) {
        stringarray_value[index] = o;
    }

    public void setElem(String index, String o) {
        map_value.put(index, o);
    }

    public void setElem(Date index, String o) {
        map_value.put(new Long(index.getTime()), o);
    }

    public boolean equals(Object other) {
        switch (indexTypeCode) {
        case PType.TYPECODE_DATE:
        case PType.TYPECODE_STRING:
            return map_value.equals(((GcIndexAccessor) other).map_value);
        default:
            if (((GcIndexAccessor) other).stringarray_value.length != stringarray_value.length) {
                return false;
            }
            for (int i = 0; i < ((GcIndexAccessor) other).stringarray_value.length; i++) {
                if (stringarray_value[i] == ((GcIndexAccessor) other).stringarray_value[i]) {
                    continue;
                }
                if ((((GcIndexAccessor) other).stringarray_value[i] == null)
                    || (stringarray_value[i] == null)) {
                    return false;
                }
                if (!((GcIndexAccessor) other).stringarray_value[i].equals(stringarray_value[i])) {
                    return false;
                }
            }
            return true;
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PGenClassAccessor INTERFACE

    public void paAdd(PIndexedElem elem, Object conn) throws PException {
        switch (indexTypeCode) {
        case PType.TYPECODE_DATE:
            map_value.put(new Long(elem.pieGetDateIndexField("index").getTime()), elem.pieGetStringElem());
            break;
        case PType.TYPECODE_STRING:
            map_value.put(elem.pieGetStringIndexField("index"), elem.pieGetStringElem());
            break;
        case PType.TYPECODE_BYTE:
            stringarray_value[elem.pieGetByteIndexField("index")] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_CHAR:
            stringarray_value[elem.pieGetCharIndexField("index") - ' ' - 1] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_SHORT:
            stringarray_value[elem.pieGetShortIndexField("index")] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_INT:
            stringarray_value[elem.pieGetIntIndexField("index")] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_LONG:
            stringarray_value[(int) elem.pieGetLongIndexField("index")] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_OBJBYTE:
            stringarray_value[elem.pieGetObyteIndexField("index").intValue()] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_OBJCHAR:
            stringarray_value[elem.pieGetOcharIndexField("index").charValue() - ' ' - 1] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_OBJSHORT:
            stringarray_value[elem.pieGetOshortIndexField("index").intValue()] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_OBJINT:
            stringarray_value[elem.pieGetOintIndexField("index").intValue()] = elem.pieGetStringElem();
            break;
        case PType.TYPECODE_OBJLONG:
            stringarray_value[elem.pieGetOlongIndexField("index").intValue()] = elem.pieGetStringElem();
            break;
        default:
        }
    }

    public boolean paDeltaSupported() {
        return false;
    }

    public int paGetNbElem() {
        return nbElem;
    }

    public Iterator paIterator() {
        return new GcIterator(this);
    }

    public void paSetNbElem(int nbelem) {
    }

    // IMPLEMENTATION OF METHODS FROM THE PAccessor INTERFACE

    public Object getMemoryInstance() {
        return this;
    }

    // IMPLEMENTATION OF METHODS FROM THE PIndexedElemFactory INTERFACE

    public PIndexedElem createPIndexedElem() throws PException {
        return new IStringIndexedElem(indexTypeCode);
    }

    // THE ITERATOR FOR paIterator
    class GcIterator implements Iterator {
        private int nextToRead = 0;
        private GcIndexAccessor theGC;
        private Iterator mapIter;

        GcIterator(GcIndexAccessor gc) {
            theGC = gc;
            switch (theGC.indexTypeCode) {
            case PType.TYPECODE_DATE:
            case PType.TYPECODE_STRING:
                mapIter = theGC.map_value.keySet().iterator();
                break;
            default:
            }
        }

        public boolean hasNext() {
            switch (theGC.indexTypeCode) {
            case PType.TYPECODE_DATE:
            case PType.TYPECODE_STRING:
                return mapIter.hasNext();
            default:
                return nextToRead < theGC.nbElem;
            }
        }

        public Object next() {
            if (!hasNext()) {
                return null;
            }
            try {
                PIndexedElem pie = theGC.createPIndexedElem();
                switch (theGC.indexTypeCode) {
                case PType.TYPECODE_DATE:
                    Long ld = (Long) mapIter.next();
                    pie.pieSetDateIndexField("index", new Date(ld.longValue()));
                    pie.pieSetStringElem((String) theGC.map_value.get(ld));
                    break;
                case PType.TYPECODE_STRING:
                    pie.pieSetStringIndexField("index", (String) mapIter.next());
                    pie.pieSetStringElem((String) theGC.map_value.get(pie.pieGetStringIndexField("index")));
                    break;
                case PType.TYPECODE_BYTE:
                    pie.pieSetByteIndexField("index", (byte) nextToRead);
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_CHAR:
                    pie.pieSetCharIndexField("index", (char) (nextToRead + ' ' + 1));
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_SHORT:
                    pie.pieSetShortIndexField("index", (short) nextToRead);
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_INT:
                    pie.pieSetIntIndexField("index", nextToRead);
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_LONG:
                    pie.pieSetLongIndexField("index", nextToRead);
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJBYTE:
                    pie.pieSetObyteIndexField("index", new Byte((byte) nextToRead));
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJCHAR:
                    pie.pieSetOcharIndexField("index", new Character((char) (nextToRead + ' ' + 1)));
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJSHORT:
                    pie.pieSetOshortIndexField("index", new Short((short) nextToRead));
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJINT:
                    pie.pieSetOintIndexField("index", new Integer(nextToRead));
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                case PType.TYPECODE_OBJLONG:
                    pie.pieSetOlongIndexField("index", new Long(nextToRead));
                    pie.pieSetStringElem(theGC.stringarray_value[nextToRead]);
                    break;
                }
                nextToRead++;
                return pie;
            } catch (PException e) {
                e.printStackTrace(System.out);  //To change body of catch statement use Options | File Templates.
            }
            return null;
        }

        public void remove() {
        }
    }
}

class IStringIndexedElem implements PIndexedElem {
    private byte byte_index;
    private char char_index;
    private short short_index;
    private int int_index;
    private long long_index;
    private Object object_index;
    private int typeCode;
    private String value;

    public IStringIndexedElem(int typecode) throws PException {
        typeCode = typecode;
    }

    public byte getElemStatus() {
        return 0;
    }

    public boolean pieGetBooleanElem() throws PException {
        throw new PException("Wrong type!!");
    }

    public Boolean pieGetObooleanElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public byte pieGetByteElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Byte pieGetObyteElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public byte pieGetByteIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_BYTE) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return byte_index;
        }
        throw new PException("Unknown index name.");
    }

    public Byte pieGetObyteIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_OBJBYTE) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return (Byte) object_index;
        }
        throw new PException("Unknown index name.");
    }

    public char pieGetCharElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Character pieGetOcharElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public char pieGetCharIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_CHAR) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return char_index;
        }
        throw new PException("Unknown index name.");
    }

    public Character pieGetOcharIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_OBJCHAR) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return (Character) object_index;
        }
        throw new PException("Unknown index name.");
    }

    public short pieGetShortElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Short pieGetOshortElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public short pieGetShortIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_SHORT) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return short_index;
        }
        throw new PException("Unknown index name.");
    }

    public Short pieGetOshortIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_OBJSHORT) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return (Short) object_index;
        }
        throw new PException("Unknown index name.");
    }

    public int pieGetIntElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Integer pieGetOintElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public int pieGetIntIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_INT) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return int_index;
        }
        throw new PException("Unknown index name.");
    }

    public Integer pieGetOintIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_OBJINT) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return (Integer) object_index;
        }
        throw new PException("Unknown index name.");
    }

    public long pieGetLongElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Long pieGetOlongElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public long pieGetLongIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_LONG) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return long_index;
        }
        throw new PException("Unknown index name.");
    }

    public Long pieGetOlongIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_OBJLONG) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return (Long) object_index;
        }
        throw new PException("Unknown index name.");
    }

    public float pieGetFloatElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Float pieGetOfloatElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public double pieGetDoubleElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Double pieGetOdoubleElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public String pieGetStringElem() throws PException {
        return value;
    }

    public String pieGetStringIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_STRING) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return (String) object_index;
        }
        throw new PException("Unknown index name.");
    }

    public Date pieGetDateElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Date pieGetDateIndexField(String fn) throws PException {
        if (typeCode != PType.TYPECODE_DATE) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            return (Date) object_index;
        }
        throw new PException("Unknown index name.");
    }

    public char[] pieGetCharArrayElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public byte[] pieGetByteArrayElem() throws PException {
            throw new PException("Wrong type!!");
    }

    public Serializable pieGetSerializedElem() throws PException {
            throw new PException("Wrong type!!");
    }

	public BigDecimal pieGetBigDecimalElem() throws PException {
			throw new PException("Wrong type!!");
	}

	public BigInteger pieGetBigIntegerElem() throws PException {
			throw new PException("Wrong type!!");
	}

    public PName pieGetRefElem() throws PException {
        throw new PException("Wrong type!!");
    }

    public void pieSetBooleanElem(boolean value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetObooleanElem(Boolean value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetByteElem(byte value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetObyteElem(Byte value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetByteIndexField(String fn, byte value) throws PException {
        if (typeCode != PType.TYPECODE_BYTE) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            byte_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetObyteIndexField(String fn, Byte value) throws PException {
        if (typeCode != PType.TYPECODE_OBJBYTE) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            object_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetCharElem(char value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetOcharElem(Character value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetCharIndexField(String fn, char value) throws PException {
        if (typeCode != PType.TYPECODE_CHAR) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            char_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetOcharIndexField(String fn, Character value) throws PException {
        if (typeCode != PType.TYPECODE_OBJCHAR) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            object_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetShortElem(short value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetOshortElem(Short value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetShortIndexField(String fn, short value) throws PException {
        if (typeCode != PType.TYPECODE_SHORT) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            short_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetOshortIndexField(String fn, Short value) throws PException {
        if (typeCode != PType.TYPECODE_OBJSHORT) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            object_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetIntElem(int value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetOintElem(Integer value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetIntIndexField(String fn, int value) throws PException {
        if (typeCode != PType.TYPECODE_INT) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            int_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetOintIndexField(String fn, Integer value) throws PException {
        if (typeCode != PType.TYPECODE_OBJINT) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            object_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetLongElem(long value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetOlongElem(Long value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetLongIndexField(String fn, long value) throws PException {
        if (typeCode != PType.TYPECODE_LONG) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            long_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetOlongIndexField(String fn, Long value) throws PException {
        if (typeCode != PType.TYPECODE_OBJLONG) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            object_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetFloatElem(float value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetOfloatElem(Float value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetDoubleElem(double value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetOdoubleElem(Double value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetStringElem(String value) throws PException {
        this.value = value;
    }

    public void pieSetStringIndexField(String fn, String value) throws PException {
        if (typeCode != PType.TYPECODE_STRING) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            object_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetDateElem(Date value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetDateIndexField(String fn, Date value) throws PException {
        if (typeCode != PType.TYPECODE_DATE) {
            throw new PException("Wrong index type.");
        }
        if (fn.equals("index")) {
            object_index = value;
            return;
        }
        throw new PException("Unknown index name.");
    }

    public void pieSetCharArrayElem(char[] value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetByteArrayElem(byte[] value) throws PException {
            throw new PException("Wrong type!!");
    }

    public void pieSetSerializedElem(Serializable value) throws PException {
            throw new PException("Wrong type!!");
    }

	public void pieSetBigDecimalElem(BigDecimal value) throws PException {
			throw new PException("Wrong type!!");
	}

	public void pieSetBigIntegerElem(BigInteger value) throws PException {
			throw new PException("Wrong type!!");
	}

    public void pieSetRefElem(PName value) throws PException {
        throw new PException("Wrong type!!");
    }
}
