/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.multitable;

import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.jorm.runtime.PolyIdPNG;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdBinder;
import org.objectweb.jorm.facility.naming.polymorphid.PolymorphIdPNG;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Map;
import java.util.HashMap;

/**
 * Created by IntelliJ IDEA.
 * User: chassase
 * Date: 1 mai 2003
 * Time: 08:18:53
 * To change this template use Options | File Templates.
 */
public class TestClassRefMultitable extends TestRuntimeHelper {

    private final static String LOGGER_NAME
        = "test.org.objectweb.jorm.multitable.classref";

    private final static String ClassRef_SN_MT
        = "org.objectweb.jorm.pobject.multitable.ClassRef_SN_MT";
    private final static String ClassRef_CN_MT
        = "org.objectweb.jorm.pobject.multitable.ClassRef_CN_MT";

    private final static String StringReferencedClass_SN_MT
        = "org.objectweb.jorm.pobject.multitable.StringReferencedClass_SN_MT";
    private final static String PolyIdReferencedClass_CN_MT
        = "org.objectweb.jorm.pobject.multitable.PolyIdReferencedClass_CN_MT";

    Map cn2binder;

    public TestClassRefMultitable(String s) throws Exception {
        super(s);
        cn2binder = new HashMap(1);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        PBinder binder = (PBinder) cn2binder.get(className);
        if (binder != null) {
            return binder;
        }
        try {
            if (ClassRef_SN_MT.equals(className)) {
                binder = new BasidBinder();
                binder.setNullPName(new Long(-1));
                ((BasidBinder) binder).setCodingType(PNameCoder.CTLONG);
            } else if (ClassRef_CN_MT.equals(className)
                || PolyIdReferencedClass_CN_MT.equals(className)) {
                binder = new PolymorphIdBinder();
                binder.setNullPName(new PolyIdPNG());
            } else if (StringReferencedClass_SN_MT.equals(className)) {
                binder = new BasidBinder();
                binder.setNullPName(null);
                ((BasidBinder) binder).setCodingType(PNameCoder.CTSTRING);
            }
        } catch (PException e) {
            Exception current = e;
            while (current instanceof PException
                    && ((PException) current).getNestedException() != null) {
                current = ((PException) current).getNestedException();
            }
            throw current;
        }
        if (binder == null) {
            throw new PException("Unmanaged class " + className);
        }
        cn2binder.put(className, binder);
        return binder;
    }


    private void classRefTest(PBinding crPb, PBinding rc1Pb, PBinding rc2Pb) {

        //Make persistent the referencedClass
        ReferencedClass_MT rc1Accw = new ReferencedClass_MT();
        rc1Accw.f1 = 10;
        ReferencedClass_MT rc1Accr = new ReferencedClass_MT();
        logger.log(BasicLevel.DEBUG, "create and read referenced class 1");
        writeRead(rc1Pb, rc1Accw, rc1Accr);
        assertEquals("bad referenced class (1) values", rc1Accw, rc1Accr);

        ReferencedClass_MT rc2Accw = new ReferencedClass_MT();
        rc2Accw.f1 = 20;
        ReferencedClass_MT rc2Accr = new ReferencedClass_MT();
        logger.log(BasicLevel.DEBUG, "create and read referenced class 2");
        writeRead(rc2Pb, rc2Accw, rc2Accr);
        assertEquals("bad referenced class (2) values", rc2Accw, rc2Accr);

        ClassRef_MT crAccw = new ClassRef_MT(rc1Pb.getPName(), rc2Pb.getPName());
        ClassRef_MT crAccr = new ClassRef_MT();
        logger.log(BasicLevel.DEBUG, "create and read classref");
        writeRead(crPb, crAccw, crAccr);
        assertEquals("bad class reference values", crAccw, crAccr);

        ClassRef_MT crAccw2 = new ClassRef_MT(rc2Pb.getPName(), rc1Pb.getPName());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(crPb, crAccw2, crAccr);
        assertEquals("bad class reference values", crAccw2, crAccr);

        logger.log(BasicLevel.DEBUG, "remove referenced class 1");
        unexport(rc1Pb, rc1Accw);
        logger.log(BasicLevel.DEBUG, "remove referenced class 2");
        unexport(rc2Pb, rc2Accw);
        logger.log(BasicLevel.DEBUG, "remove class reference");
        unexport(crPb, crAccw);
    }

    public void testClassRefSingle() {
        changeLogger(LOGGER_NAME + ".single");
        logger.log(BasicLevel.DEBUG, "testClassRefSingle begin");
        classRefTest(
            export(ClassRef_SN_MT, new Long(45)),
            export(StringReferencedClass_SN_MT, "azerty"),
            export(StringReferencedClass_SN_MT, "qsdfgh")
            );
        logger.log(BasicLevel.DEBUG, "testClassRefSingle end");
    }

    public void testClassRefComposite() {
        changeLogger(LOGGER_NAME + ".composite");
        logger.log(BasicLevel.DEBUG, "testPrimitiveComposite begin");
        classRefTest(
            export(ClassRef_CN_MT, new PolyIdPNG(12, 34)),
            export(PolyIdReferencedClass_CN_MT, new PolyIdPNG(78, 90)),
            export(PolyIdReferencedClass_CN_MT, new PolyIdPNG(13, 24))
            );
        logger.log(BasicLevel.DEBUG, "testPrimitiveComposite end");
    }
}
