/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.namedef;

import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.ExceptionHelper;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Date;
import java.math.BigDecimal;
import java.math.BigInteger;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestNameDefSingle extends TestRuntimeHelper {

    private final static String LOGGER_NAME
            = "test.org.objectweb.jorm.singlename";

    public TestNameDefSingle(String name) throws Exception {
        super(name);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        int idx = className.lastIndexOf('.');
        String cn = className.substring(idx + 1);
        PType type = null;
        int ct = -1;
        Object nullvalue = null;
        if (cn.equals("ASingleByteId")) {
            type = PTypeSpace.BYTE;
            ct = PNameCoder.CTBYTE;
            nullvalue = new Byte((byte) -1);
        } else if (cn.equals("ASingleObyteId")) {
            type = PTypeSpace.OBJBYTE;
            ct = PNameCoder.CTOBYTE;
        } else if (cn.equals("ASingleCharId")) {
            type = PTypeSpace.CHAR;
            ct = PNameCoder.CTCHAR;
            nullvalue = new Character('0');
        } else if (cn.equals("ASingleOcharId")) {
            type = PTypeSpace.OBJCHAR;
            ct = PNameCoder.CTOCHAR;
        } else if (cn.equals("ASingleShortId")) {
            type = PTypeSpace.SHORT;
            ct = PNameCoder.CTSHORT;
            nullvalue = new Short((short) -1);
        } else if (cn.equals("ASingleOshortId")) {
            type = PTypeSpace.OBJSHORT;
            ct = PNameCoder.CTOSHORT;
        } else if (cn.equals("ASingleIntId")) {
            type = PTypeSpace.INT;
            ct = PNameCoder.CTINT;
            nullvalue = new Integer(-1);
        } else if (cn.equals("ASingleOintId")) {
            type = PTypeSpace.OBJINT;
            ct = PNameCoder.CTOINT;
        } else if (cn.equals("ASingleLongId")) {
            type = PTypeSpace.LONG;
            ct = PNameCoder.CTLONG;
            nullvalue = new Long(-1);
        } else if (cn.equals("ASingleOlongId")) {
            type = PTypeSpace.OBJLONG;
            ct = PNameCoder.CTOLONG;
        } else if (cn.equals("ASingleStringId")) {
            type = PTypeSpace.STRING;
            ct = PNameCoder.CTSTRING;
		} else if (cn.equals("ASingleDateId")) {
			type = PTypeSpace.DATE;
			ct = PNameCoder.CTDATE;
		} else if (cn.equals("ASingleBigDecimalId")) {
			type = PTypeSpace.BIGDECIMAL;
			ct = PNameCoder.CTBIGDECIMAL;
		} else if (cn.equals("ASingleBigIntegerId")) {
			type = PTypeSpace.BIGINTEGER;
			ct = PNameCoder.CTBIGINTEGER;
        } else if (cn.equals("ASingleBytearrayId")) {
            type = PTypeSpace.BYTEARRAY;
            ct = PNameCoder.CTBYTEARRAY;
        } else if (cn.equals("ASingleChararrayId")) {
            type = PTypeSpace.CHARARRAY;
            ct = PNameCoder.CTCHARARRAY;
        } else {
            throw new Exception(
                    "Unable to find the type of the binder for the class: "
                    + className);
        }
        logger.log(BasicLevel.DEBUG, "Allocate a BasidBinder to manage "
                + type.getJormName());
        PBinder binder = new BasidBinder();
        binder.setNullPName(nullvalue);
        ((BasidBinder) binder).setCodingType(ct);
        return binder;
    }

    private static int f1 = 0;

    private static synchronized int getF1() {
        return f1++;
    }


    private final static String CNBYTE = "org.objectweb.jorm.pobject.singlename.ASingleByteId";

    public void testByte() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        logger.log(BasicLevel.DEBUG, "testByte begin");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        byte nv = 56;
        PBinding pb = export(CNBYTE, new Byte(nv));
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeByte());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testByte end");
    }

    private final static String CNOBYTE = "org.objectweb.jorm.pobject.singlename.ASingleObyteId";

    public void testObyte() throws Exception {
        changeLogger(LOGGER_NAME + ".byte");
        logger.log(BasicLevel.DEBUG, "testObyte begin");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        Byte nv = new Byte((byte) 56);
        PBinding pb = export(CNOBYTE, nv);
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeObyte());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testObyte end");
    }

    private final static String CNSHORT = "org.objectweb.jorm.pobject.singlename.ASingleShortId";

    public void testShort() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        short nv = 56;
        PBinding pb = export(CNSHORT, new Short(nv));
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeShort());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testShort end");
    }

    private final static String CNOSHORT = "org.objectweb.jorm.pobject.singlename.ASingleOshortId";

    public void testOshort() throws Exception {
        changeLogger(LOGGER_NAME + ".short");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        Short nv = new Short((short) 56);
        PBinding pb = export(CNOSHORT, nv);
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeOshort());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testOshort end");
    }

    private final static String CNINT = "org.objectweb.jorm.pobject.singlename.ASingleIntId";

    public void testInt() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        int nv = 56;
        PBinding pb = export(CNINT, new Integer(nv));
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeInt());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testInt end");
    }

    private final static String CNOINT = "org.objectweb.jorm.pobject.singlename.ASingleOintId";

    public void testOint() throws Exception {
        changeLogger(LOGGER_NAME + ".int");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        Integer nv = new Integer(56);
        PBinding pb = export(CNOINT, nv);
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeOint());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testOint end");
    }

    private final static String CNLONG = "org.objectweb.jorm.pobject.singlename.ASingleLongId";

    public void testLong() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        long nv = 56;
        PBinding pb = export(CNLONG, new Long(nv));
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeLong());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testLong end");
    }

    private final static String CNOLONG = "org.objectweb.jorm.pobject.singlename.ASingleOlongId";

    public void testOlong() throws Exception {
        changeLogger(LOGGER_NAME + ".long");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        Long nv = new Long(56);
        PBinding pb = export(CNOLONG, nv);
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeOlong());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testOlong end");
    }

    private final static String CNCHAR = "org.objectweb.jorm.pobject.singlename.ASingleCharId";

    public void testChar() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        char nv = 'e';
        PBinding pb = export(CNCHAR, new Character(nv));
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeChar());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testChar end");
    }

    private final static String CNOCHAR = "org.objectweb.jorm.pobject.singlename.ASingleOcharId";

    public void testOchar() throws Exception {
        changeLogger(LOGGER_NAME + ".char");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        Character nv = new Character('e');
        PBinding pb = export(CNOCHAR, nv);
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeOchar());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testOchar end");
    }

    private final static String CNSTRING = "org.objectweb.jorm.pobject.singlename.ASingleStringId";

    public void testString() throws Exception {
        changeLogger(LOGGER_NAME + ".string");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        String nv = "azerty";
        PBinding pb = export(CNSTRING, nv);
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEquals("Bad pname encoded value", nv, pb.getPName().encodeString());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testString end");
    }

	private final static String CNDATE = "org.objectweb.jorm.pobject.singlename.ASingleDateId";

	public void testDate() throws Exception {
		changeLogger(LOGGER_NAME + ".date");
		int f = getF1();
		NameDefAccessor acc2 = new NameDefAccessor();
		Date nv = getTime();
		PBinding pb = export(CNDATE, nv);
		NameDefAccessor acc1 = new NameDefAccessor(f,
				pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
		assertEquals("Bad pname encoded value", nv, pb.getPName().encodeDate());
		logger.log(BasicLevel.DEBUG, "create and read");
		writeRead(pb, acc1, acc2);
		assertEquals("bad f1 value", f, acc2.f1);
		assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
		NameDefAccessor acc3 = new NameDefAccessor(-f,
				pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
		logger.log(BasicLevel.DEBUG, "update and read");
		writeRead(pb, acc3, acc2);
		assertEquals("bad f1 value", -f, acc2.f1);
		assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
		logger.log(BasicLevel.DEBUG, "remove");
		unexport(pb, acc1);
		logger.log(BasicLevel.DEBUG, "testDate end");
	}

	private final static String CNBIGDECIMAL = "org.objectweb.jorm.pobject.singlename.ASingleBigDecimalId";

	public void testBigDecimal() throws Exception {
		changeLogger(LOGGER_NAME + ".bigdecimal");
		int f = getF1();
		NameDefAccessor acc2 = new NameDefAccessor();
		BigDecimal nv = new BigDecimal("123456789.45");
		PBinding pb = export(CNBIGDECIMAL, nv);
		NameDefAccessor acc1 = new NameDefAccessor(f,
				pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
		assertEquals("Bad pname encoded value", nv, pb.getPName().encodeBigDecimal());
		logger.log(BasicLevel.DEBUG, "create and read");
		writeRead(pb, acc1, acc2);
		assertEquals("bad f1 value", f, acc2.f1);
		assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
		NameDefAccessor acc3 = new NameDefAccessor(-f,
				pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
		logger.log(BasicLevel.DEBUG, "update and read");
		writeRead(pb, acc3, acc2);
		assertEquals("bad f1 value", -f, acc2.f1);
		assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
		logger.log(BasicLevel.DEBUG, "remove");
		unexport(pb, acc1);
		logger.log(BasicLevel.DEBUG, "testDate end");
	}

	private final static String CNBIGINTEGER = "org.objectweb.jorm.pobject.singlename.ASingleBigIntegerId";

	public void testBigInteger() throws Exception {
		changeLogger(LOGGER_NAME + ".biginteger");
		int f = getF1();
		NameDefAccessor acc2 = new NameDefAccessor();
		BigInteger nv = new BigInteger("123456789");
		PBinding pb = export(CNBIGINTEGER, nv);
		NameDefAccessor acc1 = new NameDefAccessor(f,
				pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
		assertEquals("Bad pname encoded value", nv, pb.getPName().encodeBigInteger());
		logger.log(BasicLevel.DEBUG, "create and read");
		writeRead(pb, acc1, acc2);
		assertEquals("bad f1 value", f, acc2.f1);
		assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc1.myself2, acc2.myself2);
		NameDefAccessor acc3 = new NameDefAccessor(-f,
				pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
		logger.log(BasicLevel.DEBUG, "update and read");
		writeRead(pb, acc3, acc2);
		assertEquals("bad f1 value", -f, acc2.f1);
		assertEquals("Bad myself1 value", acc3.myself1, acc2.myself1);
		assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
		logger.log(BasicLevel.DEBUG, "remove");
		unexport(pb, acc1);
		logger.log(BasicLevel.DEBUG, "testDate end");
	}

    private final static String CNBYTEARRAY = "org.objectweb.jorm.pobject.singlename.ASingleBytearrayId";

    public void testBytearray() throws Exception {
        changeLogger(LOGGER_NAME + ".bytearray");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        byte[] nv = "azerty".getBytes();
        PBinding pb = export(CNBYTEARRAY, nv);
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEqual("Bad pname encoded value", nv, pb.getPName().encode());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEqual("Bad myself2 value",
                acc1.myself2.encode(), acc2.myself2.encode());
        assertEquals("Bad linked pnc value",
                pb.getPClassMapping().getPBinder(), acc2.myself2.getPNameManager());
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEqual("Bad myself1 value",
                acc3.myself1.encode(), acc2.myself1.encode());
        assertEquals("Bad linked pnc value",
                pb.getPClassMapping().getPBinder(), acc2.myself1.getPNameManager());
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testBytearray end");
    }

    private final static String CNCHARARRAY = "org.objectweb.jorm.pobject.singlename.ASingleChararrayId";

    public void testChararray() throws Exception {
        changeLogger(LOGGER_NAME + ".chararray");
        int f = getF1();
        NameDefAccessor acc2 = new NameDefAccessor();
        char[] nv = new char[]{'a', 'z', 'e', 'r', 't', 'y'};
        PBinding pb = export(CNCHARARRAY, nv);
        NameDefAccessor acc1 = new NameDefAccessor(f,
                pb.getPClassMapping().getPBinder().getNull(), pb.getPName());
        assertEqual("Bad pname encoded value", nv, pb.getPName().encodeCharArray());
        assertEquals("Bad pname encoded value",
                pb.getPClassMapping().getPBinder(), pb.getPName().getPNameManager());
        logger.log(BasicLevel.DEBUG, "create and read");
        writeRead(pb, acc1, acc2);
        assertEquals("bad f1 value", f, acc2.f1);
        assertEquals("Bad myself1 value", acc1.myself1, acc2.myself1);
        assertEqual("Bad myself2 encoded value",
                acc1.myself2.encodeCharArray(), acc2.myself2.encodeCharArray());
        assertEquals("Bad linked pnc value",
                pb.getPClassMapping().getPBinder(), acc2.myself2.getPNameManager());
        NameDefAccessor acc3 = new NameDefAccessor(-f,
                pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
        logger.log(BasicLevel.DEBUG, "update and read");
        writeRead(pb, acc3, acc2);
        assertEquals("bad f1 value", -f, acc2.f1);
        assertEqual("Bad myself1 value",
                acc3.myself1.encodeCharArray(), acc2.myself1.encodeCharArray());
        assertEquals("Bad linked pnc value",
                pb.getPClassMapping().getPBinder(), acc2.myself1.getPNameManager());
        assertEquals("Bad myself2 value", acc3.myself2, acc2.myself2);
        logger.log(BasicLevel.DEBUG, "remove");
        unexport(pb, acc1);
        logger.log(BasicLevel.DEBUG, "testCharray end");
    }
}
