/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2003 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */

package org.objectweb.jorm.runtime.pnameiterator;

import org.objectweb.jorm.runtime.TestRuntimeHelper;
import org.objectweb.jorm.runtime.namedef.NameDefAccessor;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Iterator;

/**
 *
 * @author	P. Dechamboux
 */
public class TestSingleNameIterator extends TestRuntimeHelper {

    private final static String LOGGER_NAME
            = "test.org.objectweb.jorm.singlename";

    public TestSingleNameIterator(String name) throws Exception {
        super(name);
    }

    protected String getLoggerName() {
        return LOGGER_NAME;
    }

    protected PBinder getBinder(String className) throws Exception {
        int idx = className.lastIndexOf('.');
        String cn = className.substring(idx + 1);
        PType type = null;
        int ct = -1;
        Object nullvalue = null;
        if (cn.equals("ASingleIntId")) {
            type = PTypeSpace.INT;
            ct = PNameCoder.CTINT;
            nullvalue = new Integer(-1);
        } else {
            throw new Exception(
                    "Unable to find the type of the binder for the class: "
                    + className);
        }
        logger.log(BasicLevel.DEBUG, "Allocate a BasidBinder to manage "
                + type.getJormName());
        PBinder binder = new BasidBinder();
        binder.setNullPName(nullvalue);
        ((BasidBinder) binder).setCodingType(ct);
        return binder;
    }

    private final static String CNINT = "org.objectweb.jorm.pobject.singlename.ASingleIntId";

	public void trueTester(int NBOBJ, boolean prefetch) throws Exception {
		PClassMapping pcm = getMapping(CNINT);
		Object conn = mapper.getConnection();
		for (int i = 0; i < NBOBJ; i++) {
			PBinding pb = export(CNINT, new Integer(i));
			NameDefAccessor accwrite = new NameDefAccessor(i, pb.getPName(), pb.getPClassMapping().getPBinder().getNull());
			pb.write(conn, accwrite);
		}
		Iterator it = pcm.getPNameIterator(conn, false, prefetch, conn);
		int nbiter = 0;
		PBinding[] pb = new PBinding[NBOBJ];
		NameDefAccessor accread = new NameDefAccessor();
		while (it.hasNext()) {
			PName pn = (PName) it.next();
			pb[nbiter] = bind(CNINT, pn);
			pb[nbiter].read(conn, accread, conn);
			assertEquals("Bad object pname and F1 should be equal", accread.f1, pn.encodeInt());
			nbiter++;
		}
		assertEquals("Wrong number of objects iterated", NBOBJ, nbiter);
		mapper.getPrefetchCache().invalidatePrefetchBuffer(conn);
		mapper.closeConnection(conn);
		mapper.getPMapCluster(CNINT).deleteData();
    }

	public void testIntNameIteratorNoPrefetchNoElem() throws Exception {
		int NBOBJ = 0;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, false);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorNoPrefetchOneElem() throws Exception {
		int NBOBJ = 1;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, false);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorNoPrefetch10Elem() throws Exception {
		int NBOBJ = 10;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, false);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorNoPrefetch100Elem() throws Exception {
		int NBOBJ = 100;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, false);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorNoPrefetch1000Elem() throws Exception {
		int NBOBJ = 1000;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, false);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorPrefetchNoElem() throws Exception {
		int NBOBJ = 0;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, true);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorPrefetchOneElem() throws Exception {
		int NBOBJ = 1;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, true);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorPrefetch10Elem() throws Exception {
		int NBOBJ = 10;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, true);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorPrefetch100Elem() throws Exception {
		int NBOBJ = 100;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, true);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}

	public void testIntNameIteratorPrefetch1000Elem() throws Exception {
		int NBOBJ = 1000;
		changeLogger(LOGGER_NAME + ".int." + NBOBJ);
		trueTester(NBOBJ, true);
		logger.log(BasicLevel.DEBUG, "testInt end");
	}
}
